<?php
/**
 * Plugin Name: First Party Analytics
 * Description: A plugin for first-party analytics.
 * Version:     1.0.80
 * Text Domain: first-party-analytics
 * Requires PHP: 7.2
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics;

use Exception;
use WP_Filesystem_Direct;

/**
 * Main First_Party_Analytics class.
 */
final class First_Party_Analytics {

    /**
     * File System Direct instance.
     *
     * @var WP_Filesystem_Direct
     */
    public static $fs;

    /**
     * AJAX class.
     *
     * @var AJAX\AJAX
     */
    public static $ajax;

    /**
     * Options.
     *
     * @var Options
     */
    public static $options;

    /**
     * Raw analytics data handler (write/read).
     *
     * @var Raw_Analytics_Data
     */
    public static $raw_analytics_data_handler;

    /**
     * Plugin slug.
     *
     * @var string
     */
    public static $slug;

    /**
     * Plugin version.
     *
     * @var string
     */
    public static $version;

    /**
     * Plugin name.
     *
     * @var string
     */
    public static $name;

    /**
     * The minimum amount of requests to use for percentiles.
     *
     * @var int
     */
    const PERCENTILES_THRESHOLD = 100;

    /**
     * Whether the premium version of the plugin is active.
     *
     * @var bool
     */
    public static $premium_enabled = false;

    /**
     * Constructor.
     *
     * @throws Exception Exception.
     */
    public function __construct() {
        $this->define_constants();
        $this->import_plugin_files();
        $this->load_wp_fs_class();

        self::$fs = new WP_Filesystem_Direct( null );

        // Options.
        self::$options = new Options();

        // Activation.
        register_activation_hook( FIRST_PARTY_ANALYTICS_FILE, [ $this, 'activation_handler' ] );

        add_action(
            'plugins_loaded',
            function() {
                new \WP_Plugins_Core\WP_Plugins_Core( $this );

                // Load translations.
                $this->load_plugin_textdomain();

                if ( class_exists( 'First_Party_Analytics_Premium\First_Party_Analytics_Premium' ) ) {
                    self::$premium_enabled = true;
                }

                // Handle version update.
                $this->handle_version_update();

                if ( self::$version === self::$options->get( 'last-handled-version-update' ) ) {
                    // MU Plugin (AJAX Helper (to deload all plugins except for the bare minimum AJAX requests)).
                    new Mu_Plugin();

                    // Adds pluggable functions.
                    if ( ! function_exists( 'wp_rand' ) ) {
                        require_once ABSPATH . WPINC . '/pluggable.php';
                    }

                    // Adds plugin settings links to plugins admin screen.
                    add_filter( 'plugin_action_links_' . FIRST_PARTY_ANALYTICS_BASENAME, [ $this, 'plugin_action_links' ] );

                    // Assets.
                    Assets\Assets::init();

                    // Post-activation.
                    add_action( 'init', [ $this, 'post_activation_handler' ] );

                    // Analytics data.
                    self::$raw_analytics_data_handler = new Raw_Analytics_Data();

                    // Raw analytics data parsing (reads and adds anonymised data to the DBs).
                    new Raw_Analytics_Data_Parsing();

                    // Deletes visitors for the previous day.
                    new Remove_Visitors_For_The_Previous_Day();

                    // Post publish box.
                    new Representation\Publish_Box;

                    // Search tracking.
                    new Tracking\Helpers\Search;

                    // MaxMind Database.
                    new MaxMind\Database;
                    new MaxMind\Update_Interval;

                    // Product tracking data.
                    new WooCommerce\Products_Tracking_Data;

                    // Ping attribute to links.
                    new Ping_Attribute_To_Links();

                    // Data tracking.
                    new WooCommerce\WC_Order;

                    // Data sharing.
                    new Data_Sharing\Init;

                    // Fields mapping.
                    new Fields_Mapping();

                    add_action(
                        'init',
                        function () {
                            // Menu.
                            new Admin_Menu();

                            // AJAX Handler.
                            self::$ajax = new AJAX\AJAX;
                        }
                    );

                    add_action(
                        'wp',
                        function () {
                            // NoScript tracking.
                            new Tracking\Server_Side_NoScript;

                            // Server side analytics tracking.
                            new Tracking\Server_Side;
                        }
                    );
                }
            }
        );
    }

    /**
     * Defines constants.
     */
    private function define_constants() {
        require_once __DIR__ . '/data/constants.php';

        /**
         * Plugin name.
         */
        self::$name = get_file_data( FIRST_PARTY_ANALYTICS_FILE, [ 'Plugin Name' ] )[0];

        /**
         * Plugin slug.
         */
        $dir_parts  = explode( '/', FIRST_PARTY_ANALYTICS_DIR );
        self::$slug = $dir_parts[ array_search( 'plugins', $dir_parts, true ) + 1 ];

        /**
         * Plugin version.
         */
        self::$version = FIRST_PARTY_ANALYTICS_VERSION;
    }

    /**
     * Imports plugin files.
     */
    private function import_plugin_files() {
        $src_files = [
            'assets/screens/class-assets-main-screen',
            'assets/screens/class-assets-settings-screen',
            'assets/screens/class-assets-post-screen',
            'assets/screens/class-assets-plugins-screen',
            'assets/screens/class-assets-dashboard',
            'assets/screens/wc-taxonomies/class-assets',
            'assets/screens/wc-taxonomies/class-assets-wc-edit-attributes',
            'assets/screens/wc-taxonomies/class-assets-wc-edit-attribute-terms',
            'assets/screens/wc-taxonomies/class-assets-wc-edit-tags',
            'assets/screens/wc-taxonomies/class-assets-wc-edit-categories',
            'assets/public/class-public-assets',
            'assets/public/class-public-admin-bar-assets',
            'assets/class-assets',
            'class-admin-menu',
            'ajax/class-ajax',
            'ajax/class-admin-ajax',
            'ajax/class-public-ajax',
            'ajax/class-non-ajax-requests',
            'class-options',
            'class-settings-export',
            'class-raw-analytics-data',
            'class-raw-analytics-data-parsing',
            'tracking/class-server-side',
            'tracking/class-server-side-noscript',
            'tracking/class-client-side',
            'tracking/class-client-side-post-visit',
            'tracking/class-link-visits',
            'tracking/class-searches',
            'tracking-helpers/class-search',
            'class-tracking-cookies',
            'class-utils',
            'class-third-party-data-extractors',
            'class-sessions',
            'class-sources',
            'class-web-vitals',
            'class-cache',
            'class-fields-mapping',
            'class-average-math',
            'class-databases',
            'class-geolocation',
            'class-countries',
            'class-unique-visits',
            'class-debug',
            'class-data-removal-helpers',
            'class-requests-exclusion',
            'class-object-cache',
            'class-no-tracking-cookie',
            'class-raw-requests-storage',
            'class-ping-attribute-to-links',
            'class-stop-words',
            'class-tabs-data',
            'class-migrations',
            'class-mu-plugin',
            'class-revision-specific-data',
            'class-get-web-vitals-debug-data',
            'maxmind-database/class-maxmind-database',
            'maxmind-database/class-update-interval',
            'database/class-database',
            'database/class-time-of-day',
            'database/class-visitors',
            'database/class-visitor-details',
            'database/class-page-visits',
            'database/class-visitors',
            'database/class-page-visits-on-finish',
            'database/class-search-words',
            'database/class-link-visits',
            'database/class-sessions',
            'database/class-special-url-params',
            'class-remove-visitors-for-the-previous-day',
            'representation/tables/class-table',
            'representation/tables/class-searches',
            'representation/tables/class-countries',
            'representation/tables/class-user-agents',
            'representation/tables/class-referers',
            'representation/tables/class-google-featured-snippet-highlighted-text',
            'representation/tables/class-pages',
            'representation/tables/class-visited-links',
            'representation/tables/class-sources',
            'representation/tables/class-visits-by-sources',
            'representation/tables/class-pages-performance',
            'representation/class-publish-box',
            'representation/graphs/class-general-views',
            'representation/graphs/class-hour-views',
            'representation/graphs/class-weekday-views',
            'representation/graphs/class-average-view-time',
            'representation/graphs/class-average-load-time',
            'representation/graphs/class-bounce-rate',
            'representation/graphs/class-average-page-views-per-session',
            'representation/graphs/class-average-session-duration',
            'representation/graphs/class-returning-visitors',
            'representation/graphs/class-number-of-sessions',
            'debug/class-db-population',
            'woocommerce/class-products-tracking-data',
            'woocommerce/class-pages-tracking-data',
            'woocommerce/class-wc-taxonomies-data',
            'woocommerce/class-wc-order',
            'data-sharing/class-init',
            'data-sharing/class-data-sharing',
            'data-sharing/class-recommendations-fetch',
            'data-sharing/class-token',
        ];
        foreach ( $src_files as $file ) {
            require_once FIRST_PARTY_ANALYTICS_DIR . 'src/' . $file . '.php';
        }

        $files = [
            'vendor/autoload',
            'vendor/tmmtech/wp-plugins-core/wp-plugins-core',
            'vendor/woocommerce/action-scheduler/action-scheduler',
            'vendor/autoload',
        ];
        foreach ( $files as $file ) {
            require_once FIRST_PARTY_ANALYTICS_DIR . $file . '.php';
        }
    }

    /**
     * Loads textdomain.
     */
    private function load_plugin_textdomain() {
        load_plugin_textdomain(
            'first-party-analytics',
            false,
            dirname( FIRST_PARTY_ANALYTICS_BASENAME ) . '/languages'
        );
    }

    /**
     * Loads WP_Filesystem_Direct class.
     */
    private function load_wp_fs_class() {
        if ( ! function_exists( 'WP_Filesystem' ) ) {
            require_once ABSPATH . 'wp-admin/includes/file.php';
        }

        if ( ! class_exists( 'WP_Filesystem_Base' ) ) {
            WP_Filesystem();
        }
    }

    /**
     * Show settings link on the plugin screen.
     *
     * @param mixed $links Plugin Action links.
     *
     * @return array
     */
    public function plugin_action_links( $links ) {
        $action_links = array(
            'settings' => '<a href="' . admin_url( 'admin.php?page=first-party-analytics-settings' ) . '" aria-label="' .
                esc_attr__( 'View First Party Analytics settings', 'first-party-analytics' ) . '">' .
                esc_html__( 'Settings', 'first-party-analytics' ) . '</a>',
        );

        return array_merge( $action_links, $links );
    }

    /**
     * Handles version update.
     */
    private function handle_version_update() {
        $last_handled_version_update = self::$options->get( 'last-handled-version-update' );
        if ( self::$version !== $last_handled_version_update ) {
            self::$options->set( 'last-handled-version-update', self::$version );
            new Migrations( $last_handled_version_update );
            DB::db_delta();
        }
    }

    /**
     * Activation handled.
     *
     * Just marks that the plugin was activated but the activation wasn't handled yet.
     */
    public function activation_handler() {
        self::$options->set( 'activation-handled', false );
    }

    /**
     * Site post-activation handler.
     *
     * Handler the plugin activation after it was marked in the above method.
     *
     * Sends the request to the backend.
     * And if the request fails, deactivates the plugin and sends the admin notice.
     */
    public function post_activation_handler() {
        if (
            is_admin() &&
            current_user_can( 'activate_plugins' ) &&
            isset( $_GET['activate'] ) && 
            ! self::$options->get( 'activation-handled' )
        ) {
            self::$options->set( 'activation-handled', true );

            add_action(
                'admin_enqueue_scripts',
                function() {
                    wp_localize_script(
                        'first-party-analytics-admin-plugins-screen-script',
                        'firstPartyAnalyticsRecommendationsPopup',
                        [
                            'show' => ! self::$options->get( 'setting-enable-data-sharing-and-recommendations' ),
                        ]
                    );
                }
            );
        }
    }
}
new First_Party_Analytics();
