<?php
/**
 * AJAX handlers.
 *
 * Each AJAX method here should have nonce check.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\AJAX;

use Exception;
use First_Party_Analytics\Cache;
use First_Party_Analytics\Data_Removal_Helpers;
use First_Party_Analytics\Databases\Page_Visits_Finish;
use First_Party_Analytics\DB;
use First_Party_Analytics\First_Party_Analytics;
use First_Party_Analytics\Representation\Tables\Google_Featured_Snippet_Highlighted_Text as Google_Featured_Snippet_Highlighted_Text_Table;
use First_Party_Analytics\Representation\Tables\Pages_Performance;
use First_Party_Analytics\Representation\Tables\Referers as Referers_Table;
use First_Party_Analytics\Representation\Tables\User_Agents as User_Agents_Table;
use First_Party_Analytics\Representation\Tables\Countries as Countries_Table;
use First_Party_Analytics\Representation\Tables\Searches as Searches_Table;
use First_Party_Analytics\Representation\Tables\Visited_Links as Visited_Links_Table;
use First_Party_Analytics\Representation\Tables\Most_Visited_Pages as Most_Visited_Pages_Table;
use First_Party_Analytics\Representation\Tables\Sources as Sources;
use First_Party_Analytics\Representation\Tables\Visits_By_Sources as Sources_Visits_Table;
use First_Party_Analytics\Debug;
use First_Party_Analytics\Tabs_Data;
use First_Party_Analytics\Web_Vitals;
use First_Party_Analytics\WooCommerce\WC_Order;
use First_Party_Analytics_Premium\First_Party_Analytics_Premium;
use IPLib\Factory;
use WP_Plugins_Core\Sanitize;

/**
 * Manages Admin AJAX.
 */
final class Admin {

    /**
     * Constructor.
     */
    public function __construct() {
        $this->add_listeners();
    }

    /**
     * Adds AJAX listeners.
     */
    private function add_listeners() {
        $ajax_events = [
            'get_most_viewed_pages_table',
            'get_searches_table',
            'get_visited_links_table',
            'get_countries_table',
            'get_user_agents_table',
            'get_referers_table',
            'get_google_featured_snippet_highlighted_text_table',
            'save',
            'get_debug_data',
            'import_settings',
            'reset_settings',
            'reset_sources',
            'get_sources_visits_table',
            'get_sources_table',
            'get_pages_performance_table',
            'get_tab_data',
            'get_web_vitals',
            'delete_plugin_data',
            'enable_recommendations',
            'recommendation_check',
        ];

        foreach ( $ajax_events as $ajax_event ) {
            add_action( 'wp_ajax_first_party_analytics_' . $ajax_event, [ $this, $ajax_event ] );
        }
    }

    /**
     * Save plugin settings.
     *
     * @throws Exception Exception.
     */
    public function save() {
        /*
         * Nonce check.
         */
        check_ajax_referer( 'first-party-analytics-menu', 'nonceToken' );

        if ( array_key_exists( 'decimate', $_REQUEST ) ) {
            $decimate = (int) sanitize_text_field( wp_unslash( $_REQUEST['decimate'] ) );
            First_Party_Analytics::$options->set( 'setting-decimate', max( 1, $decimate ) );
        }

        if ( array_key_exists( 'exclusionCookies', $_REQUEST ) ) {
            $exclusion_cookies = Sanitize::sanitize_array(
                (array) json_decode( wp_unslash( $_REQUEST['exclusionCookies'] ), true ) 
            );
            First_Party_Analytics::$options->set( 'setting-exclusion-cookies', $exclusion_cookies );
        }

        if ( array_key_exists( 'exclusionIPs', $_REQUEST ) ) {
            $exclusion_ips = Sanitize::sanitize_array(
                (array) json_decode( wp_unslash( $_REQUEST['exclusionIPs'] ), true ) 
            );
            $exclusion_ips = array_filter(
                $exclusion_ips,
                function( $ip ) {
                    return Factory::parseRangeString( $ip ); // Correct IP or a range.
                }
            );
            First_Party_Analytics::$options->set( 'setting-exclusion-ips', $exclusion_ips );
        }

        if ( array_key_exists( 'maxMindLicenseKey', $_REQUEST ) ) {
            $maxmind_license_key = trim( sanitize_text_field( wp_unslash( $_REQUEST['maxMindLicenseKey'] ) ) );
            First_Party_Analytics::$options->set( 'maxmind-license-key', $maxmind_license_key );
        }

        if ( array_key_exists( 'useObjectCache', $_REQUEST ) ) {
            $use_object_cache = '1' === sanitize_text_field( wp_unslash( $_REQUEST['useObjectCache'] ) );
            First_Party_Analytics::$options->set( 'setting-use-object-cache', $use_object_cache );
        }

        if ( array_key_exists( 'enableDataSharingAndRecommendations', $_REQUEST ) ) {
            $enable_data_sharing_and_recommendations = '1' === sanitize_text_field( wp_unslash( $_REQUEST['enableDataSharingAndRecommendations'] ) );
            First_Party_Analytics::$options->set( 'setting-enable-data-sharing-and-recommendations', $enable_data_sharing_and_recommendations );
        }

        if ( array_key_exists( 'addPingAttribute', $_REQUEST ) ) {
            $add_ping_attribute = '1' === sanitize_text_field( wp_unslash( $_REQUEST['addPingAttribute'] ) );
            First_Party_Analytics::$options->set( 'setting-add-ping-attribute', $add_ping_attribute );
        }

        if ( array_key_exists( 'domainTypeToUseForAnalyticsBlockDetection', $_REQUEST ) ) {
            $domain_type_to_use_for_analytics_block_detection = sanitize_text_field( wp_unslash( $_REQUEST['domainTypeToUseForAnalyticsBlockDetection'] ) );
            First_Party_Analytics::$options->set( 'setting-analytics-block-detection-domain-type-to-use', $domain_type_to_use_for_analytics_block_detection );
        }

        if ( array_key_exists( 'analyticsBlockDetectionOwnDomain', $_REQUEST ) ) {
            $analytics_block_detection_own_domain = sanitize_text_field( wp_unslash( $_REQUEST['analyticsBlockDetectionOwnDomain'] ) );
            First_Party_Analytics::$options->set( 'setting-analytics-block-detection-own-domain', $analytics_block_detection_own_domain );
        }

        if ( array_key_exists( 'analyticsBlockDetectionCustomDomain', $_REQUEST ) ) {
            $analytics_block_detection_custom_domain = sanitize_text_field( wp_unslash( $_REQUEST['analyticsBlockDetectionCustomDomain'] ) );
            First_Party_Analytics::$options->set( 'setting-analytics-block-detection-custom-domain', $analytics_block_detection_custom_domain );
        }

        if ( array_key_exists( 'domainTypeToUseForAdsBlockDetection', $_REQUEST ) ) {
            $domain_type_to_use_for_ads_block_detection = sanitize_text_field( wp_unslash( $_REQUEST['domainTypeToUseForAdsBlockDetection'] ) );
            First_Party_Analytics::$options->set( 'setting-ads-block-detection-domain-type-to-use', $domain_type_to_use_for_ads_block_detection );
        }

        if ( array_key_exists( 'adsBlockDetectionOwnDomain', $_REQUEST ) ) {
            $ads_block_detection_own_domain = sanitize_text_field( wp_unslash( $_REQUEST['adsBlockDetectionOwnDomain'] ) );
            First_Party_Analytics::$options->set( 'setting-ads-block-detection-own-domain', $ads_block_detection_own_domain );
        }

        if ( array_key_exists( 'adsBlockDetectionCustomDomain', $_REQUEST ) ) {
            $ads_block_detection_custom_domain = sanitize_text_field( wp_unslash( $_REQUEST['adsBlockDetectionCustomDomain'] ) );
            First_Party_Analytics::$options->set( 'setting-ads-block-detection-custom-domain', $ads_block_detection_custom_domain );
        }

        if ( First_Party_Analytics::$premium_enabled && method_exists( 'First_Party_Analytics_Premium\Settings_Menu', 'save' ) ) {
            \First_Party_Analytics_Premium\Settings_Menu::save();
        }

        wp_send_json_success();
    }

    /**
     * Returns debug data.
     *
     * @throws Exception Exception.
     */
    public static function get_debug_data() {
        /*
         * Nonce check.
         */
        check_ajax_referer( 'first-party-analytics-menu', 'nonceToken' );

        $debug_object = print_r( Debug::get_debug_array(), true ); 

        $html = '
            <div class="first-party-analytics-debug-data-container">
                <h3>Main debug object</h3>
                <pre class="text-align-left first-party-analytics-debug-block">' . $debug_object . '</pre>
            </div>
        ';

        wp_send_json( [ 'html' => $html ] );
    }

    /**
     * Returns viewed pages table content.
     */
    public function get_most_viewed_pages_table() {
        /*
         * Nonce check.
         */
        check_ajax_referer( 'first-party-analytics-menu', 'nonceToken' );

        if (
            array_key_exists( 'postType', $_REQUEST ) &&
            array_key_exists( 'visitType', $_REQUEST ) &&
            array_key_exists( 'date', $_REQUEST ) &&
            array_key_exists( 'page', $_REQUEST )
        ) {
            $post_type  = sanitize_text_field( wp_unslash( $_REQUEST['postType'] ) );
            $visit_type = sanitize_text_field( wp_unslash( $_REQUEST['visitType'] ) );
            $days_range = sanitize_text_field( wp_unslash( $_REQUEST['date'] ) );
            $page       = (int) sanitize_text_field( wp_unslash( $_REQUEST['page'] ) );
            wp_send_json_success(
                [
                    'content'   => Most_Visited_Pages_Table::print_table( true, $post_type, $visit_type, $days_range, $page ),
                    'postType'  => $post_type,
                    'visitType' => $visit_type,
                    'date'      => $days_range,
                    'page'      => $page,
                ]
            );
        }

        wp_send_json_error();
    }

    /**
     * Returns searches table content.
     */
    public function get_searches_table() {
        /*
         * Nonce check.
         */
        check_ajax_referer( 'first-party-analytics-menu', 'nonceToken' );

        if (
            array_key_exists( 'page', $_REQUEST ) &&
            array_key_exists( 'searchType', $_REQUEST )
        ) {
            $page        = (int) sanitize_text_field( wp_unslash( $_REQUEST['page'] ) );
            $search_type = sanitize_text_field( wp_unslash( $_REQUEST['searchType'] ) );
            wp_send_json_success(
                [
                    'content'    => Searches_Table::print_table( true, $page, $search_type ),
                    'page'       => $page,
                    'searchType' => $search_type,
                ]
            );
        }

        wp_send_json_error();
    }

    /**
     * Returns visited links table content.
     */
    public function get_visited_links_table() {
        /*
         * Nonce check.
         */
        check_ajax_referer( 'first-party-analytics-menu', 'nonceToken' );

        if (
            array_key_exists( 'page', $_REQUEST ) &&
            array_key_exists( 'date', $_REQUEST ) &&
            array_key_exists( 'dstType', $_REQUEST )
        ) {
            $page     = (int) sanitize_text_field( wp_unslash( $_REQUEST['page'] ) );
            $date     = sanitize_text_field( wp_unslash( $_REQUEST['date'] ) );
            $dst_type = sanitize_text_field( wp_unslash( $_REQUEST['dstType'] ) );
            wp_send_json_success(
                [
                    'content' => Visited_Links_Table::print_table( true, $dst_type, $date, $page ),
                    'page'    => $page,
                    'date'    => $date,
                    'dstType' => $dst_type,
                ]
            );
        }

        wp_send_json_error();
    }

    /**
     * Returns countries table content.
     */
    public function get_countries_table() {
        /*
         * Nonce check.
         */
        check_ajax_referer( 'first-party-analytics-menu', 'nonceToken' );

        if (
            array_key_exists( 'page', $_REQUEST )
        ) {
            $page = (int) sanitize_text_field( wp_unslash( $_REQUEST['page'] ) );
            wp_send_json_success(
                [
                    'content' => Countries_Table::print_table( true, $page ),
                    'page'    => $page,
                ]
            );
        }

        wp_send_json_error();
    }

    /**
     * Returns User-Agents table content.
     */
    public function get_user_agents_table() {
        /*
         * Nonce check.
         */
        check_ajax_referer( 'first-party-analytics-menu', 'nonceToken' );

        if (
            array_key_exists( 'page', $_REQUEST )
        ) {
            $page = (int) sanitize_text_field( wp_unslash( $_REQUEST['page'] ) );
            wp_send_json_success(
                [
                    'content' => User_Agents_Table::print_table( true, $page ),
                    'page'    => $page,
                ]
            );
        }

        wp_send_json_error();
    }

    /**
     * Returns referers table content.
     */
    public function get_referers_table() {
        /*
         * Nonce check.
         */
        check_ajax_referer( 'first-party-analytics-menu', 'nonceToken' );

        if (
            array_key_exists( 'page', $_REQUEST ) &&
            array_key_exists( 'displayType', $_REQUEST )
        ) {
            $page         = (int) sanitize_text_field( wp_unslash( $_REQUEST['page'] ) );
            $display_type = sanitize_text_field( wp_unslash( $_REQUEST['displayType'] ) );
            wp_send_json_success(
                [
                    'content'     => Referers_Table::print_table( true, $page, $display_type ),
                    'page'        => $page,
                    'displayType' => $display_type,
                ]
            );
        }

        wp_send_json_error();
    }

    /**
     * Returns referers table content.
     */
    public function get_google_featured_snippet_highlighted_text_table() {
        /*
         * Nonce check.
         */
        check_ajax_referer( 'first-party-analytics-menu', 'nonceToken' );

        if (
            array_key_exists( 'page', $_REQUEST )
        ) {
            $page = (int) sanitize_text_field( wp_unslash( $_REQUEST['page'] ) );
            wp_send_json_success(
                [
                    'content' => Google_Featured_Snippet_Highlighted_Text_Table::print_table( true, $page ),
                    'page'    => $page,
                ]
            );
        }

        wp_send_json_error();
    }

    /**
     * Imports settings from the file.
     *
     * Accepts: $_REQUEST['file']
     *
     * @throws Exception Exception.
     */
    public function import_settings() {
        /*
         * Nonce check.
         */
        check_ajax_referer( 'first-party-analytics-menu', 'nonceToken' );

        if ( array_key_exists( 'file', $_REQUEST ) ) {
            $settings = Sanitize::sanitize_array( (array) json_decode( wp_unslash( $_REQUEST['file'] ), true ) ); 
            foreach ( $settings as $option_name => $option_value ) {
                if ( in_array( $option_name, First_Party_Analytics::$options->settings_options, true ) ) {
                    First_Party_Analytics::$options->set( $option_name, $option_value );
                }
            }
        }

        wp_send_json_success();
    }

    /**
     * Resets all plugin settings.
     *
     * @throws Exception Exception.
     */
    public function reset_settings() {
        /*
         * Nonce check.
         */
        check_ajax_referer( 'first-party-analytics-menu', 'nonceToken' );

        // Delete all settings options.
        foreach ( First_Party_Analytics::$options->settings_options as $option ) {
            First_Party_Analytics::$options->delete( $option );
        }

        // Set them to defaults to trigger all event listeners.
        foreach ( First_Party_Analytics::$options->settings_options as $option ) {
            $option_data = First_Party_Analytics::$options->all_options[ $option ];
            if ( array_key_exists( 'default', $option_data ) ) {
                First_Party_Analytics::$options->set( $option, $option_data['default'] );
            }
        }

        wp_send_json_success();
    }

    /**
     * Resets sources
     *
     * @throws Exception Exception.
     */
    public function reset_sources() {
        /*
         * Nonce check.
         */
        check_ajax_referer( 'first-party-analytics-menu', 'nonceToken' );

        First_Party_Analytics::$options->delete( 'setting-sources' );

        wp_send_json_success();
    }

    /**
     * Updates sources.
     *
     * @throws Exception Exception.
     */
    public function get_sources_visits_table() {
        /*
         * Nonce check.
         */
        check_ajax_referer( 'first-party-analytics-menu', 'nonceToken' );

        if ( isset( $_REQUEST['sources'] ) && isset( $_REQUEST['dateRange'] ) ) {
            $sources    = Sanitize::sanitize_array( (array) json_decode( wp_unslash( $_REQUEST['sources'] ), true ) ); 
            $date_range = sanitize_text_field( wp_unslash( $_REQUEST['dateRange'] ) );

            First_Party_Analytics::$options->set( 'setting-sources', $sources );

            wp_send_json_success( [ 'html' => Sources_Visits_Table::get_html( $sources, $date_range ) ] );
        }

        wp_send_json_error();
    }

    /**
     * Get sources table.
     *
     * @throws Exception Exception.
     */
    public function get_sources_table() {
        /*
         * Nonce check.
         */
        check_ajax_referer( 'first-party-analytics-menu', 'nonceToken' );

        if ( isset( $_REQUEST['sources'] ) ) {
            // Update sources.

            $sources = Sanitize::sanitize_array( (array) json_decode( wp_unslash( $_REQUEST['sources'] ), true ) ); 

            First_Party_Analytics::$options->set( 'setting-sources', $sources );

            // Get the table.

            ob_start();
            Sources::print_referers_children( $sources );
            $table = ob_get_clean();

            // Return the table.

            wp_send_json_success( [ 'html' => $table ] );
        }

        wp_send_json_error();
    }

    /**
     * Returns "pages performance" table content.
     */
    public function get_pages_performance_table() {
        /*
         * Nonce check.
         */
        check_ajax_referer( 'first-party-analytics-menu', 'nonceToken' );

        if (
            array_key_exists( 'page', $_REQUEST ) &&
            array_key_exists( 'date', $_REQUEST )
        ) {
            $page = (int) sanitize_text_field( wp_unslash( $_REQUEST['page'] ) );
            $date = sanitize_text_field( wp_unslash( $_REQUEST['date'] ) );

            wp_send_json_success(
                [
                    'content' => Pages_Performance::print_table( true, $page, $date ),
                    'page'    => $page,
                    'date'    => $date,
                ]
            );
        }

        wp_send_json_error();
    }

    /**
     * Get tab data.
     *
     * @throws Exception Exception.
     */
    public function get_tab_data() {
        /*
         * Nonce check.
         */
        check_ajax_referer( 'first-party-analytics-menu', 'nonceToken' );

        if (
            isset( $_REQUEST['tabName'] ) &&
            isset( $_REQUEST['useCache'] )
        ) {
            $tab_name  = sanitize_text_field( wp_unslash( $_REQUEST['tabName'] ) );
            $use_cache = 'true' === sanitize_text_field( wp_unslash( $_REQUEST['useCache'] ) );

            $method = 'get_' . $tab_name . '_tab_data';

            // Cache (1 minute).

            $revision   = isset( $_REQUEST['fpaRevision'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ) : '';
            $cache_name = 'get_' . $tab_name . '_' . $revision . '_tab_data';

            $cache_data = $use_cache ? Cache::get( $cache_name ) : false;

            if ( false !== $cache_data ) {
                $tab_data = $cache_data;
            } else {
                $tab_data = method_exists( 'First_Party_Analytics\Tabs_Data', $method ) ? Tabs_Data::{$method}() : []; // Get.
                $tab_data = apply_filters(
                    "first_party_analytics_get_{$tab_name}_tab_data",
                    $tab_data
                );
                Cache::save( $cache_name, $tab_data );
            }

            // Return.

            wp_send_json_success( $tab_data );
        }

        wp_send_json_error();
    }

    /**
     * Get web vitals.
     *
     * @throws Exception Exception.
     */
    public function get_web_vitals() {
        /*
         * Nonce check.
         */
        check_ajax_referer( 'first-party-analytics-menu', 'nonceToken' );

        if (
            isset( $_REQUEST['date'] ) &&
            isset( $_REQUEST['pageUrl'] )
        ) {
            $date     = sanitize_text_field( wp_unslash( $_REQUEST['date'] ) );
            $page_url = sanitize_text_field( wp_unslash( $_REQUEST['pageUrl'] ) );

            $days_range = 'all-time' !== $date ? (int) $date : 0; // 0 = all-time.

            // Return.

            wp_send_json_success(
                [
                    'webVitals' => Web_Vitals::get_web_vitals(
                        $page_url,
                        $days_range,
                        true,
                        true,
                        false,
                        true
                    ),
                    'shares'    => Web_Vitals::get_web_vitals_grades_share( $page_url, $days_range ),
                    'date'      => $date,
                    'pageUrl'   => $page_url,
                ]
            );
        }

        wp_send_json_error();
    }

    /**
     * Delete plugin data.
     */
    public function delete_plugin_data() {
        /*
         * Nonce check.
         */
        check_ajax_referer( 'first-party-analytics-menu', 'nonceToken' );

        // 1. Deletes options.

        foreach ( First_Party_Analytics::$options->all_options as $option => $option_data ) {
            delete_option( $option );
        }

        // 2. Deletes tables.

        global $wpdb;

        $tables_prefix = DB::get_tables_prefix();

        $wpdb->query( 'SET FOREIGN_KEY_CHECKS = 0;' ); 

        foreach ( DB::get_tables() as $table ) {
            $wpdb->query( "DROP TABLE IF EXISTS {$tables_prefix}{$table}" ); 
        }

        $wpdb->query( 'SET FOREIGN_KEY_CHECKS = 1;' ); 

        // 3. Delete posts meta.

        // 3.1. Delete "like" meta-keys.

        Data_Removal_Helpers::delete_like_meta_keys();

        // 3.2. The rest of meta fields.

        $meta_keys = [
            WC_Order::UID_META_KEY,
            WC_Order::LAST_USER_INITIATED_STATUS_CHANGE_TIME_META_KEY,
            WC_Order::VISITED_PAGES_META_KEY,
        ];

        foreach ( $meta_keys as $meta_key ) {
            $pages = get_posts(
                [
                    'post_type'      => 'any',
                    'post_status'    => 'any',
                    'posts_per_page' => -1,
                    'meta_query'     => [
                        [
                            'key' => $meta_key,
                        ],
                    ],
                    'fields'         => 'ids',
                ]
            );

            foreach ( $pages as $page_id ) {
                delete_post_meta( $page_id, $meta_key );
            }
        }

        wp_send_json_success();
    }

    /**
     * Enable recommendations.
     */
    public function enable_recommendations() {
        First_Party_Analytics::$options->set( 'setting-enable-data-sharing-and-recommendations', true );
    }

    /**
     * When recommendation was checked.
     */
    public function recommendation_check() {
        /*
         * Nonce check.
         */
        check_ajax_referer( 'first-party-analytics-menu', 'nonceToken' );

        if ( array_key_exists( 'id', $_REQUEST ) ) {
            $recommendation_id = sanitize_text_field( wp_unslash( $_REQUEST['id'] ) );

            $recommendations_list = First_Party_Analytics::$options->get( 'first-party-analytics-recommendations' );

            if ( isset( $recommendations_list[ $recommendation_id ] ) ) {
                $recommendations_list[ $recommendation_id ]['done']   = true;
                $recommendations_list[ $recommendation_id ]['doneAt'] = gmdate( 'Y-m-d H:i:s' );

                First_Party_Analytics::$options->set( 'first-party-analytics-recommendations', $recommendations_list );
            }
        }
    }
}
