<?php
/**
 * AJAX handlers.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\AJAX;

use Exception;
use First_Party_Analytics\Databases\Visitors;
use First_Party_Analytics\Tracking\Client_Side as Client_Side_Tracking;
use First_Party_Analytics\Tracking\Client_Side_Post_Visit as Client_Side_Post_Visit_Tracking;
use First_Party_Analytics\Tracking\Link_Visits as Link_Visits_Tracking;
use First_Party_Analytics\Tracking\Searches as Searches_Tracking;
use First_Party_Analytics\Tracking_Cookies;
use WP_Plugins_Core\Sanitize;

/**
 * Manages Public AJAX.
 */
final class Publ {

    /**
     * Constructor.
     */
    public function __construct() {
        $this->add_ajax_events();
    }

    /**
     * Adds AJAX listeners.
     */
    private function add_ajax_events() {
        $ajax_events = [
            'track',
            'track_post_visit',
            'track_search',
            'track_link_visit',
            'in_page_url_changed',
        ];

        foreach ( $ajax_events as $ajax_event ) {
            add_action( 'wp_ajax_first_party_analytics_' . $ajax_event, [ $this, $ajax_event ] );
            add_action( 'wp_ajax_nopriv_first_party_analytics_' . $ajax_event, [ $this, $ajax_event ] );
        }
    }

    /**
     * Tracks user visit data.
     *
     * @throws Exception Exception.
     */
    public function track() {
        if ( isset( $_REQUEST['userData'] ) ) { 
            $user_data =
                Sanitize::sanitize_array( (array) json_decode( wp_unslash( $_REQUEST['userData'] ), true ) ); 

            Client_Side_Tracking::record( $user_data );

            wp_send_json_success();
        }
        wp_send_json_error();
    }

    /**
     * Tracks user post visit data.
     *
     * @throws Exception Exception.
     */
    public function track_post_visit() {
        if ( isset( $_REQUEST['userData'] ) ) { 
            $user_data = Sanitize::sanitize_array(
                (array) json_decode( wp_unslash( $_REQUEST['userData'] ), true ) 
            );

            Client_Side_Post_Visit_Tracking::record( $user_data );

            wp_send_json_success();
        }
        wp_send_json_error();
    }

    /**
     * Tracks the visited link.
     */
    public function track_link_visit() {
        if (
            isset( $_REQUEST['source'] ) && 
            isset( $_REQUEST['href'] ) && 
            isset( $_REQUEST['time'] ) && 
            isset( $_REQUEST['postId'] ) 
        ) {
            $source = sanitize_text_field( wp_unslash( $_REQUEST['source'] ) ); 
            $href   = sanitize_text_field( wp_unslash( $_REQUEST['href'] ) ); 
            $time   = (int) sanitize_text_field( wp_unslash( $_REQUEST['time'] ) ); 
            $post_id = (int) sanitize_text_field( wp_unslash( $_REQUEST['postId'] ) ); 

            Link_Visits_Tracking::handle( $source, $href, $time, $post_id );

            wp_send_json_success();
        }
        wp_send_json_error();
    }

    /**
     * Tracks search.
     */
    public function track_search() {
        if (
            array_key_exists( 'searchText', $_REQUEST ) && 
            array_key_exists( 'page', $_REQUEST ) && 
            array_key_exists( 'selector', $_REQUEST ) && 
            array_key_exists( 'time', $_REQUEST ) 
        ) {
            $search_text = sanitize_text_field( wp_unslash( $_REQUEST['searchText'] ) ); 
            $page        = sanitize_text_field( wp_unslash( $_REQUEST['page'] ) ); 
            $selector    = sanitize_text_field( wp_unslash( $_REQUEST['selector'] ) ); 
            $time        = (int) sanitize_text_field( wp_unslash( $_REQUEST['time'] ) ); 

            Searches_Tracking::handle( $search_text, $page, $selector, $time );

            wp_send_json_success();
        }

        wp_send_json_error();
    }

    /**
     * In-page URL changed.
     */
    public function in_page_url_changed() {
        if (
            array_key_exists( 'url', $_REQUEST ) && 
            array_key_exists( 'time', $_REQUEST ) 
        ) {
            $url  = sanitize_text_field( wp_unslash( $_REQUEST['url'] ) ); 
            $time = sanitize_text_field( wp_unslash( $_REQUEST['time'] ) ); 

            $time       = gmdate( 'Y-m-d H:i:s', $time );
            $visitor_id = Tracking_Cookies::get_cookie();

            Visitors::write( $time, $visitor_id, $url, 'in-page-url-change' );

            wp_send_json_success();
        }

        wp_send_json_error();
    }
}
