<?php
/**
 * Assets
 *
 * Loads public assets (JS, CSS), adds data for them.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\Assets\Publ;

use First_Party_Analytics\First_Party_Analytics;
use First_Party_Analytics\Tracking_Cookies;
use First_Party_Analytics\Utils;

/**
 * Public assets class.
 */
final class Publ {

    /**
     * FPA-hosted analytics mimic file.
     */
    const FPA_HOSTED_ANALYTICS_MIMIC_FILE = 'https://fxhpaoxqyajvmdg.firstpartyanalytics.nl/analytics.js';

    /**
     * FPA-hosted ads mimic file.
     */
    const FPA_HOSTED_ADS_MIMIC_FILE = 'https://fxhpaoxqyajvmdg.firstpartyanalytics.nl/prebid-ads.js';

    /**
     * Inits.
     */
    public static function init() {
        $class = __CLASS__;
        new $class();
    }

    /**
     * Constructor.
     */
    public function __construct() {
        $this->register_libs();
        $this->styles();
        $this->scripts();
        $this->data_to_scripts();
    }

    /**
     * Registers libs.
     */
    private function register_libs() {}

    /**
     * Loads styles.
     */
    private function styles() {
        wp_enqueue_style(
            'first-party-analytics-public-style',
            FIRST_PARTY_ANALYTICS_URL . 'assets/build/public/index.css',
            [],
            FIRST_PARTY_ANALYTICS_VERSION
        );
    }

    /**
     * Loads scripts.
     */
    private function scripts() {

        // Timeme.js.

        wp_register_script(
            'first-party-analytics-lib-timeme.js',
            FIRST_PARTY_ANALYTICS_URL . 'libs/timeme.js/timeme.min.js',
            [],
            '2.0.0',
            true
        );

        // css-path.

        wp_register_script(
            'first-party-analytics-lib-css-path',
            FIRST_PARTY_ANALYTICS_URL . 'libs/css-path/script.min.js',
            [],
            '1.0.0',
            true
        );

        wp_enqueue_script(
            'first-party-analytics-public-head-script',
            FIRST_PARTY_ANALYTICS_URL . 'assets/build/public/head-script.js',
            [
                'first-party-analytics-lib-timeme.js',
                'first-party-analytics-lib-css-path',
            ],
            FIRST_PARTY_ANALYTICS_VERSION,
            false
        );

        // Analytics block detection script (with a variable) - analytics.js.

        $analytics_domain_type_to_use = First_Party_Analytics::$options->get( 'setting-analytics-block-detection-domain-type-to-use' );
        if ( in_array( $analytics_domain_type_to_use, [ 'fpa', 'own-domain' ], true ) ) {
            if ( 'fpa' === $analytics_domain_type_to_use ) {
                $analytics_mimic_script_address = self::FPA_HOSTED_ANALYTICS_MIMIC_FILE;
            } else {
                $own_domain                     = First_Party_Analytics::$options->get( 'setting-analytics-block-detection-own-domain' );
                $url                            = FIRST_PARTY_ANALYTICS_URL . 'assets/src/public/analytics.js';
                $analytics_mimic_script_address = str_replace(
                    wp_parse_url( $url, PHP_URL_HOST ) . ( wp_parse_url( $url, PHP_URL_PORT ) ? ':' . wp_parse_url( $url, PHP_URL_PORT ) : '' ),
                    wp_parse_url( $own_domain, PHP_URL_HOST ) . ( wp_parse_url( $own_domain, PHP_URL_PORT ) ? ':' . wp_parse_url( $own_domain, PHP_URL_PORT ) : '' ),
                    $url
                );
            }

            wp_enqueue_script( 
                'first-party-analytics-public-analytics-mimic-script',
                $analytics_mimic_script_address,
                [],
                null,
                true
            );
        }

        // Ads block detection script (with a variable) - prebid-ads.js.

        $ads_domain_type_to_use = First_Party_Analytics::$options->get( 'setting-ads-block-detection-domain-type-to-use' );
        if ( in_array( $ads_domain_type_to_use, [ 'fpa', 'own-domain' ], true ) ) {
            if ( 'fpa' === $ads_domain_type_to_use ) {
                $ads_mimic_script_script_address = self::FPA_HOSTED_ADS_MIMIC_FILE;
            } else {
                $own_domain                      = First_Party_Analytics::$options->get( 'setting-ads-block-detection-own-domain' );
                $url                             = FIRST_PARTY_ANALYTICS_URL . 'assets/src/public/prebid-ads.js';
                $ads_mimic_script_script_address = str_replace(
                    wp_parse_url( $url, PHP_URL_HOST ) . ( wp_parse_url( $url, PHP_URL_PORT ) ? ':' . wp_parse_url( $url, PHP_URL_PORT ) : '' ),
                    wp_parse_url( $own_domain, PHP_URL_HOST ) . ( wp_parse_url( $own_domain, PHP_URL_PORT ) ? ':' . wp_parse_url( $own_domain, PHP_URL_PORT ) : '' ),
                    $url
                );
            }

            wp_enqueue_script( 
                'first-party-analytics-public-ads-mimic-script',
                $ads_mimic_script_script_address,
                [],
                null,
                true
            );
        }

        if ( ! is_user_logged_in() ) {
            wp_enqueue_script(
                'first-party-analytics-public-footer-script',
                FIRST_PARTY_ANALYTICS_URL . 'assets/build/public/footer-script.js',
                [],
                FIRST_PARTY_ANALYTICS_VERSION,
                true
            );
        }
    }

    /**
     * Loads data to scripts.
     */
    private function data_to_scripts() {
        $post_id = Utils::get_currently_requested_post_id();

        $page_embed_data = [
            'ajaxurl'                 => admin_url( 'admin-ajax.php' ),
            'postId'                  => $post_id,
            'postType'                => get_post_type( $post_id ),
            'postLatestRevisionId'    => Utils::get_post_latest_revision_id( $post_id ),
            'trackingCookie'          => [
                'name'       => Tracking_Cookies::COOKIE_NAME,
                'length'     => Tracking_Cookies::COOKIE_LENGTH,
                'expiration' => Tracking_Cookies::COOKIE_EXPIRATION,
            ],
            'addPingAttribute'        => '1' === First_Party_Analytics::$options->get( 'setting-add-ping-attribute' ),
            'decimateValue'           => First_Party_Analytics::$options->get( 'setting-decimate' ),
            'analyticsBlockDetection' => [
                'domainTypeToUse' => First_Party_Analytics::$options->get( 'setting-analytics-block-detection-domain-type-to-use' ),
                'ownDomain'       => First_Party_Analytics::$options->get( 'setting-analytics-block-detection-own-domain' ),
                'customDomain'    => First_Party_Analytics::$options->get( 'setting-analytics-block-detection-custom-domain' ),
            ],
            'adsBlockDetection'       => [
                'domainTypeToUse' => First_Party_Analytics::$options->get( 'setting-ads-block-detection-domain-type-to-use' ),
                'ownDomain'       => First_Party_Analytics::$options->get( 'setting-ads-block-detection-own-domain' ),
                'customDomain'    => First_Party_Analytics::$options->get( 'setting-ads-block-detection-custom-domain' ),
            ],
        ];

        $page_embed_data['settings'] = [];

        $page_embed_data['txt'] = [
            'wasntAbleToTrackTheData'     => __( "Wasn't able to track the data.", 'first-party-analytics' ),
            'searchSourceInputIsNotFound' => __( 'Search source is not found. It was deleted or moved to a new place.', 'first-party-analytics' ),
        ];

        // Adds the data for attributes for the current variable product.

        wp_localize_script(
            'first-party-analytics-public-head-script',
            'firstPartyAnalytics',
            apply_filters( 'first_party_analytics_public_assets_data', $page_embed_data )
        );
    }
}
