<?php
/**
 * Cache.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics;

/**
 * Manages Cache.
 */
final class Cache {

    /**
     * Transient cache expiration.
     *
     * Basically, the same as raw-analytics-data-parsing interval.
     */
    const TRANSIENT_CACHE_EXPIRATION = MINUTE_IN_SECONDS;

    /**
     * Transient cache prefix.
     */
    const TRANSIENT_CACHE_PREFIX = 'first_party_analytics_cache_';

    /**
     * Returns paged data from transient cache if exists, otherwise return false.
     *
     * @param string $name The name of the cache.
     * @param array  $data Data to cache.
     * @param array  $args Arguments - used as an addition to the cache name.
     */
    public static function save( $name, array $data, $args = [] ) {
        $request_hash = hash( 'md5', wp_json_encode( $args ) ); // serialize() is discouraged due to security issues.
        $key          = self::TRANSIENT_CACHE_PREFIX . $name . '_' . $request_hash;

        $data = [
            'data'       => $data,
            'expiration' => time() + self::TRANSIENT_CACHE_EXPIRATION,
        ];

        set_transient( $key, $data, self::TRANSIENT_CACHE_EXPIRATION );
    }

    /**
     * Returns paged data from transient cache if exists, otherwise return false.
     *
     * @param string $name The name of the cache.
     * @param mixed  $args Arguments - used as an addition to the cache name.
     *
     * @return array|bool Cached data or false if no cache found.
     */
    public static function get( $name, $args = [] ) {
        $request_hash = hash( 'md5', wp_json_encode( $args ) ); // serialize() is discouraged due to security issues.
        $key          = self::TRANSIENT_CACHE_PREFIX . $name . '_' . $request_hash;

        // Cache during the page generation (so the transient can't expire during this time).
        // This is necessary because we send don't "no-cache" header for shop pages if all entities are in the hash,
        // so we wouldn't like it to expire during the page generation.
        static $cached_data = [];
        if ( array_key_exists( $key, $cached_data ) ) {
            return $cached_data[ $key ];
        }

        $transient_data = get_transient( $key );
        if (
            false !== $transient_data &&
            is_array( $transient_data ) &&
            array_key_exists( 'expiration', $transient_data ) &&
            $transient_data['expiration'] > time() &&
            array_key_exists( 'data', $transient_data )
        ) {
            $cached_data[ $key ] = $transient_data['data'];
            return $transient_data['data'];
        } else {
            return false;
        }
    }
}
