<?php
/**
 * Custom database tables.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics;

/**
 * Manages custom DB tables.
 */
final class DB {

    /**
     * Current tables schema version.
     */
    const TABLES_VERSION = 2;

    /**
     * Custom tables prefix.
     */
    const TABLES_PREFIX = 'first_party_analytics_v' . self::TABLES_VERSION . '_';

    /**
     * Creates/modifies database tables for the plugin.
     *
     * @see WC_Install::create_tables()
     */
    public static function db_delta() {
        global $wpdb;
        $wpdb->hide_errors();
        require_once ABSPATH . 'wp-admin/includes/upgrade.php';
        dbDelta( self::get_schema() );
    }

    /**
     * Return a list of tables.
     *
     * @return array Tables list.
     */
    public static function get_tables() {
        return [ // TODO: Create some dynamic list of tables instead of typing them manually.
            'link_visits',
            'page_visits',
            'page_visits_on_finish',
            'search_words',
            'sessions',
            'time_of_day',
            'visitor_details',
            'visitors',
            'special_url_params',
        ];
    }

    /**
     * Return full tables prefix.
     *
     * @return string Tables prefix.
     */
    public static function get_tables_prefix() {
        global $wpdb;
        return $wpdb->prefix . self::TABLES_PREFIX;
    }

    /**
     * Returns tables schema.
     *
     * @return string Tables schema.
     */
    public static function get_schema() {
        global $wpdb;

        $collate      = '';
        $table_prefix = self::get_tables_prefix();

        if ( $wpdb->has_cap( 'collation' ) ) {
            $collate = $wpdb->get_charset_collate();
        }

        $web_vitals_schema = '';
        foreach (
            array_merge(
                array_map( 'strtolower', array_keys( Web_Vitals::THRESHOLDS ) ),
                [ 'web_vitals_score' ]
            )
            as $web_vital
        ) {
            $web_vitals_schema .=
                "$web_vital FLOAT UNSIGNED DEFAULT 0,
                {$web_vital}_good INT UNSIGNED DEFAULT 0,
                {$web_vital}_poor INT UNSIGNED DEFAULT 0,
                {$web_vital}_needs_improvement INT UNSIGNED DEFAULT 0,
                {$web_vital}_mobile FLOAT UNSIGNED DEFAULT 0,
                {$web_vital}_mobile_good INT UNSIGNED DEFAULT 0,
                {$web_vital}_mobile_poor INT UNSIGNED DEFAULT 0,
                {$web_vital}_mobile_needs_improvement INT UNSIGNED DEFAULT 0,";
        }

        /**
         * full_search: True - recorded whole content; False - recorded just a word of the search.
         */
        $tables = "
            CREATE TABLE {$table_prefix}time_of_day (
              id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT,
              day_of_week TINYINT NOT NULL, 
              minute_of_day SMALLINT UNSIGNED NOT NULL,
              visits_count INT UNSIGNED NOT NULL,
              revision_id BIGINT(20) UNSIGNED DEFAULT 0,
              PRIMARY KEY (id),
              KEY idx_minute_of_day_revision_id (minute_of_day, revision_id),
              UNIQUE KEY uniq_day_of_week_minute_of_day_revision_id (day_of_week, minute_of_day, revision_id)
            ) $collate;
            CREATE TABLE {$table_prefix}visitor_details (
              id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT,
              a_day DATE NOT NULL,
              a_key VARCHAR(255) NOT NULL,
              a_value VARCHAR(512) NOT NULL,
              visits_count INT UNSIGNED NOT NULL,
              revision_id BIGINT(20) UNSIGNED DEFAULT 0,
              PRIMARY KEY (id),              
              KEY idx_a_key_revision_id (a_key, revision_id),
              UNIQUE KEY uniq_a_day_a_key_revision_id (a_day, a_key, revision_id)
            ) $collate;
            CREATE TABLE {$table_prefix}page_visits (
              id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT,
              a_day DATE NOT NULL,
              a_url VARCHAR(512) NOT NULL,
              post_id BIGINT(20) UNSIGNED NOT NULL,
              post_type VARCHAR(255) NOT NULL,
              unique_visits INT UNSIGNED NOT NULL,
              bot_visits INT UNSIGNED NOT NULL,
              total_visits INT UNSIGNED NOT NULL,
              internal_visits INT UNSIGNED NOT NULL,
              incoming_visits INT UNSIGNED NOT NULL,
              sessions INT UNSIGNED NOT NULL DEFAULT 0,
              bounce_rate FLOAT UNSIGNED NOT NULL DEFAULT 0,
              revision_id BIGINT(20) UNSIGNED DEFAULT 0,
              PRIMARY KEY (id),
              KEY idx_post_id_revision_id (post_id, revision_id),
              KEY idx_a_day_unique_visits_revision_id (a_day, unique_visits, revision_id),
              KEY idx_a_day_post_type_unique_visits_revision_id (a_day, post_type, unique_visits, revision_id),
              KEY idx_a_url_a_day_revision_id (a_url, a_day, revision_id),
              UNIQUE KEY uniq_post_id_a_day_revision_id (post_id, a_day, revision_id)
            ) $collate;
            CREATE TABLE {$table_prefix}page_visits_on_finish (
              id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT,
              a_day DATE NOT NULL,
              a_url VARCHAR(512) NOT NULL,
              post_id BIGINT(20) UNSIGNED NOT NULL,
              total_visits INT UNSIGNED NOT NULL,
              avg_load_time FLOAT UNSIGNED NOT NULL,
              load_times_count INT UNSIGNED DEFAULT 0,
              max_load_time FLOAT UNSIGNED NOT NULL,
              min_load_time FLOAT UNSIGNED NOT NULL,
              avg_view_time FLOAT UNSIGNED NOT NULL,
              view_times_count INT UNSIGNED DEFAULT 0,
              avg_asset_less_page_size INT UNSIGNED DEFAULT 0,
              avg_website_data_download_size INT UNSIGNED DEFAULT 0,
              avg_viewport_completed_loading_time FLOAT UNSIGNED DEFAULT 0,
              viewport_completed_loading_time_count INT UNSIGNED DEFAULT 0,
              $web_vitals_schema
              revision_id BIGINT(20) UNSIGNED DEFAULT 0,
              post_revision_id BIGINT(20) UNSIGNED DEFAULT 0,
              bf_cache_on INT UNSIGNED DEFAULT 0,
              bf_cache_off INT UNSIGNED DEFAULT 0,
              PRIMARY KEY (id),
              KEY idx_a_day_revision_id (a_day, revision_id),
              KEY idx_a_url_a_day_revision_id (a_url, a_day, revision_id),
              KEY idx_a_url_a_day_post_revision_id_revision_id (a_url, a_day, post_revision_id, revision_id),
              KEY idx_post_id_a_day_revision_id (post_id, a_day, revision_id),
              UNIQUE KEY uniq_post_id_a_day_post_revision_id_revision_id (post_id, a_day, post_revision_id, revision_id)
            ) $collate;
            CREATE TABLE {$table_prefix}search_words (
              id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT,
              word VARCHAR(255) NOT NULL,
              page_url VARCHAR(512) NOT NULL DEFAULT '',
              input_selector VARCHAR(512) NOT NULL DEFAULT '',
              `time` DATETIME DEFAULT 0,
              full_search TINYINT NOT NULL DEFAULT 0,
              revision_id BIGINT(20) UNSIGNED DEFAULT 0,
              PRIMARY KEY (id),
              KEY idx_word_revision_id (word, revision_id),
              KEY idx_page_url_revision_id (page_url, revision_id)
            ) $collate;
            CREATE TABLE {$table_prefix}link_visits (
              id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT,
              post_id BIGINT(20) UNSIGNED NOT NULL,
              visitor_id VARCHAR(255) NOT NULL,
              `time` DATETIME NOT NULL,
              `source` VARCHAR(512) NOT NULL DEFAULT '',
              `href` VARCHAR(512) NOT NULL DEFAULT '',
              `href_host` VARCHAR(512) NOT NULL DEFAULT '',
              `href_main_domain` VARCHAR(512) NOT NULL DEFAULT '',
              `href_url_params` VARCHAR(512) DEFAULT '',
              `external_dst` TINYINT NOT NULL DEFAULT 0,
              revision_id BIGINT(20) UNSIGNED DEFAULT 0,
              KEY idx_time_external_dst_revision_id (`time`, external_dst, revision_id),
              PRIMARY KEY (id)
            ) $collate;
            CREATE TABLE {$table_prefix}special_url_params (
              id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT,
              a_day DATE NOT NULL,
              referer_host VARCHAR(512) NOT NULL,
              referer_main_domain VARCHAR(512) NOT NULL,
              utm_medium VARCHAR(512) NOT NULL,
              utm_source VARCHAR(512) NOT NULL,
              utm_campaign VARCHAR(512) NOT NULL,
              utm_term VARCHAR(512) NOT NULL,
              utm_content VARCHAR(512) NOT NULL,
              revision_id BIGINT(20) UNSIGNED DEFAULT 0,
              PRIMARY KEY (id),
              KEY idx_referer_host_revision_id (referer_host, revision_id),
              KEY idx_referer_main_domain_revision_id (referer_main_domain, revision_id)
            ) $collate;
            CREATE TABLE {$table_prefix}sessions (
              id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT,
              visitor_id VARCHAR(255) NOT NULL,
              visited_pages INT UNSIGNED NOT NULL,
              first_time DATETIME NOT NULL,
              last_time DATETIME NOT NULL,
              bounced TINYINT NOT NULL DEFAULT 1,
              processed_after_finish TINYINT NOT NULL DEFAULT 0,
              revision_id BIGINT(20) UNSIGNED DEFAULT 0,
              PRIMARY KEY (id),
              KEY idx_visitor_id_revision_id (visitor_id, revision_id)
            ) $collate;
            CREATE TABLE {$table_prefix}visitors (
              id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT,
              `time` DATETIME DEFAULT 0,
              visitor_id VARCHAR(255) NOT NULL,
              `value` VARCHAR(512) NOT NULL,
              type VARCHAR(512) NOT NULL,
              PRIMARY KEY (id),
              KEY idx_visitor_id (visitor_id),
              KEY idx_visitor_id_value (visitor_id, `value`)
            ) $collate;
        ";

        return $tables;
    }
}
