<?php
/**
 * A class to handle fields mapping.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics;

use Exception;

/**
 * Class Fields_Mapping.
 */
final class Fields_Mapping {

    /**
     * Fields mapping.
     */
    const FIELDS_MAPPING_LOOKUP_CLASS = [
        'client_side_analytics'            => 'First_Party_Analytics\Tracking\Client_Side',
        'client_side_post_visit_analytics' => 'First_Party_Analytics\Tracking\Client_Side_Post_Visit',
        'server_side_analytics'            => 'First_Party_Analytics\Tracking\Server_Side',
        'server_side_noscript_analytics'   => 'First_Party_Analytics\Tracking\Server_Side_NoScript',
        'ajax_requests'                    => 'First_Party_Analytics_Premium\Tracking\Ajax_Requests',
    ];

    /**
     * Constructor.
     */
    public function __construct() {
        /**
         * When the fields structure of requests sequence was changed, reset the requests sequence.
         *
         * Start the check after premium files were loaded.
         */
        add_action(
            'plugins_loaded',
            [ $this, 'flush_requests_sequence_when_structure_changed' ],
            12
        );
    }

    /**
     * When the fields structure of requests sequence was changed, reset the requests sequence.
     */
    public static function flush_requests_sequence_when_structure_changed() {
        $hashes              = First_Party_Analytics::$options->get( 'fields-mapping-last-known-hashes' );
        $were_hashes_changed = false;
        foreach ( self::FIELDS_MAPPING_LOOKUP_CLASS as $name => $class ) {
            if ( class_exists( $class ) ) {
                $hash = md5( wp_json_encode( $class::FIELDS ) );

                if ( ! isset( $hashes[ $name ] ) || $hash !== $hashes[ $name ] ) {
                    $hashes[ $name ]     = $hash;
                    $were_hashes_changed = true;

                    // Flush the requests sequence.

                    First_Party_Analytics::$raw_analytics_data_handler->delete_from_file( $name );
                    First_Party_Analytics::$raw_analytics_data_handler->delete_from_cache( $name );
                }
            }
        }
        if ( $were_hashes_changed ) { // Flush the all-time requests.
            delete_option( 'first-party-analytics-raw-requests' );
            delete_option( 'first-party-analytics-premium-raw-requests' );

            First_Party_Analytics::$options->set( 'fields-mapping-last-known-hashes', $hashes );
        }
    }

    /**
     * Breaks file lines into an array.
     *
     * @param array $lines Lines which will be broken into an array.
     *
     * @return array The same lines. But values of those are broken into array.
     *
     * @throws Exception Exception.
     */
    public static function break_lines_into_arrays( $lines ) {
        foreach ( $lines as &$line ) {
            $line = json_decode( $line, true );
        }
        return array_filter(
            $lines,
            function( $line ) {
                return is_array( $line ) && ! ( 1 === count( $line ) );
            }
        );
    }

    /**
     * Maps file line into associative array.
     *
     * @param string $fieldset_name Fieldset name.
     * @param array  $lines         File lines to map.
     *
     * @throws Exception Exception.
     */
    public static function map_lines_into_assoc_arrays( $fieldset_name, array $lines ) {
        $mapped_lines = [];

        foreach ( $lines as $line ) {
            $mapped_array = [];

            $fields = self::FIELDS_MAPPING_LOOKUP_CLASS[ $fieldset_name ]::FIELDS;

            foreach ( $line as $i => $value ) {
                $mapped_array[ $fields[ $i ] ] = $value; // Decode if JSON-encoded data.
            }

            $mapped_lines[] = $mapped_array;
        }

        return $mapped_lines;
    }

    /**
     * Returns mapped fields for the file.
     *
     * @param string  $fieldset_name       Fieldset name. AKA request type name. AKA file name.
     * @param bool    $only_unparsed_lines Get only unparsed lines (instead of all lines in the file).
     * @param boolean $for_premium         For premium plugin.
     *
     * @throws Exception Exception.
     */
    public static function get_mapped_fields( $fieldset_name, $only_unparsed_lines = true, $for_premium = false ) {
        if ( $only_unparsed_lines ) {
            $lines = First_Party_Analytics::$raw_analytics_data_handler->get_content( $fieldset_name );
            Raw_Requests_Storage::write_for_fieldset( $fieldset_name, $lines, $for_premium ); // Populate raw requests storage (1k max), for percentiles.
        } else {
            $lines = Raw_Requests_Storage::get_for_fieldset( $fieldset_name, $for_premium ); // Return from raw requests storage.
        }
        return $lines ? self::map_lines_into_assoc_arrays( $fieldset_name, $lines ) : [];
    }

    /**
     * Write to file.
     *
     * Checks that all fields are in place.
     *
     * @param string $fieldset_name Fieldset name.
     * @param mixed  ...$fields     The list of fields with their values.
     *
     * @throws Exception Exception.
     */
    public static function write( $fieldset_name, ...$fields ) {
        $how_many_fields_passed   = count( $fields );
        $how_many_fields_expected = count( self::FIELDS_MAPPING_LOOKUP_CLASS[ $fieldset_name ]::FIELDS );
        if ( $how_many_fields_passed !== $how_many_fields_expected ) {
            throw new Exception( "Passed $how_many_fields_passed fields but expected $how_many_fields_expected." );
        }

        $un_tab_strings = function( $field ) use ( &$un_tab_strings ) {
            /**
             * Replace tabs to spaces.
             *
             * @param $string
             *
             * @return string
             */
            $un_tab = function( $string ) {
                return preg_replace( '/\t/', '', $string );
            };

            if ( is_array( $field ) ) {
                return array_map( $un_tab_strings, $field ); // If the field is an array, then json_encode it.
            } elseif ( is_string( $field ) ) {
                return $un_tab( $field );
            } else {
                return $field;
            }
        };

        $fields = array_map(
            $un_tab_strings,
            $fields
        );

        if ( Object_Cache::use_object_cache() ) {
            First_Party_Analytics::$raw_analytics_data_handler->write_to_cache( $fieldset_name, $fields );
        } else {
            First_Party_Analytics::$raw_analytics_data_handler->write_to_file( $fieldset_name, wp_json_encode( $fields ) );
        }
    }
}
