<?php
/**
 * A class to do geolocation.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics;

/**
 * Class GeoLocation.
 */
final class GeoLocation {

    /**
     * Returns the visitor IP.
     *
     * Supports CloudFlare.
     *
     * @return string IP or empty string if IP can't be determined.
     *
     * Inspired by @see \WC_Geolocation::get_ip_address
     */
    public static function get_visitor_ip() {
        if ( isset( $_SERVER['HTTP_CF_CONNECTING_IP'] ) ) { // CloudFlare.
            return sanitize_text_field( wp_unslash( $_SERVER['HTTP_CF_CONNECTING_IP'] ) );
        } elseif ( isset( $_SERVER['HTTP_X_REAL_IP'] ) ) {
            return sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_REAL_IP'] ) );
        } elseif ( isset( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) { // Forwarded.
            // Proxy servers can send through this header like this: X-Forwarded-For: client1, proxy1, proxy2
            // Make sure we always only send through the first IP in the list which should always be the client IP.
            return (string) rest_is_ip_address( trim( current( preg_split( '/,/', sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) ) ) ) );
        } elseif ( isset( $_SERVER['REMOTE_ADDR'] ) ) {
            return sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) );
        } else {
            return '';
        }
    }

    /**
     * Fetches the country code from the request headers, if one is available.
     *
     * @return string The country code pulled from the headers, or empty string if one was not found.
     *
     * Inspired by @see \WC_Geolocation::get_country_code_from_headers
     */
    public static function get_country_code_from_headers() {
        $country_code = '';

        $headers = [
            'MM_COUNTRY_CODE',
            'GEOIP_COUNTRY_CODE',
            'HTTP_CF_IPCOUNTRY',
            'HTTP_X_COUNTRY_CODE',
        ];

        foreach ( $headers as $header ) {
            if ( empty( $_SERVER[ $header ] ) ) {
                continue;
            }

            $country_code = strtoupper( sanitize_text_field( wp_unslash( $_SERVER[ $header ] ) ) );
            break;
        }

        return $country_code;
    }

    /**
     * Returns IP country.
     *
     * @param string $ip IP address.
     *
     * @return string ISO country code if was able to geolocate, or empty string otherwise.
     */
    public static function get_ip_country( $ip ) {
        if ( ! $ip ) {
            return '';
        }

        if ( method_exists( 'WC_Geolocation', 'geolocate_ip' ) ) {
            $geolocation = \WC_Geolocation::geolocate_ip( $ip );
            if (
                isset( $geolocation['country'] ) &&
                $geolocation['country']
            ) {
                return $geolocation['country'];
            }
        }

        return MaxMind\Database::get_instance()->get_iso_country_code_for_ip( $ip );
    }
}
