<?php
/**
 * A class for web vitals debug data download.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics;

use Exception;
use First_Party_Analytics\Databases\Page_Visits_Finish;

/**
 * Web vitals debug data download class.
 */
final class Get_Web_Vitals_Debug_Data {

    /**
     * Listens for the export action.
     *
     * Accepts: $_GET['nonce']
     *          $_GET['first-party-analytics-action']
     */
    public static function add_listeners() {
        if (
            isset( $_GET['nonce'], $_GET['first-party-analytics-action'] ) && 
            wp_verify_nonce( wp_unslash( $_GET['nonce'] ), 'first-party-analytics-menu' ) && 
            'download-web-vitals-debug-data' === wp_unslash( $_GET['first-party-analytics-action'] ) 
        ) {
            self::export();
        }
    }

    /**
     * Does the export.
     */
    private static function export() {
        self::send_headers();
        self::send_content( self::get_json_data() );
        die();
    }

    /**
     * Returns JSON filename.
     *
     * @return string
     */
    private static function get_filename() {
         return 'first-party-analytics-web-vitals-debug-data-' . time() . '.json';
    }

    /**
     * Sends the export headers.
     */
    private static function send_headers() {
        if ( function_exists( 'gc_enable' ) ) {
            gc_enable(); // phpcs:ignore PHPCompatibility.FunctionUse.NewFunctions.gc_enableFound
        }
        if ( function_exists( 'apache_setenv' ) ) {
            @apache_setenv( 'no-gzip', 1 ); 
        }
        @ini_set( 'zlib.output_compression', 'Off' ); 
        @ini_set( 'output_buffering', 'Off' ); 
        @ini_set( 'output_handler', '' ); 
        ignore_user_abort( true );
        if ( function_exists( 'wc_set_time_limit' ) ) {
            wc_set_time_limit( 0 );
        }
        if ( function_exists( 'wc_nocache_headers' ) ) {
            wc_nocache_headers();
        }
        header( 'Content-Type: text/csv; charset=utf-8' );
        header( 'Content-Disposition: attachment; filename=' . self::get_filename() );
        header( 'Pragma: no-cache' );
        header( 'Expires: 0' );
    }

    /**
     * Gets web vitals debug data for export.
     *
     * @return string Web vitals debug data.
     *
     * @throws Exception Exception.
     */
    private static function get_json_data() {
        $data = [];

        $pages = get_posts(
            [
                'post_type'      => 'any',
                'post_status'    => 'any',
                'posts_per_page' => -1,
                'meta_query'     => [
                    [
                        'key'         => Page_Visits_Finish::REQUESTS_ENQUEUE_POST_META_NAME,
                        'compare_key' => 'LIKE',
                    ],
                ],
                'fields'         => 'ids',
            ]
        );

        foreach ( $pages as $page_id ) {
            $post_meta = get_post_meta( $page_id );

            $web_vitals_keys = array_filter(
                array_keys( $post_meta ),
                function( $meta_key ) {
                    return str_starts_with( $meta_key, Page_Visits_Finish::REQUESTS_ENQUEUE_POST_META_NAME );
                }
            );

            if ( $web_vitals_keys ) {
                $data[ $page_id ] = [];

                foreach ( $web_vitals_keys as $meta_key ) {
                    $data[ $page_id ][ $meta_key ] = unserialize( $post_meta[ $meta_key ][0] ); 
                }
            }
        }

        return wp_json_encode( $data );
    }

    /**
     * Sends the export content.
     *
     * @param string $json_data JSON content.
     */
    private static function send_content( $json_data ) {
        echo $json_data; 
    }
}
