<?php
/**
 * A class to manage migrations.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics;

/**
 * Class Migrations.
 */
final class Migrations {

    /**
     * Constructor.
     *
     * @param int $last_handled_version_update Last handled version update.
     */
    public function __construct( $last_handled_version_update ) {
        if ( ! $last_handled_version_update ) { // No need to migrate for the fresh installation.
            return;
        }

        $methods_raw = array_values(
            array_filter(
                get_class_methods( $this ),
                function ( $method ) {
                    return str_starts_with( $method, 'v_' );
                }
            )
        );
        sort( $methods_raw );

        $versions_with_methods = [];
        foreach ( $methods_raw as $method ) {
            $version                           = str_replace( '_', '.', str_replace( 'v_', '', $method ) );
            $versions_with_methods[ $version ] = $method;
        }

        $versions_to_run_migrators_for = array_filter(
            array_keys( $versions_with_methods ),
            function ( $method ) use ( $last_handled_version_update ) {
                return 1 === version_compare( $method, $last_handled_version_update );
            }
        );

        foreach ( $versions_to_run_migrators_for as $version_to_run_migrator_for ) {
            $this->{$versions_with_methods[ $version_to_run_migrator_for ]}();
        }
    }

    /**
     * Delete tables ROWS with nulls.
     */
    private function v_0_2_1() {
        global $wpdb;

        $table_name = $wpdb->prefix . 'first_party_analytics_v2_link_visits';

        $wpdb->query( 
            "
            DELETE FROM $table_name WHERE href = 'null';
            "
        );
    }

    /**
     * Drop obsolete unique indexes for tables.
     */
    private function v_0_2_12() {
        global $wpdb;

        $table_name = Databases\Time_Of_Day::get_table_name();
        $wpdb->query( "ALTER TABLE $table_name DROP INDEX `uniq_day_of_week_minute_of_day`;" ); 

        $table_name = Databases\Visitor_Details::get_table_name();
        $wpdb->query( "ALTER TABLE $table_name DROP INDEX `uniq_a_day_a_key_a_value`;" ); 

        $table_name = Databases\Page_Visits::get_table_name();
        $wpdb->query( "ALTER TABLE $table_name DROP INDEX `uniq_a_url_a_day`;" ); 

        $table_name = Databases\Page_Visits_Finish::get_table_name();
        $wpdb->query( "ALTER TABLE $table_name DROP INDEX `uniq_a_url_a_day`;" ); 
    }

    /**
     * Drop obsolete indexes for tables.
     */
    private function v_0_2_15() {
        global $wpdb;

        $table_name = Databases\Time_Of_Day::get_table_name();
        $wpdb->query( "ALTER TABLE $table_name DROP INDEX `idx_minute_of_day`;" ); 
        $wpdb->query( "ALTER TABLE $table_name DROP INDEX `idx_day_of_week_minute_of_day`;" ); 

        $table_name = Databases\Visitor_Details::get_table_name();
        $wpdb->query( "ALTER TABLE $table_name DROP INDEX `idx_a_key`;" ); 
        $wpdb->query( "ALTER TABLE $table_name DROP INDEX `idx_a_day_a_key`;" ); 
        $wpdb->query( "ALTER TABLE $table_name DROP INDEX `idx_a_day_a_key_a_value`;" ); 

        $table_name = Databases\Page_Visits::get_table_name();
        $wpdb->query( "ALTER TABLE $table_name DROP INDEX `idx_post_id`;" ); 
        $wpdb->query( "ALTER TABLE $table_name DROP INDEX `idx_a_day_unique_visits`;" ); 
        $wpdb->query( "ALTER TABLE $table_name DROP INDEX `idx_a_day_post_type_unique_visits`;" ); 
        $wpdb->query( "ALTER TABLE $table_name DROP INDEX `idx_a_url_a_day`;" ); 

        $table_name = Databases\Page_Visits_Finish::get_table_name();
        $wpdb->query( "ALTER TABLE $table_name DROP INDEX `idx_a_day`;" ); 
        $wpdb->query( "ALTER TABLE $table_name DROP INDEX `idx_a_url_a_day`;" ); 

        $table_name = Databases\Search_Words::get_table_name();
        $wpdb->query( "ALTER TABLE $table_name DROP INDEX `idx_word`;" ); 
        $wpdb->query( "ALTER TABLE $table_name DROP INDEX `idx_page_url`;" ); 

        $table_name = Databases\Special_Url_Params::get_table_name();
        $wpdb->query( "ALTER TABLE $table_name DROP INDEX `idx_referer_host`;" ); 
        $wpdb->query( "ALTER TABLE $table_name DROP INDEX `idx_referer_main_domain`;" ); 

        $table_name = Databases\Sessions::get_table_name();
        $wpdb->query( "ALTER TABLE $table_name DROP INDEX `idx_visitor_id`;" ); 
    }

    /**
     * Delete unnecessary options.
     */
    private function v_0_2_16() {
        delete_option( 'first-party-analytics-avg-non-product-page-visits-per-session-data' );
        delete_option( 'first-party-analytics-avg-product-page-visits-per-session-data' );
        delete_option( 'first-party-analytics-avg-shop-page-visits-per-session-data' );
    }

    /**
     * Flush the table to reset the counter for new columns.
     */
    private function v_0_2_23() {
        global $wpdb;

        $table_name = Databases\Page_Visits_Finish::get_table_name();
        $wpdb->query( "TRUNCATE TABLE $table_name;" ); 
    }

    /**
     * Flush the table to reset the counter for new columns.
     */
    private function v_0_2_29() {
        global $wpdb;

        $table_name = Databases\Page_Visits_Finish::get_table_name();
        $wpdb->query( "TRUNCATE TABLE $table_name;" ); 
    }

    /**
     * Delete obsolete indexes.
     */
    private function v_1_0_1() {
        global $wpdb;

        $table_name = Databases\Page_Visits_Finish::get_table_name();
        $wpdb->query( "ALTER TABLE $table_name DROP INDEX `uniq_a_url_a_day_revision_id`;" ); 
    }

    /**
     * Delete obsolete indexes.
     */
    private function v_1_0_2() {
        global $wpdb;

        $table_name = Databases\Page_Visits_Finish::get_table_name();
        $wpdb->query( "ALTER TABLE $table_name DROP INDEX `uniq_a_url_a_day_post_revision_id_revision_id`;" ); 

        $table_name = Databases\Page_Visits::get_table_name();
        $wpdb->query( "ALTER TABLE $table_name DROP INDEX `uniq_a_url_a_day_revision_id`;" ); 
    }

    /**
     * Fix indexes.
     */
    private function v_1_0_6() {
        global $wpdb;

        $table_name = Databases\Visitor_Details::get_table_name();
        $wpdb->query( "ALTER TABLE $table_name DROP INDEX `idx_a_day_a_key_revision_id`;" ); 
    }

    /**
     * Fix indexes.
     */
    private function v_1_0_8() {
        global $wpdb;

        $table_name = Databases\Visitor_Details::get_table_name();
        $wpdb->query( "ALTER IGNORE TABLE $table_name ADD UNIQUE `uniq_a_day_a_key_revision_id` (a_day, a_key, revision_id);" ); 
    }

    /**
     * New structure.
     */
    private function v_1_0_11() {
        global $wpdb;

        $table_name = Databases\Page_Visits_Finish::get_table_name();
        $wpdb->query( "DROP TABLE $table_name;" ); 
    }

    /**
     * Rename (fix) table column.
     */
    private function v_1_0_18() {
        global $wpdb;

        $table_name = Databases\Page_Visits_Finish::get_table_name();
        $wpdb->query( "ALTER TABLE $table_name CHANGE `tffb_count` `ttfb_count` INT(10) UNSIGNED NULL DEFAULT '0';" ); 
    }

    /**
     * Rename (fix) table column.
     */
    private function v_1_0_23() {
        global $wpdb;

        $table_name = Databases\Page_Visits_Finish::get_table_name();
        $wpdb->query( "UPDATE $table_name SET `load_times_count` = `total_visits`, `view_times_count` = `total_visits`;" ); 
    }

    /**
     * Change tables.
     */
    private function v_1_0_36() {
        global $wpdb;

        $table_name = Databases\Page_Visits_Finish::get_table_name();

        // Delete averages and counts for web vitals (replaced by buckets).

        $wpdb->query( 
            "
             ALTER TABLE $table_name
                DROP COLUMN `avg_ttfb`,
                DROP COLUMN `ttfb_count`,
                DROP COLUMN `avg_fcp`,
                DROP COLUMN `fcp_count`,
                DROP COLUMN `avg_lcp`,
                DROP COLUMN `lcp_count`,
                DROP COLUMN `avg_fid`,
                DROP COLUMN `fid_count`,
                DROP COLUMN `avg_cls`,
                DROP COLUMN `cls_count`,
                DROP COLUMN `avg_inp`,
                DROP COLUMN `inp_count`,
                DROP COLUMN `avg_si`,
                DROP COLUMN `si_count`,
                DROP COLUMN `avg_tbt`,
                DROP COLUMN `tbt_count`,
                DROP COLUMN `avg_tti`,
                DROP COLUMN `tti_count`;
            "
        );

        // Reset bucket.

        $wpdb->query( 
            "
             UPDATE $table_name SET
                 `ttfb_good` = 0,
                 `ttfb_poor` = 0,
                 `ttfb_needs_improvement` = 0,
                 `fcp_good` = 0,
                 `fcp_poor` = 0,
                 `fcp_needs_improvement` = 0,
                 `lcp_good` = 0,
                 `lcp_poor` = 0,
                 `lcp_needs_improvement` = 0,
                 `fid_good` = 0,
                 `fid_poor` = 0,
                 `fid_needs_improvement` = 0,
                 `cls_good` = 0,
                 `cls_poor` = 0,
                 `cls_needs_improvement` = 0,
                 `inp_good` = 0,
                 `inp_poor` = 0,
                 `inp_needs_improvement` = 0,
                 `si_good` = 0,
                 `si_poor` = 0,
                 `si_needs_improvement` = 0,
                 `tbt_good` = 0,
                 `tbt_poor` = 0,
                 `tbt_needs_improvement` = 0,
                 `tti_good` = 0,
                 `tti_poor` = 0,
                 `tti_needs_improvement` = 0,
                 `web_vitals_score_mobile_good` = 0,
                 `web_vitals_score_mobile_poor` = 0,
                 `web_vitals_score_mobile_needs_improvement` = 0,
                 `web_vitals_score_desktop_good` = 0,
                 `web_vitals_score_desktop_poor` = 0,
                 `web_vitals_score_desktop_needs_improvement` = 0;
            "
        );
    }

    /**
     * Drop the table with the old structure.
     */
    private function v_1_0_41() {
        global $wpdb;

        $table_name = Databases\Page_Visits_Finish::get_table_name();

        $wpdb->query( 
            "DROP TABLE $table_name"
        );
    }

    /**
     * Drop the table with the wrong data.
     */
    private function v_1_0_44() {
        global $wpdb;

        $table_name = Databases\Page_Visits_Finish::get_table_name();

        $wpdb->query( 
            "DROP TABLE $table_name"
        );
    }

    /**
     * Delete a wrong index (as unique is already used).
     */
    private function v_1_0_52() {
        global $wpdb;

        $table_name = Databases\Time_Of_Day::get_table_name();

        $wpdb->query( 
            "ALTER TABLE $table_name DROP INDEX `idx_day_of_week_minute_of_day_revision_id`;"
        );
    }

    /**
     * Flush the all-time requests due to the fixed bug in the requests.
     */
    public function v_1_0_75() {
        delete_option( 'first-party-analytics-raw-requests' );
        delete_option( 'first-party-analytics-premium-raw-requests' );
    }
}
