<?php
/**
 * Class to handle plugin options.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics;

use Exception;
use First_Party_Analytics\Data_Sharing\Data_Sharing;
use First_Party_Analytics\Data_Sharing\Recommendations_Fetch;

/**
 * Manages Options using the WordPress options API.
 */
final class Options {

    /**
     * All options.
     *
     * Values:
     *  'default':  Default option value.           If not set, equals false.
     *  'autoload': Whether to autoload the option. If not set, equals true.
     *
     * @var array
     */
    public $all_options = [];

    /**
     * Settings options.
     *
     * @var array
     */
    public $settings_options = [];

    /**
     * Constructor.
     */
    public function __construct() {
        $this->all_options = require FIRST_PARTY_ANALYTICS_DIR . 'data/options.php';

        $this->define_settings_options();
        $this->add_listeners();
    }

    /**
     * Defines settings options.
     */
    private function define_settings_options() {
        foreach ( $this->all_options as $option => $option_data ) {
            if ( preg_match( '@^first-party-analytics-setting-@', $option ) ) {
                $this->settings_options[] = $option;
            }
        }
    }

    /**
     * Adds listeners for options to execute the logic on their change.
     */
    private function add_listeners() {
        $options_with_handlers = [
            'first-party-analytics-maxmind-license-key' => [ $this, 'maxmind_license_key' ],
            'first-party-analytics-setting-enable-data-sharing-and-recommendations' => [ $this, 'data_sharing' ],
        ];

        foreach ( $options_with_handlers as $option => $handler ) {
            add_action( "delete_option_$option", $handler, 10, 1 );
            add_action( "add_option_$option", $handler, 10, 2 );
            add_action( "update_option_$option", $handler, 10, 3 );
        }
    }

    /**
     * Handles MaxMind license key change.
     *
     * When added or changed, downloads the database.
     *
     * @param mixed  $old_value The old option value.
     * @param mixed  $value     The new option value.
     * @param string $option    Option.
     *
     * @throws Exception Exception.
     */
    public function maxmind_license_key( $old_value = null, $value = null, $option = null ) {
        if ( null === $value ) { // delete_option_$option (1 args).
        } else {
            if (
                null !== $option &&
                $value !== $old_value
            ) { // update_option_$option (3 args).
                if ( $value ) {
                    MaxMind\Database::get_instance()->update_database( false );
                }
            } else { // add_option_$option (2 args).
                if ( $value ) {
                    MaxMind\Database::get_instance()->update_database( false );
                }
            }
        }
    }

    /**
     * Handles data sharing setting change.
     *
     * @param mixed  $old_value The old option value.
     * @param mixed  $value     The new option value.
     * @param string $option    Option.
     *
     * @throws Exception Exception.
     */
    public function data_sharing( $old_value = null, $value = null, $option = null ) {
        if ( null === $value ) { // delete_option_$option (1 args).
            Data_Sharing::unschedule_interval();
        } else {
            if (
                null !== $option &&
                $value !== $old_value
            ) { // update_option_$option (3 args).
                if ( ! $value ) {
                    Data_Sharing::unschedule_interval();
                    Recommendations_Fetch::unschedule_interval();
                }
            } else { // add_option_$option (2 args).
                if ( ! $value ) {
                    Data_Sharing::unschedule_interval();
                    Recommendations_Fetch::unschedule_interval();
                }
            }
        }
    }

    /**
     * Validates option name, and if it does not exist, throws an exception.
     *
     * @param string $option_name Name of the option to validate.
     *
     * @throws Exception Exception.
     */
    private function validation_option( $option_name ) {
        if ( ! array_key_exists( $option_name, $this->all_options ) ) {
            throw new Exception(
                First_Party_Analytics::$name . ': ' . esc_html__( 'Unknown option name:', 'tmm-wp-plugins-core' ) . ' ' . esc_html( $option_name )
            );
        }
    }

    /**
     * Gets the option value. Returns the default value if the value does not exist.
     *
     * @param string $option_name Name of the option to get.
     *
     * @return mixed Option value.
     *
     * @throws Exception Exception.
     */
    public function get( $option_name ) {
        try {
            $this->validation_option( $option_name );
        } catch ( Exception $e ) {
            $option_name = First_Party_Analytics::$slug . '-' . $option_name;
            $this->validation_option( $option_name );
        }

        $option_data = $this->all_options[ $option_name ];
        return get_option( $option_name, array_key_exists( 'default', $option_data ) ? $option_data['default'] : false );
    }

    /**
     * Sets the option. Update the value if the option for the given name already exists.
     *
     * @param string $option_name Name of the option to set.
     * @param mixed  $value       Value to set for the option.
     *
     * @throws Exception Exception.
     */
    public function set( $option_name, $value ) {
        try {
            $this->validation_option( $option_name );
        } catch ( Exception $e ) {
            $option_name = First_Party_Analytics::$slug . '-' . $option_name;
            $this->validation_option( $option_name );
        }

        $option_data = $this->all_options[ $option_name ];
        update_option( $option_name, $value, array_key_exists( 'autoload', $option_data ) ? $option_data['autoload'] : null );
    }

    /**
     * Deletes the option value.
     *
     * @param string $option_name Name of the option to delete.
     *
     * @throws Exception Exception.
     */
    public function delete( $option_name ) {
        try {
            $this->validation_option( $option_name );
        } catch ( Exception $e ) {
            $option_name = First_Party_Analytics::$slug . '-' . $option_name;
            $this->validation_option( $option_name );
        }

        delete_option( $option_name );
    }
}
