<?php
/**
 * Raw analytics data parsing (reads and adds anonymised data to the DBs).
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics;

use Exception;
use First_Party_Analytics\Databases\Page_Visits;
use First_Party_Analytics\Databases\Time_Of_Day;
use First_Party_Analytics\Databases\Visitor_Details;
use First_Party_Analytics\Databases\Page_Visits_Finish;
use First_Party_Analytics\Databases\Sessions as Sessions_DB;
use First_Party_Analytics\Databases\Special_Url_Params as Special_Url_Params_DB;
use First_Party_Analytics\Tracking\Server_Side;
use First_Party_Analytics\Tracking\Client_Side;
use First_Party_Analytics\Tracking\Server_Side_NoScript;
use First_Party_Analytics\Tracking\Client_Side_Post_Visit;

/**
 * Class Raw_Analytics_Data_Parsing.
 */
final class Raw_Analytics_Data_Parsing {

    /**
     * Interval hook name.
     */
    const INTERVAL_HOOK_NAME = 'first_party_analytics_process_raw_data';

    /**
     * Interval.
     */
    const INTERVAL = MINUTE_IN_SECONDS;

    /**
     * Constructor.
     */
    public function __construct() {
        // Schedule the action.
        add_action( 'init', [ $this, 'schedule_interval' ] );

        // Unschedule interval action on plugin deactivation.
        register_deactivation_hook( FIRST_PARTY_ANALYTICS_FILE, [ $this, 'plugin_deactivation_handler' ] );

        // Do raw analytics data parsing.
        add_action( self::INTERVAL_HOOK_NAME, [ $this, 'process' ] );
    }

    /**
     * Schedules the interval action if it isn't scheduled.
     *
     * @throws Exception Exception.
     */
    public static function schedule_interval() {
        if ( ! as_next_scheduled_action( self::INTERVAL_HOOK_NAME ) ) {
            as_schedule_recurring_action(
                time(),
                self::INTERVAL,
                self::INTERVAL_HOOK_NAME
            );
        }
    }

    /**
     * Plugin deactivation handler.
     */
    public function plugin_deactivation_handler() {
        as_unschedule_all_actions( self::INTERVAL_HOOK_NAME );
    }

    /**
     * Multiplies requests by their decimate.
     *
     * @param array $requests Requests.
     *
     * @return array Requests multiplied.
     */
    public static function multiply_requests_by_decimate( &$requests ) {
        foreach ( $requests as $request ) {
            if (
                isset( $request['decimate'] ) &&
                (int) $request['decimate'] > 1
            ) {
                for (
                    $how_many_copies_left_create = (int) $request['decimate'] - 1;
                    $how_many_copies_left_create;
                    $how_many_copies_left_create --
                ) {
                    $requests[] = $request;
                }
            }
        }
        return $requests;
    }

    /**
     * Truncates the raw analytics data files and returns requests sequence.
     *
     * Merges 3 types of analytics into one sequence:
     *
     * - server_side_analytics
     * - client_side_analytics
     * - server_side_noscript_analytics
     *
     * @return array An array which contains 2 sequences: Main requests sequence (the merge of the three types of analytics above)
     *               and page visits finish sequence.
     * @throws Exception Exception.
     */
    private function get_requests_sequence() {
        $server_side_data = Fields_Mapping::get_mapped_fields( Server_Side::NAME );
        $client_side_data = Fields_Mapping::get_mapped_fields( Client_Side::NAME );
        $noscript_data    = Fields_Mapping::get_mapped_fields( Server_Side_NoScript::NAME );

        // Converts three different types of analytics requests into one requests sequence.

        $requests = [];
        foreach ( $server_side_data as $row ) {
            $row['type'] = 'server';
            $requests[]  = $row;
        }
        foreach ( $client_side_data as $row ) {
            $row['type'] = 'client';
            $requests[]  = $row;
        }
        foreach ( $noscript_data as $row ) {
            $row['type'] = 'noscript';
            $requests[]  = $row;
        }

        /**
         * Sorts requests by time, ascending.
         *
         * @param array $requests Requests.
         */
        $sort_requests_by_time = function( &$requests ) {
            // Sort requests by time.
            usort(
                $requests,
                function( $a, $b ) {
                    if ( $a['time'] > $b['time'] ) {
                        return 1;
                    } elseif ( $a['time'] < $b['time'] ) {
                        return -1;
                    } else {
                        return 0;
                    }
                }
            );
        };

        $requests = self::multiply_requests_by_decimate( $requests );
        $sort_requests_by_time( $requests );

        /**
         * Converts initial requests sequence into requests_for_users object.
         */

        $requests_for_users = [];
        foreach ( $requests as $request ) {
            $id = $request['id'];
            if ( ! array_key_exists( $id, $requests_for_users ) ) {
                $requests_for_users[ $id ] = [];
            }
            $requests_for_users[ $id ][] = $request;
        }

        /**
         * Converts $requests_for_users variable into single requests sequence.
         *
         * Does not add detached requests (with id='') to the sequence.
         *
         * @param array $requests_for_users Requests for users.
         *
         * @return array Requests sequence.
         */
        $get_requests_sequence = function( $requests_for_users ) use ( $sort_requests_by_time ) {
            $requests_sequence = [];
            foreach ( $requests_for_users as $user_id => $user_requests ) {
                if ( '' !== $user_id ) {
                    foreach ( $user_requests as $request ) {
                        $requests_sequence[] = $request;
                    }
                }
            }
            $sort_requests_by_time( $requests_sequence );
            return $requests_sequence;
        };

        /**
         * Attaches user-detached requests to the found user (id) with the same IP.
         *
         * Detached requests are server requests for the first page visit when no cookie present yet, and so
         * we are attaching them to the user ID, if anything with the same URL and IP is present for this chunk.
         *
         * Time should be less than of the attached request.
         */

        if ( array_key_exists( '', $requests_for_users ) ) {
            $user_detached_requests = $requests_for_users[''];

            /**
             * Attaches detached requests to the requests for users, if found anything acceptable.
             *
             * The IP should be the same, and the time should be less than any other request for this IP.
             *
             * @param array $detached_request A detached request.
             */
            $attach_request_if_possible = function( $detached_request ) use ( &$requests_for_users, $get_requests_sequence, $sort_requests_by_time ) {
                $requests_sequence = $get_requests_sequence( $requests_for_users );
                foreach ( $requests_sequence as $request ) {
                    if ( $request['ip'] === $detached_request['ip'] ) {
                        if ( $request['time'] > $detached_request['time'] ) {
                            $detached_request['id']           = $request['id'];
                            $detached_request['was_detached'] = true;

                            $requests_for_users[ $request['id'] ][] = $detached_request;
                            $sort_requests_by_time( $requests_for_users[ $request['id'] ] );
                        }
                        break;
                    }
                }
            };

            foreach ( $user_detached_requests as $detached_request ) {
                $attach_request_if_possible( $detached_request );
            }
        }

        /**
         * Deletes duplicate requests.
         *
         * For example, when user visits a page, then
         * two requests can be created: 1. Server-side; 2. Client-side / NoScript.
         *
         * So if the next found request for the same page for the same user has type client-side or noscript,
         * then the server request can be deleted because it's a (less comprehensive) duplicate.
         *
         * If the next found request for the same page for the same user has type "server" then do nothing.
         */

        foreach ( $requests_for_users as &$user_requests ) {
            $requests_count = count( $user_requests );
            foreach ( $user_requests as $index => &$request ) {
                if ( 'server' === $request['type'] ) {

                    /**
                     * Returns the ID of the next request made by this user for the same URL.
                     *
                     * If found, and the type is other than "server",
                     * then deletes the previous type, i.e. "server" request (because it's duplicate).
                     *
                     * But some server-requests data (like page and postType) should be copied to the current request
                     * (which will be preserved) because they are detected more precisely server-side than client-side
                     * (e.g. get_queried_object_id()) can be used to detect the currently requested page.
                     *
                     * @return false|int
                     */
                    $get_this_url_this_client_next_request_id = function() use ( $request, $index, $requests_count, $user_requests ) {
                        $page_url = $request['url'];
                        for ( $i = $index + 1; $i < $requests_count; $i ++ ) {
                            if ( isset( $user_requests[ $i ] ) ) {
                                $potential_next_request = $user_requests[ $i ];
                                if ( $potential_next_request['url'] === $page_url ) {
                                    return $i;
                                }
                            }
                        }
                        return false;
                    };

                    $next_request_id_same_user_same_page = $get_this_url_this_client_next_request_id();
                    if (
                        $next_request_id_same_user_same_page &&
                        'server' !== $user_requests[ $next_request_id_same_user_same_page ]['type']
                    ) {
                        /**
                         * 1. Maybe supplement the fields which can be more precisely determined server-side.
                         *
                         * So if the value is set, then it's on purpose for these fields, and don't do server-side supplementation for them.
                         *
                         * @important Basically, contains the stuff that might come from the headers.
                         *
                         * @example If we do noscript tracking, and have HTML caching enabled (like from Cloudflare),
                         * then obviously noscript headers-based fields are more trustworthy than server-side.
                         */

                        $fields_to_maybe_supplement = [
                            'country', // Can be determined with country header noscript-side.
                            'dnt',     // Can't be reliably determined client-side.
                            'referer', // Can't be determined noscript-side.
                        ];
                        foreach ( $fields_to_maybe_supplement as $field ) {
                            if (
                                isset( $request[ $field ] ) && $request[ $field ] &&
                                (
                                    ! isset( $user_requests[ $next_request_id_same_user_same_page ][ $field ] ) ||
                                    ! $user_requests[ $next_request_id_same_user_same_page ][ $field ] ||
                                    'fpa-null-val' === $user_requests[ $next_request_id_same_user_same_page ][ $field ]
                                )
                            ) {
                                $user_requests[ $next_request_id_same_user_same_page ][ $field ] = $request[ $field ];
                            }
                        }

                        /**
                         * 2. Set the flag.
                         */

                        $user_requests[ $next_request_id_same_user_same_page ]['were_fields_supplemented_from_server_side'] = true;

                        /**
                         * 3. Delete the "duplicate" server-side request.
                         */

                        unset( $user_requests[ $index ] );
                    }
                }
            }
        }

        return $get_requests_sequence( $requests_for_users );
    }

    /**
     * Parse raw analytics data.
     *
     * @throws Exception Exception.
     */
    public function process() {

        // Main requests sequence.

        $unparsed_requests_sequence = Requests_Exclusion::exclude( $this->get_requests_sequence() );

        Time_Of_Day::convert_sequence_into_db_population( $unparsed_requests_sequence );
        Visitor_Details::convert_sequence_into_db_population( $unparsed_requests_sequence );
        Page_Visits::convert_sequence_into_db_population( $unparsed_requests_sequence );
        Sessions_DB::convert_sequence_into_db_population( $unparsed_requests_sequence );
        Special_Url_Params_DB::convert_sequence_into_db_population( $unparsed_requests_sequence );

        // Post visit finish requests sequence.

        $post_visit_finish_unparsed_requests_sequence = Fields_Mapping::get_mapped_fields(
            Client_Side_Post_Visit::NAME
        );
        $unfiltered_requests_count                    = count( $post_visit_finish_unparsed_requests_sequence );
        $post_visit_finish_unparsed_requests_sequence = Requests_Exclusion::exclude( $post_visit_finish_unparsed_requests_sequence );

        Sessions_DB::convert_sequence_into_db_population( $post_visit_finish_unparsed_requests_sequence, false );

        $post_visit_finish_all_requests_sequence = Fields_Mapping::get_mapped_fields(
            Client_Side_Post_Visit::NAME,
            false
        );
        $all_requests_count                      = count( $post_visit_finish_all_requests_sequence );

        if (
            $all_requests_count > First_Party_Analytics::PERCENTILES_THRESHOLD // Do not write when percentiles base isn't formed yet.
        ) {
            // When the percentiles base is formed for the first time, write all requests, because they weren't written until it was formed (so, until this moment).
            if ( $all_requests_count - $unfiltered_requests_count <= First_Party_Analytics::PERCENTILES_THRESHOLD ) {
                $post_visit_finish_unparsed_requests_sequence = Requests_Exclusion::exclude( $post_visit_finish_all_requests_sequence );
            }

            Page_Visits_Finish::convert_sequence_into_db_population(
                self::multiply_requests_by_decimate( $post_visit_finish_unparsed_requests_sequence ),
                $post_visit_finish_all_requests_sequence
            );
        }

        // Misc.

        Sessions::process_ended_sessions();
    }
}
