<?php
/**
 * A class to handle raw analytics data.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics;

use First_Party_Analytics\Tracking\Server_Side_NoScript;
use Exception;

/**
 * Class Raw_Analytics_Data.
 */
final class Raw_Analytics_Data {

    /**
     * Cache key.
     *
     * @var string
     */
    const CACHE_KEY = 'raw-requests';

    /**
     * Temp directory where analytics data is stored.
     *
     * @var string
     */
    private $analytics_dir;

    /**
     * Constructor.
     */
    public function __construct() {
        $this->analytics_dir = trailingslashit( sys_get_temp_dir() );
    }

    /**
     * Returns real filename. Its name is combined with salt (generated randomly per plugin install) and hashed.
     *
     * Used to maximize the difficulty to hijack its name.
     *
     * @param string $filename Filename.
     *
     * @return string Real filename.
     * @throws Exception Exception.
     */
    private function get_real_filename( $filename ) {
        // Do not add salt for NoScript tracker because it's a standalone script with no access to options.
        $salt = Server_Side_NoScript::NAME !== $filename ? Utils::get_salt() : '';
        return md5( $filename . $salt );
    }

    /**
     * Returns filepath.
     *
     * The name is used is not the passed one but the hashed with salt.
     *
     * @param string $filename Filename.
     *
     * @return string Filepath in temp directory.
     * @throws Exception Exception.
     */
    private function get_filepath( $filename ) {
        return $this->analytics_dir . self::get_real_filename( $filename );
    }

    /**
     * Returns file content.
     *
     * And also flushes its content.
     *
     * @param string $fieldset_name Fieldset name.
     *
     * @return array Content, where each array item is one record.
     * @throws Exception Exception.
     */
    private function get_file_content( $fieldset_name ) {
        $live_file = $this->get_filepath( $fieldset_name );

        $content = [];

        /**
         * Returns file data: file pointer and formatted file content (array of lines).
         *
         * The file is opened with lock until "fclose( $file_data['fp'] );".
         *
         * @param string $file Path to the file.
         *
         * @return array
         */
        $get_file_data = function ( $file ) {
            $fp = fopen( $file, 'r+' ); 
            flock( $fp, LOCK_EX );

            $filesize         = filesize( $file );
            $raw_file_content = $filesize ? fread( $fp, $filesize ) : ''; 
            $file_content     = explode( PHP_EOL, $raw_file_content );

            if ( $filesize ) {
                array_pop( $file_content );
            }

            $content = array_map(
                function ( $x ) {
                    return trim( $x, "\n" );
                },
                $file_content
            );

            return [
                'fp'          => $fp,               // FP to close the file.
                'filesize'    => $filesize,
                'raw_content' => $raw_file_content, // Raw file content, a string.
                'content'     => $content,          // Array of lines.
            ];
        };

        /*
         * Reads live file.
         */

        if ( First_Party_Analytics::$fs->is_file( $live_file ) ) {
            $file_data = $get_file_data( $live_file );
            ftruncate( $file_data['fp'], 0 );
            fclose( $file_data['fp'] ); 
            array_push( $content, ...$file_data['content'] );
        }

        return Fields_Mapping::break_lines_into_arrays( $content );
    }

    /**
     * Returns cache content.
     *
     * And also flushed cache value for this fieldset to an empty array.
     *
     * @param string $fieldset_name Fieldset name.
     *
     * @return array Content, where each array item is one record.
     * @throws Exception Exception.
     */
    private function get_cache_content( $fieldset_name ) {
        $cache_value = wp_cache_get( self::CACHE_KEY, First_Party_Analytics::$slug );

        $value = [];

        if ( $cache_value && isset( $cache_value[ $fieldset_name ] ) ) {
            $value = $cache_value[ $fieldset_name ];
            // Set cache value for this fieldset to an empty array as it's already retrieved.
            $cache_value[ $fieldset_name ] = [];
            wp_cache_set( self::CACHE_KEY, $cache_value, First_Party_Analytics::$slug );
        }

        return $value;
    }

    /**
     * Returns fieldset content.
     *
     * And also flushed them, for both file and cache.
     *
     * But records in Raw_Requests_Storage class, for percentiles; and obviously populates to the DB.
     *
     * @param string $fieldset_name Fieldset name.
     *
     * @return array File content, where each array is a file line.
     * @throws Exception Exception.
     */
    public function get_content( $fieldset_name ) {
        return array_merge(
            $this->get_file_content( $fieldset_name ),
            $this->get_cache_content( $fieldset_name )
        );
    }

    /**
     * Adds specified content to the specified file (its name is md5-hashed with salt for security).
     *
     * @param string $fieldset_name Fieldset name.
     * @param string $content       File content.
     *
     * @throws Exception Exception.
     */
    public function write_to_file( $fieldset_name, $content ) {
        file_put_contents( $this->get_filepath( $fieldset_name ), $content . PHP_EOL, FILE_APPEND | LOCK_EX ); 
    }

    /**
     * Adds specified content under the specified cache key (its name is md5-hashed with salt for security).
     *
     * @param string $fieldset_name Fieldset name.
     * @param array  $fields        Fields to write.
     *
     * @throws Exception Exception.
     */
    public function write_to_cache( $fieldset_name, $fields ) {
        $cache_value                   = wp_cache_get( self::CACHE_KEY, First_Party_Analytics::$slug );
        $cache_value[ $fieldset_name ] = isset( $cache_value[ $fieldset_name ] ) ? array_merge( $cache_value[ $fieldset_name ], [ $fields ] ) : [ $fields ];
        wp_cache_set( self::CACHE_KEY, $cache_value, First_Party_Analytics::$slug );
    }

    /**
     * Deletes content of the specified file.
     *
     * @param string $fieldset_name Fieldset name.
     */
    public function delete_from_file( $fieldset_name ) {
        wp_delete_file( $this->get_filepath( $fieldset_name ) );
    }

    /**
     * Delete the data from the cache.
     *
     * @param string $fieldset_name Fieldset name.
     */
    public function delete_from_cache( $fieldset_name ) {
        $cache_value                   = wp_cache_get( self::CACHE_KEY, First_Party_Analytics::$slug );
        $cache_value[ $fieldset_name ] = [];
        wp_cache_set( self::CACHE_KEY, $cache_value, First_Party_Analytics::$slug );
    }
}
