<?php
/**
 * A class for requests exclusion, based on cookie, or IP.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics;

use Exception;
use IPLib\Factory;

/**
 * Class Requests_Exclusion.
 */
final class Requests_Exclusion {

    /**
     * Excludes requests.
     *
     * @param array $requests Requests.
     *
     * @throws Exception Exception.
     */
    public static function exclude( array $requests ) {
        return self::exclude_by_ip( self::exclude_by_cookie( $requests ) );
    }

    /**
     * Excludes requests by cookie.
     *
     * @param array $requests Requests.
     *
     * @return array Requests, excluded by cookie.
     *
     * @throws Exception Exception.
     */
    private static function exclude_by_cookie( array $requests ) {
        $exclusion_cookies = array_column( First_Party_Analytics::$options->get( 'setting-exclusion-cookies' ), 'value', 'name' );

        // No-tracking cookie.
        $exclusion_cookies[ No_Tracking_Cookie::COOKIE_NAME ] = '';

        if ( $exclusion_cookies ) {
            foreach ( $requests as $i => $request ) {
                $cookies = isset( $request['cookies'] ) ? $request['cookies'] : [];
                foreach ( $cookies as $cookie_name => $cookie_value ) {
                    if (
                        array_key_exists( $cookie_name, $exclusion_cookies ) &&
                        in_array( $exclusion_cookies[ $cookie_name ], [ '', $cookie_value ], true ) // Empty string is for any value.
                    ) {
                        unset( $requests[ $i ] );
                    }
                }
            }
        }

        return $requests;
    }

    /**
     * Excludes requests by IP.
     *
     * @param array $requests Requests.
     *
     * @return array Requests, excluded by IP.
     *
     * @throws Exception Exception.
     */
    private static function exclude_by_ip( array $requests ) {
        $exclusion_ips = First_Party_Analytics::$options->get( 'setting-exclusion-ips' );

        if ( $exclusion_ips ) {
            foreach ( $requests as $i => $request ) {
                $request_ip = $request['ip'];
                $address    = Factory::parseAddressString( $request_ip );

                foreach ( $exclusion_ips as $exclusion_ip ) {
                    $range = Factory::parseRangeString( $exclusion_ip );
                    if ( $range && $range->contains( $address ) ) {
                        unset( $requests[ $i ] );
                    }
                }
            }
        }

        return $requests;
    }
}
