<?php
/**
 * Class to manage sessions.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics;

use First_Party_Analytics\Databases\Page_Visits;
use First_Party_Analytics\Databases\Sessions as Sessions_DB;
use First_Party_Analytics\Databases\Visitors as Visitors_DB;
use First_Party_Analytics\Databases\Page_Visits as Page_Visits_DB;
use First_Party_Analytics\Sessions as Sessions_Class;
use First_Party_Analytics\WooCommerce\WC_Order;

/**
 * A class for sessions management.
 */
final class Sessions {

    /**
     * Session timeout.
     */
    const SESSION_TIMEOUT = MINUTE_IN_SECONDS * 30;

    /**
     * Process finished sessions.
     *
     * - Update bounce rate for the pages visited during the session.
     */
    public static function process_ended_sessions() {
        global $wpdb;

        $session_table_name = Sessions_DB::get_table_name();

        $expired_time = gmdate( 'Y-m-d H:i:s', time() - Sessions_Class::SESSION_TIMEOUT );

        $ended_sessions_to_process = $wpdb->get_results( 
            $wpdb->prepare(
                "
                    SELECT * FROM $session_table_name
                    WHERE processed_after_finish = 0 AND last_time < %s;
                ",
                $expired_time
            ),
            ARRAY_A
        );

        foreach ( $ended_sessions_to_process as $session ) {
            $visits_table_name     = Visitors_DB::get_table_name();
            $page_visits_db        = Page_Visits_DB::get_table_name();
            $session_visited_pages = $wpdb->get_results( 
                $wpdb->prepare( // INNER JOIN to get post type mainly.
                    "
                    SELECT *, DATE(`time`) as a_day FROM $visits_table_name
                    LEFT JOIN $page_visits_db pv
                        ON value = pv.a_url AND a_day BETWEEN DATE(%s) and DATE(%s)
                    WHERE type = 'page-visit' AND visitor_id = %s AND `time` BETWEEN %s and %s
                    ORDER BY `time`;
                ",
                    $session['first_time'],
                    $session['last_time'],
                    $session['visitor_id'],
                    $session['first_time'],
                    $session['last_time']
                ),
                ARRAY_A
            );

            /**
             * Add the list of visited pages to the orders that were created by the user during the session.
             */

            $session_orders_ids = get_posts( // wc_get_orders() doesn't support meta_query.
                [
                    'post_type'   => 'shop_order',
                    'post_status' => array_keys( wc_get_order_statuses() ),
                    'numberposts' => -1,
                    'orderby'     => 'date',
                    'order'       => 'ASC',
                    'meta_query'  => [
                        'relation' => 'AND',
                        [
                            'key'   => WC_Order::UID_META_KEY,
                            'value' => $session['visitor_id'],
                        ],
                        [
                            'key'     => WC_Order::LAST_USER_INITIATED_STATUS_CHANGE_TIME_META_KEY,
                            'value'   => [
                                strtotime( $session['first_time'] ),
                                strtotime( $session['last_time'] ) + self::SESSION_TIMEOUT - 1, // Add some more range, just in case.
                            ],
                            'type'    => 'NUMERIC',
                            'compare' => 'BETWEEN',
                        ],
                    ],
                    'fields'      => 'ids',
                ]
            );

            foreach ( $session_orders_ids as $session_orders_id ) {
                $order = wc_get_order( $session_orders_id );

                $order_user_initiated_status_change_time = $order->get_meta( WC_Order::LAST_USER_INITIATED_STATUS_CHANGE_TIME_META_KEY );
                $pages_visited_before_the_order          = array_filter(
                    $session_visited_pages,
                    function( $visited_page ) use ( $order_user_initiated_status_change_time ) {
                        return strtotime( $visited_page['time'] ) < (int) $order_user_initiated_status_change_time;
                    }
                );

                $order->update_meta_data( WC_Order::VISITED_PAGES_META_KEY, $pages_visited_before_the_order );
                $order->save();
            }

            /**
             * Update bounce rate for the pages visited during the session.
             */

            // Use array_unique() to not increment session counter several times for the same page but only once.
            $unique_ids                   = array_keys( array_unique( array_column( $session_visited_pages, 'value', 'id' ) ) );
            $session_unique_visited_pages = array_filter(
                $session_visited_pages,
                function( $visited_page ) use ( $unique_ids ) {
                    return in_array( (int) $visited_page['id'], $unique_ids, true );
                }
            );

            foreach ( $session_unique_visited_pages as $visited_page ) {
                $page = Page_Visits::get_by_url_and_day( $visited_page['value'], $visited_page['a_day'] );
                if ( $page ) {
                    $page_visits_table_name = Page_Visits::get_table_name();
                    $wpdb->query( 
                        $wpdb->prepare( // Update "sessions" at the end as it's more convenient for me to do incremental averaging with the previous total.
                            "
                            UPDATE $page_visits_table_name SET
                            
                            bounce_rate = (bounce_rate * sessions + %d) / (sessions + 1),
                            sessions = sessions + 1
    
                            WHERE id = %d;
                            ",
                            (int) $session['bounced'],
                            (int) $page['id']
                        )
                    );
                }
            }

            /**
             * Mark the session that it was processed.
             *
             * Change session "processed_after_finish" to 1.
             */

            $wpdb->query( 
                $wpdb->prepare(
                    "
                    UPDATE $session_table_name SET
                    
                    processed_after_finish = 1

                    WHERE id = %d;
                    ",
                    (int) $session['id']
                )
            );
        }
    }
}
