<?php
/**
 * Class for Settings Export.
 *
 * Many methods here are inspired by WC_CSV_Exporter class.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics;

use Exception;

/**
 * Settings Export Class.
 */
final class Settings_Export {

    /**
     * Listens for the export action.
     *
     * Accepts: $_GET['nonce']
     *          $_GET['first-party-analytics-action']
     */
    public static function add_listeners() {
        if (
            isset( $_GET['nonce'], $_GET['first-party-analytics-action'] ) && 
            wp_verify_nonce( wp_unslash( $_GET['nonce'] ), 'first-party-analytics-menu' ) && 
            'export-settings' === wp_unslash( $_GET['first-party-analytics-action'] ) 
        ) {
            self::export();
        }
    }

    /**
     * Does the export.
     */
    private static function export() {
        self::send_headers();
        self::send_content( self::get_json_data() );
        die();
    }

    /**
     * Returns JSON filename.
     *
     * @return string
     */
    private static function get_filename() {
        return 'first-party-analytics-settings-' . time() . '.json';
    }

    /**
     * Sends the export headers.
     */
    private static function send_headers() {
        if ( function_exists( 'gc_enable' ) ) {
            gc_enable(); // phpcs:ignore PHPCompatibility.FunctionUse.NewFunctions.gc_enableFound
        }
        if ( function_exists( 'apache_setenv' ) ) {
            @apache_setenv( 'no-gzip', 1 ); 
        }
        @ini_set( 'zlib.output_compression', 'Off' ); 
        @ini_set( 'output_buffering', 'Off' ); 
        @ini_set( 'output_handler', '' ); 
        ignore_user_abort( true );
        if ( function_exists( 'wc_set_time_limit' ) ) {
            wc_set_time_limit( 0 );
        }
        if ( function_exists( 'wc_nocache_headers' ) ) {
            wc_nocache_headers();
        }
        header( 'Content-Type: text/csv; charset=utf-8' );
        header( 'Content-Disposition: attachment; filename=' . self::get_filename() );
        header( 'Pragma: no-cache' );
        header( 'Expires: 0' );
    }

    /**
     * Gets JSON data for export.
     *
     * @return string All plugin settings options with their values.
     *
     * @throws Exception Exception.
     */
    private static function get_json_data() {
        $data = [];
        foreach ( First_Party_Analytics::$options->settings_options as $option ) {
            $data[ $option ] = First_Party_Analytics::$options->get( $option );
        }
        return wp_json_encode( $data );
    }

    /**
     * Sends the export content.
     *
     * @param string $json_data JSON content.
     */
    private static function send_content( $json_data ) {
        echo $json_data; 
    }
}
