<?php
/**
 * A class for sources.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics;

use First_Party_Analytics\Representation\Tables\Visits_By_Sources;
use RecursiveArrayIterator;
use RecursiveIteratorIterator;

/**
 * Class Sources.
 */
final class Sources {

    /**
     * Returns the list of sources in the same order as they are printed, from top to bottom.
     *
     * @param array|bool $sources                                 Sources.
     * @param bool       $accumulate_in_parents_children_referers Whether to accumulate in parent children referers.
     * @param array      $current_path                            Current path.
     * @param bool       $recursive_call                          If the call is recursive.
     * @param bool       $add_direct_visits                       If to add direct visits.
     *
     * @return void|array
     */
    public static function get_sources_list(
            $sources = false,
            $accumulate_in_parents_children_referers = true,
            $current_path = [],
            $recursive_call = false,
            $add_direct_visits = false
    ) {
        static $cache = [];
        $cache_key    = md5( wp_json_encode( $cache ) );
        if ( isset( $cache[ $cache_key ] ) ) {
            return $cache;
        }

        if ( false === $recursive_call ) {
            if ( false === $sources ) {
                $sources = First_Party_Analytics::$options->get( 'setting-sources' );
            }

            if ( $add_direct_visits ) {
                $sources = array_replace_recursive(
                    $sources,
                    [
                        __( 'Direct visits', 'first-party-analytics' ) => [
                            [ '', '' ],
                        ],
                    ]
                );
            }
        }

        $current_path_str = implode( ' / ', $current_path );

        static $sources_list               = [];
        $sources_list[ $current_path_str ] = [];

        $source_referers = array_values(
            array_filter(
                $sources,
                function( $v, $k ) {
                    return is_int( $k );
                },
                ARRAY_FILTER_USE_BOTH
            )
        );

        foreach ( $source_referers as $source_data ) {
            $sources_list[ $current_path_str ][] = $source_data[1];
        }

        $sources_list[ $current_path_str ] = array_unique( $sources_list[ $current_path_str ] );

        $source_groups = array_filter(
            $sources,
            function( $v, $k ) {
                return ! is_int( $k );
            },
            ARRAY_FILTER_USE_BOTH
        );

        foreach ( $source_groups as $source_group_name => $source_group_data ) {
            $next_path = array_merge( $current_path, [ $source_group_name ] );
            self::get_sources_list( $source_group_data, $accumulate_in_parents_children_referers, $next_path, true );
        }

        // Before output.

        if ( ! $recursive_call ) {
            if ( $accumulate_in_parents_children_referers ) {
                // Add referers of children sources to the parents (so parents contain all refers of their children).
                foreach ( $sources_list as $source => &$referers ) {
                    foreach ( $sources_list as $potentially_children_source => $potentially_children_source_referers ) {
                        if ( str_starts_with( $potentially_children_source, $source . ' /' ) ) {
                            $referers = array_unique( array_merge( $referers, $potentially_children_source_referers ) );
                        }
                    }
                    $sources_list[''] = array_unique( array_merge( $sources_list[''], $referers ) );
                }
            }

            $cache[ $cache_key ] = $sources_list;
            $sources_list        = [];
            return $cache[ $cache_key ];
        }
    }

    /**
     * Split last level.
     *
     * @param string $key Key.
     * @param array  $val Value.
     *
     * @return array
     */
    public static function split_last_level( $key, $val = [] ) {
        return preg_match( '#(.+) / (.+)$#', $key, $keys )
            ? self::split_last_level( $keys[1], [ $keys[2] => $val ] )
            : [ $key => $val ];
    }

    /**
     * Print sources children.
     *
     * @param array  $sources        Sources.
     * @param array  $previous_path  Previous path, when going recursively, lists the names of previous elements.
     * @param bool   $with_utm_tags  Print sources with UTM tags.
     * @param string $date_range     Date range for UTM tags retrieval. Get UTM tags only for the relevant date range.
     * @param array  $utm_list       UTM-list that was merged to this sources list. Used for detection of UTM stuff and for not filtering it out.
     */
    private static function print_source_children(
            $sources,
            $previous_path = [],
            $with_utm_tags = false,
            $date_range = Visits_By_Sources::DEFAULT_DATE_RANGE,
            $utm_list = []
    ) {
        $source_groups = $previous_path ? $sources : array_filter(
            $sources,
            function( $v, $k ) {
                return ! is_int( $k );
            },
            ARRAY_FILTER_USE_BOTH
        );

        static $num;

        if ( ! $previous_path ) {
            $num = 2;
        }

        static $all_referers_for_utm;

        /**
         * For UTM tags, do not get referers with 0 visits (the empty ones).
         *
         * So hide the source also for them.
         */
        if ( ! $previous_path && $with_utm_tags ) {
            if ( null === $all_referers_for_utm ) {
                $all_referers_for_utm = self::get_sources_list( false, false, [], false, true );
            }
            $visits_list = Visits_By_Sources::get_the_list( false, $date_range );

            $i = 0;
            foreach ( array_keys( $all_referers_for_utm ) as $k ) {
                if ( ! isset( $visits_list[ $i ] ) ) {
                    unset( $all_referers_for_utm[ $k ] );
                }
                $i ++;
            }
        }

        foreach ( $source_groups as $source_group_name => $source_group_data ) {
            $the_utm_list = $utm_list; // Do not change the value for the recursive parent.

            $current_path     = array_merge( $previous_path, [ $source_group_name ] );
            $current_path_str = implode( ' / ', $current_path );

            if ( $with_utm_tags ) {
                /**
                 * Figure out if this should be filtered.
                 *
                 * Do not print source groups with 0 visits.
                 *
                 * But do not filter-out UTM (Ads) stuff.
                 */

                if (
                    ! isset( $all_referers_for_utm[ $current_path_str ] )
                ) {
                    $closest_parent_that_is_not_filtered_out = '';
                    foreach ( array_keys( $all_referers_for_utm ) as $potential_parent_path_str ) {
                        if (
                            str_starts_with( $current_path_str, $potential_parent_path_str ) &&
                            strlen( $potential_parent_path_str ) > strlen( $closest_parent_that_is_not_filtered_out )
                        ) {
                            $closest_parent_that_is_not_filtered_out = $potential_parent_path_str;
                        }
                    }

                    if (
                        isset( array_keys( $the_utm_list )[0] ) &&
                        array_keys( $the_utm_list )[0]
                    ) {
                        $ads_key_name = array_keys( $the_utm_list )[0];
                        $ads_path_str = implode( ' / ', array_merge( explode( ' / ', $closest_parent_that_is_not_filtered_out ), [ $ads_key_name ] ) );
                        if ( ! str_starts_with( $current_path_str, $ads_path_str ) ) {
                            continue;
                        }
                    } else {
                        continue;
                    }
                }

                $referers = isset( $all_referers_for_utm[ $current_path_str ] ) ? $all_referers_for_utm[ $current_path_str ] : []; // For added source there is no referers.

                if ( $referers ) {
                    $referers_list = [];
                    foreach ( $referers as $referer ) {
                        $referers_list[] = [ '', $referer ];
                    }

                    $the_list = Visits_By_Sources::get_the_list(
                        [ '' => $referers_list ],
                        $date_range,
                        true,
                        false
                    );

                    if ( $the_list && $the_list[0] ) {

                        /**
                         * Fix the list.
                         *
                         * 1. Rename empty keys to corresponding UTM keys' missing naming;
                         * 2. Replace int (counter) to an empty list, as we do not need it.
                         *
                         * Contains an array like:
                         *
                         * [
                         *   5                         => [ 'All-time' => 13 ], <- Standard value from visits count.
                         *  '5 / Ads'                  => [ 'All-time' => 5 ],  <- Based on UTM tags.
                         *  '5 / Ads / cpc'            => [ 'All-time' => 5 ],
                         *  '5 / Ads / cpc / facebook' => [ 'All-time' => 5 ]
                         * ]
                         */

                        if ( Visits_By_Sources::DEFAULT_DATE_RANGE !== $date_range ) {
                            $the_list[0] = array_merge_recursive( ... array_values( $the_list[0] ) );
                        }

                        $one_dimensional_list = [];
                        $iterator             = new RecursiveIteratorIterator(
                            new RecursiveArrayIterator( $the_list[0] ),
                            RecursiveIteratorIterator::SELF_FIRST
                        );
                        foreach ( $iterator as $v ) {
                            for (
                                $path_elements = [], $i = 0, $z = $iterator->getDepth();
                                $i <= $z;
                                $i ++
                            ) {
                                if ( 0 === $i ) {
                                    $path_elements[] = 'Ads';
                                }

                                $key = $iterator->getSubIterator( $i )->key();

                                if ( '' === $key ) {
                                    $key = 0 === $i ? __( 'No utm_medium', 'first-party-analytics' ) :
                                        ( 1 === $i ? __( 'No utm_source', 'first-party-analytics' ) :
                                            ( 2 === $i ? __( 'No utm_campaign', 'first-party-analytics' ) :
                                                ( 3 === $i ? __( 'No utm_term', 'first-party-analytics' ) :
                                                    __( 'No utm_content', 'first-party-analytics' )
                                                )
                                            )
                                        );
                                }

                                $path_elements[] = $key;
                            }
                            $path = implode( ' / ', $path_elements );

                            $one_dimensional_list[ $path ] = [];
                        }

                        ksort( $one_dimensional_list ); // Keep the order in-sync with Visits_By_Sources.

                        // Convert one-dimensional list back to multi-dimensional.

                        $fixed_list = [];
                        foreach ( array_keys( $one_dimensional_list ) as $key ) {
                            $fixed_list = array_merge_recursive( $fixed_list, self::split_last_level( $key ) );
                        }
                        $the_utm_list = array_merge_recursive( $the_utm_list, $fixed_list );

                        $source_group_data = array_merge(
                            $the_utm_list,
                            $source_group_data
                        );
                    }
                }
            }

            echo '<li>';
            echo '<span class="setting-entity"
             data-num="' . esc_html( $num ) . '"  
             data-depth="' . esc_html( count( $previous_path ) + 1 ) . '"  
             data-path="' . esc_attr( $current_path_str ) . '">' . esc_html( $source_group_name ) . '</span>';

            $num ++;

            $children_source_groups = array_filter(
                $source_group_data,
                function( $v, $k ) {
                    return ! is_int( $k );
                },
                ARRAY_FILTER_USE_BOTH
            );

            if ( $children_source_groups ) {
                echo '<ul>';
                self::print_source_children(
                    $children_source_groups,
                    $current_path,
                    $with_utm_tags,
                    $date_range,
                    $the_utm_list
                );
                echo '</ul>';
            }

            echo '</li>';
        }
    }

    /**
     * Print sources list element.
     *
     * @param array|bool $sources           Sources.
     * @param bool       $skip_selected     Skip selected or not.
     * @param bool       $with_utm_tags     Print sources with UTM tags.
     * @param string     $date_range        Date range for UTM tags retrieval. Get UTM tags only for the relevant date range.
     * @param bool       $add_direct_visits If to add direct visits.
     */
    public static function display_sources_list(
            $sources = false,
            $skip_selected = false,
            $with_utm_tags = false,
            $date_range = Visits_By_Sources::DEFAULT_DATE_RANGE,
            $add_direct_visits = false
    ) {
        if ( ! $sources ) {
            $sources = First_Party_Analytics::$options->get( 'setting-sources' );
        }

        if ( $add_direct_visits ) {
            $sources = array_replace_recursive(
                $sources,
                [
                    __( 'Direct visits', 'first-party-analytics' ) => [
                        [ '', '' ],
                    ],
                ]
            );
        }

        ?>
        <ul>
            <li>
                <span data-num="1" class="setting-entity source-root default <?php echo ! $skip_selected ? 'selected' : ''; ?>">
                    <?php esc_html_e( 'Sources', 'first-party-analytics' ); ?>
                </span>
                <ul class="fpa-sources-container">
                    <?php self::print_source_children( $sources, [], $with_utm_tags, $date_range ); ?>
                </ul>
            </li>
        </ul>
        <?php
    }
}
