<?php
/**
 * A class with misc utils methods.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics;

use Exception;
use Pdp\Cache;
use Pdp\CurlHttpClient;
use Pdp\Rules;
use Pdp\Domain;

/**
 * Class Utils.
 */
final class Utils {

    /**
     * Returns random float between 0 and 1.
     */
    public static function get_random_float() {
        return wp_rand( 0, 100000 ) / 100000;
    }

    /**
     * Whether the current record should be written.
     *
     * Based on decimate setting. E.g. decimate 10 means each 10th visit is recorded.
     *
     * @return bool
     *
     * @throws Exception Exception.
     */
    public static function decimate() {
        return self::get_random_float() < 1 / (int) First_Party_Analytics::$options->get( 'setting-decimate' );
    }

    /**
     * Returns static salt value. It's random per plugin install.
     *
     * @throws Exception Exception.
     */
    public static function get_salt() {
        $salt = First_Party_Analytics::$options->get( 'salt' );
        if ( false === $salt ) {
            $salt = wp_generate_password();
            First_Party_Analytics::$options->set( 'salt', $salt );
        }
        return $salt;
    }

    /**
     * Adds URL param.
     *
     * @param string $url   URL to which to add param.
     * @param string $param Param name to add.
     * @param string $value Value of the param to add.
     *
     * @return string URL with added param.
     *
     * @throws Exception Exception.
     */
    public static function add_url_param( $url, $param, $value ) {
        $url_parts = wp_parse_url( $url );

        return $url_parts['scheme'] . '://' . $url_parts['host'] .
            ( array_key_exists( 'port', $url_parts ) ? ':' . $url_parts['port'] : '' ) .
            ( array_key_exists( 'path', $url_parts ) ? untrailingslashit( $url_parts['path'] ) : '' ) .
            ( wp_parse_url( $url, PHP_URL_QUERY ) ? '&' : '?' ) . "$param=$value";
    }

    /**
     * Standardize the URL (if it has query part, then remove trailing slash from the path, otherwise add it).
     *
     * @param string $url URL to which to add param.
     *
     * @return string Standardized URL.
     *
     * @throws Exception Exception.
     */
    public static function standardize_url( $url ) {
        $url_parts = wp_parse_url( $url );

        return $url_parts['scheme'] . '://' . $url_parts['host'] .
            ( array_key_exists( 'port', $url_parts ) ? ':' . $url_parts['port'] : '' ) .
            ( array_key_exists( 'path', $url_parts ) ?
                ( array_key_exists( 'query', $url_parts ) ? untrailingslashit( $url_parts['path'] ) : trailingslashit( $url_parts['path'] ) )
                : '' ) .
            ( array_key_exists( 'query', $url_parts ) ? '?' . $url_parts['query'] : '' );
    }

    /**
     * Converts array column to float.
     *
     * @param array $array   Array the column of which to convert to float.
     * @param int   $col_num Which col to convert to float.
     *
     * @return array
     */
    public static function convert_array_column_to_float( $array, $col_num ) {
        return array_map(
            function( $array ) use ( $col_num ) {
                $index           = $col_num - 1;
                $array[ $index ] = round( $array[ $index ], 2 );
                return $array;
            },
            $array
        );
    }

    /**
     * Get the domain data.
     *
     * @param string $domain Domain.
     *
     * @return array Domain data.
     */
    public static function get_domain_data( $domain ) {
        static $public_suffix_list;
        if ( null === $public_suffix_list ) { // Cache.
            $rules = Rules::createFromPath(
                FIRST_PARTY_ANALYTICS_DIR . 'libs/public-suffix/public_suffix_list.dat.txt'
            );
        }

        $domain = $rules->resolve( $domain );

        $registrable_domain = $domain->getRegistrableDomain();
        $public_suffix      = $domain->getPublicSuffix();

        return [
            'registrable_domain'  => $registrable_domain,
            'public_suffix'       => $public_suffix,
            'second_level_domain' => rtrim( $registrable_domain, $public_suffix ) // e.g. "google" for "www.pictures.google.co.uk'
        ];
    }

    /**
     * Get currently requested post ID.
     *
     * @return int Post ID or 0 is can't get.
     */
    public static function get_currently_requested_post_id() {
        $post_id = get_queried_object_id();

        if (
            ! $post_id &&
            function_exists( 'is_shop' ) &&
            is_shop()
        ) {
            $post_id = wc_get_page_id( 'shop' );
        }

        return $post_id;
    }

    /**
     * Get the last revision for the post (by ID).
     *
     * @param int $post_id Post ID to get the revision for.
     *
     * @return int Latest post revision of the post or 0 is can't get.
     */
    public static function get_post_latest_revision_id( $post_id ) {
        $revisions       = wp_get_post_revisions(
            $post_id,
            [
                'posts_per_page' => 1,
                'orderby'        => 'date',
                'order'          => 'DESC',
            ]
        );
        $latest_revision = array_shift( $revisions );
        return $latest_revision ? $latest_revision->ID : 0;
    }

    /**
     * Get MySQL server version and type.
     *
     * @return array The data.
     */
    public static function get_mysql_server_version_and_type() {
        global $wpdb;

        static $result;
        if ( null !== $result ) { // Cache.
            return $result;
        }

        $result = wp_cache_get( 'get_mysql_server_version_and_type', 'first-party-analytics' );

        if ( ! $result ) {
            if ( $wpdb->use_mysqli ) {
                $mysql_server_type = mysqli_get_server_info( $wpdb->dbh ); 
            } else {
                $mysql_server_type = mysql_get_server_info( $wpdb->dbh ); 
            }

            $server_version   = $wpdb->get_var( "SELECT version()" );
            $is_using_mariadb = (bool) stristr( $mysql_server_type, 'mariadb' );

            $result = [
                'is_mariadb' => $is_using_mariadb,
                'version'    => $server_version,
            ];

            wp_cache_set( 'get_mysql_server_version_and_type', $result, 'first-party-analytics', HOUR_IN_SECONDS );
        }

        return $result;
    }

    /**
     * Escapes HTML before the output.
     *
     * Removes "script" and "style" tags.
     *
     * @param string $html HTML to filter.
     *
     * @return string HTML.
     */
    public static function esc_html_before_output( $html ) {
        $html = preg_replace( '@<(script|style)[^>]*?>.*?</\\1>@si', '', $html );
        return trim( $html );
    }

    /**
     * Get the currency.
     *
     * WC can be disabled because of the AJAX. Or WC can't be even never installed, so then return empty string ''.
     *
     * @return string
     */
    public static function get_currency() {
        $currency = function_exists( 'get_woocommerce_currency' ) ?
            get_woocommerce_currency() :
            apply_filters( 'woocommerce_currency', get_option( 'woocommerce_currency' ) );

        if ( ! $currency ) {
            $currency = '';
        }
        return $currency;
    }
}
