<?php
/**
 * A class for web-vitals.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics;

use First_Party_Analytics\Databases\Page_Visits_Finish;

/**
 * Class Web_Vitals.
 */
final class Web_Vitals {

    /**
     * Percentile for web vitals.
     */
    const PERCENTILE = 75;

    /**
     * The list of web vitals that are used for the score calculation.
     */
    const WEB_VITALS_USED_FOR_SCORE_CALCULATION = [
        'FCP',
        'SI',
        'LCP',
        'TTI',
        'TBT',
        'CLS',
    ];

    /**
     * Web vitals thresholds.
     *
     * 2 marks between: good | needs-improvement | poor
     */
    const THRESHOLDS = [
        'TTFB' => [ 800, 1800 ],  // https://web.dev/ttfb/#what-is-a-good-ttfb-score.
        'LCP'  => [ 2500, 4000 ], // https://web.dev/lcp/#what-is-a-good-lcp-score.
        'INP'  => [ 200, 500 ],   // https://web.dev/inp/#whats-a-good-inp-value.
        'FID'  => [ 100, 300 ],   // https://web.dev/fid/#what-is-a-good-fid-score.
        'FCP'  => [ 1800, 3000 ], // https://web.dev/fcp/#what-is-a-good-fcp-score.
        'CLS'  => [ 0.1, 0.25 ],  // https://web.dev/cls/#what-is-a-good-cls-score.
        'SI'   => [ 3400, 5800 ], // https://developer.chrome.com/docs/lighthouse/performance/speed-index/#how-lighthouse-determines-your-speed-index-score.
        'TBT'  => [ 200, 600 ],   // https://developer.chrome.com/docs/lighthouse/performance/lighthouse-total-blocking-time/#how-lighthouse-determines-your-tbt-score.
        'TTI'  => [ 3900, 7300 ], // https://developer.chrome.com/docs/lighthouse/performance/interactive/#how-lighthouse-determines-your-tti-score.
    ];

    /**
     * Web-vitals sore threshold.
     */
    const SCORE_THRESHOLD = [ 90, 50 ];

    /**
     * Millisecond metrics.
     */
    const MS_METRICS = [
        'INP',
        'FID',
        'TBT',
    ];

    /**
     * Second metrics.
     */
    const SEC_METRICS = [
        'TTFB',
        'LCP',
        'FCP',
        'SI',
        'TTI',
    ];

    /**
     * Get the rating.
     *
     * @param int|float $value    Value.
     * @param string    $threshold Threshold (or "*score*" when get for the score (accumulation of all web vitals)).
     *
     * @return string Rating: poor | needs-improvement | good
     */
    public static function get_rating( $value, $threshold ) {
        $value = (float) $value;

        $threshold          = str_replace( '_MOBILE', '', strtoupper( $threshold ) );
        $is_score_threshold = str_contains( $threshold, 'SCORE' );

        $thresholds = $is_score_threshold ? self::SCORE_THRESHOLD : self::THRESHOLDS[ $threshold ];

        if ( $is_score_threshold ? ( $value < $thresholds[1] ) : ( $value > $thresholds[1] ) ) {
            return 'poor';
        } elseif ( $is_score_threshold ? ( $value < $thresholds[0] ) : ( $value > $thresholds[0] ) ) {
            return 'needs-improvement';
        } else {
            return 'good';
        }
    }

    /**
     * Get page URL.
     *
     * @param int $page_id Page ID.
     */
    public static function get_admin_page_link( $page_id ) {
        return admin_url( 'admin.php?page=first-party-analytics' ) .
            '&scroll-to-web-vitals-block=1&plugin-selected-tab=performance&web-vitals-date=all-time&web-vitals-page=' . $page_id;
    }

    /**
     * Get web vitals.
     *
     * @param int|string $post_id_or_url      The ID of the post to get the bounce rate.
     * @param int        $post_revision_id    For which revision to get the data.
     * @param null|int   $days_range          For how many days to get the data. If null then no date range (get all-time).
     * @param bool       $get_web_vitals_list Whether to get web-vitals that are only needed for the score calculation.
     * @param bool       $include_score       Whether to include score to the results.
     * @param bool       $for_the_latest_revision Whether to get the data for the latest revision.
     * @param bool       $get_web_vitals_with_measurements Whether to get web vitals with measurements.
     *
     * @return array Array of web vital values. Array key is web-vital name, and the value includes 3 values:
     *               - Value string (or float);
     *               - Grade (can be also "none");
     *               - Raw value.
     */
    public static function get_web_vitals(
        $post_id_or_url = null,
        $days_range = null,
        $get_web_vitals_list = false,
        $include_score = false,
        $for_the_latest_revision = false,
        $get_web_vitals_with_measurements = false
    ) {
        global $wpdb;

        $table_name = Page_Visits_Finish::get_table_name();

        $where_clause = '';

        if ( $days_range ) {
            $time          = time();
            $day_today     = gmdate( 'Y-m-d', $time );
            $day_from      = 1 === $days_range ? $day_today : gmdate( 'Y-m-d', $time - DAY_IN_SECONDS * $days_range );
            $where_clause .= $wpdb->prepare( 'AND a_day BETWEEN %s AND %s', $day_from, $day_today );
        }

        // FPA revision.

        if ( $for_the_latest_revision ) {
            if ( First_Party_Analytics::$premium_enabled ) {
                $current_revision_id = \First_Party_Analytics_Premium\Revisions\Revisions::get_current_revision_id();
                $where_clause       .= ' AND ' . $wpdb->prepare( 'revision_id = %d', $current_revision_id );
            }
        } else if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $where_clause .= ' AND ' . $wpdb->prepare( 'revision_id = %d', $revision );
        }

        // Post ID / URL.

        if ( $post_id_or_url ) {
            if ( is_numeric( $post_id_or_url ) ) {
                $where_clause .= ' AND ' . $wpdb->prepare( 'post_id = %d', $post_id_or_url );
            } else {
                $where_clause .= ' AND ' . $wpdb->prepare( 'a_url = %s', $post_id_or_url );
            }
        }

        $fields = [];

        if ( $get_web_vitals_list ) {
            $fields = array_keys( self::THRESHOLDS );
        }

        if ( $include_score ) {
            $fields = array_merge( $fields, [ 'web_vitals_score' ] );
        }

        $fields = array_map( 'strtolower', $fields );

        $select_statements = [];

        $server_data = Utils::get_mysql_server_version_and_type();

        $is_mysql_supports_window_functions =
            ( $server_data['is_mariadb'] && 1 === version_compare( $server_data['version'], '10.2' ) ) ||
            ( 1 === version_compare( $server_data['version'], '8.0' ) );

        foreach ( $fields as $field ) {
            for ( $i = 0; $i < 2; $i ++ ) {
                $field .= 0 === $i ? '' : '_mobile';

                $select_statement = "
                    SELECT `count`, 1 /* Columns should match. */
                    FROM (
                      SELECT SUM({$field}_good) + SUM({$field}_poor) + SUM({$field}_needs_improvement) AS `count`
                      FROM $table_name
                      WHERE {$field}_good + {$field}_poor + {$field}_needs_improvement $where_clause
                    ) t
    
                    UNION ALL
                ";

                $select_statement .= $is_mysql_supports_window_functions ? "
                    SELECT {$field}75perc, COUNT(0)  /* Use count to get even null rows. */
                    FROM
                    (
                        SELECT
                        CASE
                            WHEN SUM({$field}_good + {$field}_poor + {$field}_needs_improvement) 
                                OVER (ORDER BY $field ASC ROWS UNBOUNDED PRECEDING) >= 
                                    SUM({$field}_good + {$field}_poor + {$field}_needs_improvement)
                                OVER () * 0.75
                            AND COALESCE(SUM({$field}_good + {$field}_poor + {$field}_needs_improvement) 
                                OVER (ORDER BY $field ASC ROWS BETWEEN UNBOUNDED PRECEDING AND 1 PRECEDING), 0) < 
                                    SUM({$field}_good + {$field}_poor + {$field}_needs_improvement)
                                OVER() * 0.75
                            THEN $field
                        END AS {$field}75perc
                        FROM $table_name
                        WHERE {$field}_good + {$field}_poor + {$field}_needs_improvement $where_clause
                    ) a WHERE {$field}75perc IS NOT NULL
                    " :
                    "        
                    SELECT $field, COUNT(0) /* Use count to get even null rows. */
                    FROM (
                        SELECT 
                            $field, 
                            @lower_$field := @upper_$field as lower, 
                            @upper_$field := @upper_$field + {$field}_good + {$field}_poor + {$field}_needs_improvement AS upper
                        FROM $table_name, (SELECT @upper_$field := 0) u, (SELECT @lower_$field := 0) l
                        WHERE {$field}_good + {$field}_poor + {$field}_needs_improvement $where_clause
                        ORDER BY $field
                    ) t
                    WHERE upper >= (
                        SELECT ROUND(SUM({$field}_good + {$field}_poor + {$field}_needs_improvement) * 0.75)
                        FROM $table_name
                        WHERE {$field}_good + {$field}_poor + {$field}_needs_improvement $where_clause
                    ) AND lower < (
                        SELECT ROUND(SUM({$field}_good + {$field}_poor + {$field}_needs_improvement) * 0.75)
                        FROM $table_name
                        WHERE {$field}_good + {$field}_poor + {$field}_needs_improvement $where_clause
                    )
                    ";

                $select_statements[] = $select_statement;
            }
        }

        $select_statements = implode( 'UNION ALL', $select_statements );

        $result_raw = $wpdb->get_col( $select_statements );

        // Convert a column of values to human-readable form: ttfb, ttfb_count, lcp, lcp_count etc.

        $values = [];
        $names  = [];
        $counts = [];

        foreach ( array_chunk( $result_raw, 4 ) as $web_vital_data_index => $web_vital_data ) {
            $field_name = $fields[ $web_vital_data_index ];

            foreach ( array_chunk( $web_vital_data, 2 ) as $i => $web_vital_device_type_data ) {
                $real_name = $field_name . ( 1 === $i ? '_mobile' : '' );

                foreach ( $web_vital_device_type_data as $i2 => $value ) {
                    if ( 1 === $i2 ) {
                        $values[ $real_name ] = $value;

                        $names[ $real_name ] =
                            in_array( strtoupper( $field_name ), self::MS_METRICS, true ) ?
                                /* translators: %s is the number of milliseconds. */
                                sprintf( __( '%s ms', 'first-party-analytics' ), round( $value ) ) :
                                ( in_array( strtoupper( $field_name ), self::SEC_METRICS, true ) ?
                                    /* translators: %s is the number of seconds. */
                                    sprintf( __( '%s s', 'first-party-analytics' ), round( (float) $value / 1000, 1 ) ) :
                                    round( $value, 2 ) );
                    } else {
                        $counts[ $real_name ] = (int) $value;
                    }
                }
            }
        }

        $data = [];

        foreach ( $values as $vital => $value ) {
            $data[ strtoupper( $vital ) ] = [
                $counts[ $vital ] ? $names[ $vital ] : '-',
                $counts[ $vital ] ? self::get_rating( $value, $vital ) : 'none',
                $value,
            ];
        }

        if ( $get_web_vitals_with_measurements ) {
            return [
                'webVitals'    => $data,
                'measurements' => [
                    'desktop' => isset( $counts['ttfb'] ) ? $counts['ttfb'] : 0,
                    'mobile'  => isset( $counts['ttfb_mobile'] ) ? $counts['ttfb_mobile'] : 0,
                ],
            ];
        } else {
            return $data;
        }
    }

    /**
     * Get web-vitals grades occurrence percent (share).
     *
     * @param int|string $post_id_or_url The ID of the post to get the bounce rate.
     * @param int        $days_range     For how many days to get the data.
     *
     * @return array Like: [ 'TTFB' => [ 'good' = 26, 'needs-improvement' => 50 ... ].
     */
    public static function get_web_vitals_grades_share( $post_id_or_url, $days_range ) {
        global $wpdb;

        $table_name = Page_Visits_Finish::get_table_name();

        $where_clause = '';

        if ( $days_range ) {
            $time          = time();
            $day_today     = gmdate( 'Y-m-d', $time );
            $day_from      = 1 === $days_range ? $day_today : gmdate( 'Y-m-d', $time - DAY_IN_SECONDS * ( $days_range - 1 ) );
            $where_clause .= ( $where_clause ? ' AND ' : 'WHERE ' ) . $wpdb->prepare( 'a_day BETWEEN %s AND %s', $day_from, $day_today );
        }

        // FPA revision.

        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $where_clause .= ( $where_clause ? ' AND ' : 'WHERE ' ) . $wpdb->prepare( 'revision_id = %d', $revision );
        }

        // Post ID / URL.

        if ( $post_id_or_url ) {
            if ( is_numeric( $post_id_or_url ) ) {
                $where_clause .= ( $where_clause ? ' AND ' : 'WHERE ' ) . $wpdb->prepare( 'post_id = %d', $post_id_or_url );
            } else {
                $where_clause .= ( $where_clause ? ' AND ' : 'WHERE ' ) . $wpdb->prepare( 'a_url = %s', $post_id_or_url );
            }
        }

        $select_query_part = '';

        $web_vitals = array_map(
            'strtolower',
            array_merge( array_keys( self::THRESHOLDS ), [ 'web_vitals_score' ] )
        );

        foreach ( $web_vitals as $vital ) {
            for ( $i = 0; $i < 2; $i ++ ) {
                $vital .= 0 === $i ? '' : '_mobile';

                $select_query_part .= "{$vital}_good,";
                $select_query_part .= "{$vital}_poor,";
                $select_query_part .= "{$vital}_needs_improvement,";
            }
        }

        $select_query_part = rtrim( $select_query_part, ',' );

        $rows = $wpdb->get_results(
            "
                SELECT
                    $select_query_part
                FROM $table_name
                $where_clause;
            ",
            ARRAY_A
        );

        /**
         * Counts of the particular web vital grade.
         */
        $counts = [];

        foreach ( $web_vitals as $vital ) {
            for ( $i = 0; $i < 2; $i ++ ) {
                $vital .= 0 === $i ? '' : '_mobile';

                $counts[ $vital ] = [
                    'poor'              => 0,
                    'needs-improvement' => 0,
                    'good'              => 0,
                ];
            }
        }

        foreach ( $rows as $row ) {
            foreach ( $web_vitals as $vital ) {
                for ( $i = 0; $i < 2; $i ++ ) {
                    $vital .= 0 === $i ? '' : '_mobile';

                    $counts[ $vital ]['poor']              += (int) $row[ "{$vital}_poor" ];
                    $counts[ $vital ]['needs-improvement'] += (int) $row[ "{$vital}_needs_improvement" ];
                    $counts[ $vital ]['good']              += (int) $row[ "{$vital}_good" ];
                }
            }
        }

        foreach ( $counts as &$grades ) { // Convert numbers to percents.
            $total = array_sum( $grades );
            foreach ( $grades as &$grade ) {
                $grade = $total ? round( $grade / $total * 100, 1 ) : 0;
            }
        }

        return $counts;
    }

    /**
     * Prints dates selector.
     */
    public static function print_dates() {
        $url_param = 'web-vitals-date';

        $dates = [
            1          => __( 'Last day', 'first-party-analytics' ),
            7          => __( 'Last 7 days', 'first-party-analytics' ),
            30         => __( 'Last 30 days', 'first-party-analytics' ),
            'all-time' => __( 'All-time', 'first-party-analytics' ),
        ];

        ?>
        <label>
        <select id="first-party-analytics-web-vitals-date-selector">
        <?php
        $i = 0;
        foreach ( $dates as $date_slug => $date_name ) {
            ?>
            <option class="<?php echo 0 === $i ? esc_attr( 'default' ) : ''; ?>" value="<?php echo esc_attr( $date_slug ); ?>"
                <?php
                    selected( isset( $_GET[ $url_param ] ) ? $_GET[ $url_param ] : '', $date_slug ); 
                ?>
            >
            <?php echo esc_attr( $date_name ); ?>
            </option>
            <?php
            $i ++;
        }
        ?>
        </select>
        </label>
        <?php
    }

    /**
     * Print web vitals table.
     *
     * @param int  $post_id ID of the post to print.
     * @param bool $echo    Whether to echo the value.
     *
     * @return string|void
     */
    public static function get_web_vitals_table( $post_id, $echo = false ) {
        $web_vitals      = self::get_web_vitals(
            $post_id,
            null,
            true,
            true,
            true
        );
        $admin_page_link = self::get_admin_page_link( $post_id );

        ob_start();

        ?>
        <div class="first-party-analytics-web-vitals-table-container">
            <table class="first-party-analytics-web-vitals-table">
                <thead>
                <tr>
                    <th>
                        <?php esc_attr_e( 'Metric', 'first-party-analytics' ); ?>
                    </th>
                    <th>
                        <?php esc_attr_e( 'Desktop', 'first-party-analytics' ); ?>
                    </th>
                    <th>
                        <?php esc_attr_e( 'Mobile', 'first-party-analytics' ); ?>
                    </th>
                </tr>
                </thead>
                <tbody>
                <tr>
                    <td>
                        <?php esc_attr_e( 'Performance Score', 'first-party-analytics' ); ?>
                    </td>
                    <td class="first-party-analytics-web-vital" id="first-party-analytics-web-vitals-score"
                        data-grade="<?php echo esc_attr( $web_vitals['WEB_VITALS_SCORE'][1] ); ?>">
                        <a target="_blank" href="<?php echo esc_url( $admin_page_link ); ?>" class="first-party-analytics-web-vital-cell-block first-party-analytics-fpa-page-edit-link">
                            <div class="first-party-analytics-publish-box-image"></div>
                            <b><?php echo esc_html( $web_vitals['WEB_VITALS_SCORE'][0] ); ?></b>
                        </a>
                    </td>
                    <td class="first-party-analytics-web-vital" id="first-party-analytics-web-vitals-score-mobile"
                        data-grade="<?php echo esc_attr( $web_vitals['WEB_VITALS_SCORE_MOBILE'][1] ); ?>">
                        <a target="_blank" href="<?php echo esc_url( $admin_page_link ); ?>" class="first-party-analytics-web-vital-cell-block first-party-analytics-fpa-page-edit-link">
                            <div class="first-party-analytics-publish-box-image"></div>
                            <b><?php echo esc_html( $web_vitals['WEB_VITALS_SCORE_MOBILE'][0] ); ?></b>
                        </a>
                    </td>
                </tr>
                <tr>
                    <td>
                        <?php esc_attr_e( 'TTFB', 'first-party-analytics' ); ?>
                    </td>
                    <td class="first-party-analytics-web-vital" id="first-party-analytics-web-vital-ttfb"
                        data-grade="<?php echo esc_attr( $web_vitals['TTFB'][1] ); ?>">
                        <a target="_blank" href="<?php echo esc_url( $admin_page_link ); ?>" class="first-party-analytics-web-vital-cell-block first-party-analytics-fpa-page-edit-link">
                            <div class="first-party-analytics-publish-box-image"></div>
                            <b><?php echo esc_html( $web_vitals['TTFB'][0] ); ?></b>
                        </a>
                    </td>
                    <td class="first-party-analytics-web-vital" id="first-party-analytics-web-vital-ttfb-mobile"
                        data-grade="<?php echo esc_attr( $web_vitals['TTFB_MOBILE'][1] ); ?>">
                        <a target="_blank" href="<?php echo esc_url( $admin_page_link ); ?>" class="first-party-analytics-web-vital-cell-block first-party-analytics-fpa-page-edit-link">
                            <div class="first-party-analytics-publish-box-image"></div>
                            <b><?php echo esc_html( $web_vitals['TTFB_MOBILE'][0] ); ?></b>
                        </a>
                    </td>
                </tr>
                <tr>
                    <td>
                        <?php esc_attr_e( 'FCP', 'first-party-analytics' ); ?>
                    </td>
                    <td class="first-party-analytics-web-vital" id="first-party-analytics-web-vital-fcp"
                        data-grade="<?php echo esc_attr( $web_vitals['FCP'][1] ); ?>">
                        <a target="_blank" href="<?php echo esc_url( $admin_page_link ); ?>" class="first-party-analytics-web-vital-cell-block first-party-analytics-fpa-page-edit-link">
                            <div class="first-party-analytics-publish-box-image"></div>
                            <b><?php echo esc_html( $web_vitals['FCP'][0] ); ?></b>
                        </a>
                    </td>
                    <td class="first-party-analytics-web-vital" id="first-party-analytics-web-vital-fcp-mobile"
                        data-grade="<?php echo esc_attr( $web_vitals['FCP_MOBILE'][1] ); ?>">
                        <a target="_blank" href="<?php echo esc_url( $admin_page_link ); ?>" class="first-party-analytics-web-vital-cell-block first-party-analytics-fpa-page-edit-link">
                            <div class="first-party-analytics-publish-box-image"></div>
                            <b><?php echo esc_html( $web_vitals['FCP_MOBILE'][0] ); ?></b>
                        </a>
                    </td>
                </tr>
                <tr>
                    <td>
                        <?php esc_attr_e( 'LCP', 'first-party-analytics' ); ?>
                    </td>
                    <td class="first-party-analytics-web-vital" id="first-party-analytics-web-vital-lcp"
                        data-grade="<?php echo esc_attr( $web_vitals['LCP'][1] ); ?>">
                        <a target="_blank" href="<?php echo esc_url( $admin_page_link ); ?>" class="first-party-analytics-web-vital-cell-block first-party-analytics-fpa-page-edit-link">
                            <div class="first-party-analytics-publish-box-image"></div>
                            <b><?php echo esc_html( $web_vitals['LCP'][0] ); ?></b>
                        </a>
                    </td>
                    <td class="first-party-analytics-web-vital" id="first-party-analytics-web-vital-lcp-mobile"
                        data-grade="<?php echo esc_attr( $web_vitals['LCP_MOBILE'][1] ); ?>">
                        <a target="_blank" href="<?php echo esc_url( $admin_page_link ); ?>" class="first-party-analytics-web-vital-cell-block first-party-analytics-fpa-page-edit-link">
                            <div class="first-party-analytics-publish-box-image"></div>
                            <b><?php echo esc_html( $web_vitals['LCP_MOBILE'][0] ); ?></b>
                        </a>
                    </td>
                </tr>
                <tr>
                    <td>
                        <?php esc_attr_e( 'FID', 'first-party-analytics' ); ?>
                    </td>
                    <td class="first-party-analytics-web-vital" id="first-party-analytics-web-vital-fid"
                        data-grade="<?php echo esc_attr( $web_vitals['FID'][1] ); ?>">
                        <a target="_blank" href="<?php echo esc_url( $admin_page_link ); ?>" class="first-party-analytics-web-vital-cell-block first-party-analytics-fpa-page-edit-link">
                            <div class="first-party-analytics-publish-box-image"></div>
                            <b><?php echo esc_html( $web_vitals['FID'][0] ); ?></b>
                        </a>
                    </td>
                    <td class="first-party-analytics-web-vital" id="first-party-analytics-web-vital-fid-mobile"
                        data-grade="<?php echo esc_attr( $web_vitals['FID_MOBILE'][1] ); ?>">
                        <a target="_blank" href="<?php echo esc_url( $admin_page_link ); ?>" class="first-party-analytics-web-vital-cell-block first-party-analytics-fpa-page-edit-link">
                            <div class="first-party-analytics-publish-box-image"></div>
                            <b><?php echo esc_html( $web_vitals['FID_MOBILE'][0] ); ?></b>
                        </a>
                    </td>
                </tr>
                <tr>
                    <td>
                        <?php esc_attr_e( 'CLS', 'first-party-analytics' ); ?>
                    </td>
                    <td class="first-party-analytics-web-vital" id="first-party-analytics-web-vital-cls"
                        data-grade="<?php echo esc_attr( $web_vitals['CLS'][1] ); ?>">
                        <a target="_blank" href="<?php echo esc_url( $admin_page_link ); ?>" class="first-party-analytics-web-vital-cell-block first-party-analytics-fpa-page-edit-link">
                            <div class="first-party-analytics-publish-box-image"></div>
                            <b><?php echo esc_html( $web_vitals['CLS'][0] ); ?></b>
                        </a>
                    </td>
                    <td class="first-party-analytics-web-vital" id="first-party-analytics-web-vital-cls-mobile"
                        data-grade="<?php echo esc_attr( $web_vitals['CLS_MOBILE'][1] ); ?>">
                        <a target="_blank" href="<?php echo esc_url( $admin_page_link ); ?>" class="first-party-analytics-web-vital-cell-block first-party-analytics-fpa-page-edit-link">
                            <div class="first-party-analytics-publish-box-image"></div>
                            <b><?php echo esc_html( $web_vitals['CLS_MOBILE'][0] ); ?></b>
                        </a>
                    </td>
                </tr>
                <tr>
                    <td>
                        <?php esc_attr_e( 'INP', 'first-party-analytics' ); ?>
                    </td>
                    <td class="first-party-analytics-web-vital" id="first-party-analytics-web-vital-inp"
                        data-grade="<?php echo esc_attr( $web_vitals['INP'][1] ); ?>">
                        <a target="_blank" href="<?php echo esc_url( $admin_page_link ); ?>" class="first-party-analytics-web-vital-cell-block first-party-analytics-fpa-page-edit-link">
                            <div class="first-party-analytics-publish-box-image"></div>
                            <b><?php echo esc_html( $web_vitals['INP'][0] ); ?></b>
                        </a>
                    </td>
                    <td class="first-party-analytics-web-vital" id="first-party-analytics-web-vital-inp-mobile"
                        data-grade="<?php echo esc_attr( $web_vitals['INP_MOBILE'][1] ); ?>">
                        <a target="_blank" href="<?php echo esc_url( $admin_page_link ); ?>" class="first-party-analytics-web-vital-cell-block first-party-analytics-fpa-page-edit-link">
                            <div class="first-party-analytics-publish-box-image"></div>
                            <b><?php echo esc_html( $web_vitals['INP_MOBILE'][0] ); ?></b>
                        </a>
                    </td>
                </tr>
                <tr>
                    <td>
                        <?php esc_attr_e( 'SI', 'first-party-analytics' ); ?>
                    </td>
                    <td class="first-party-analytics-web-vital" id="first-party-analytics-web-vital-si"
                        data-grade="<?php echo esc_attr( $web_vitals['SI'][1] ); ?>">
                        <a target="_blank" href="<?php echo esc_url( $admin_page_link ); ?>" class="first-party-analytics-web-vital-cell-block first-party-analytics-fpa-page-edit-link">
                            <div class="first-party-analytics-publish-box-image"></div>
                            <b><?php echo esc_html( $web_vitals['SI'][0] ); ?></b>
                        </a>
                    </td>
                    <td class="first-party-analytics-web-vital" id="first-party-analytics-web-vital-si-mobile"
                        data-grade="<?php echo esc_attr( $web_vitals['SI_MOBILE'][1] ); ?>">
                        <a target="_blank" href="<?php echo esc_url( $admin_page_link ); ?>" class="first-party-analytics-web-vital-cell-block first-party-analytics-fpa-page-edit-link">
                            <div class="first-party-analytics-publish-box-image"></div>
                            <b><?php echo esc_html( $web_vitals['SI_MOBILE'][0] ); ?></b>
                        </a>
                    </td>
                </tr>
                <tr>
                    <td>
                        <?php esc_attr_e( 'TBT', 'first-party-analytics' ); ?>
                    </td>
                    <td class="first-party-analytics-web-vital" id="first-party-analytics-web-vital-tbt"
                        data-grade="<?php echo esc_attr( $web_vitals['TBT'][1] ); ?>">
                        <a target="_blank" href="<?php echo esc_url( $admin_page_link ); ?>" class="first-party-analytics-web-vital-cell-block first-party-analytics-fpa-page-edit-link">
                            <div class="first-party-analytics-publish-box-image"></div>
                            <b><?php echo esc_html( $web_vitals['TBT'][0] ); ?></b>
                        </a>
                    </td>
                    <td class="first-party-analytics-web-vital" id="first-party-analytics-web-vital-tbt-mobile"
                        data-grade="<?php echo esc_attr( $web_vitals['TBT_MOBILE'][1] ); ?>">
                        <a target="_blank" href="<?php echo esc_url( $admin_page_link ); ?>" class="first-party-analytics-web-vital-cell-block first-party-analytics-fpa-page-edit-link">
                            <div class="first-party-analytics-publish-box-image"></div>
                            <b><?php echo esc_html( $web_vitals['TBT_MOBILE'][0] ); ?></b>
                        </a>
                    </td>
                </tr>
                <tr>
                    <td>
                        <?php esc_attr_e( 'TTI', 'first-party-analytics' ); ?>
                    </td>
                    <td class="first-party-analytics-web-vital" id="first-party-analytics-web-vital-tti"
                        data-grade="<?php echo esc_attr( $web_vitals['TTI'][1] ); ?>">
                        <a target="_blank" href="<?php echo esc_url( $admin_page_link ); ?>" class="first-party-analytics-web-vital-cell-block first-party-analytics-fpa-page-edit-link">
                            <div class="first-party-analytics-publish-box-image"></div>
                            <b><?php echo esc_html( $web_vitals['TTI'][0] ); ?></b>
                        </a>
                    </td>
                    <td class="first-party-analytics-web-vital" id="first-party-analytics-web-vital-tti-mobile"
                        data-grade="<?php echo esc_attr( $web_vitals['TTI_MOBILE'][1] ); ?>">
                        <a target="_blank" href="<?php echo esc_url( $admin_page_link ); ?>" class="first-party-analytics-web-vital-cell-block first-party-analytics-fpa-page-edit-link">
                            <div class="first-party-analytics-publish-box-image"></div>
                            <b><?php echo esc_html( $web_vitals['TTI_MOBILE'][0] ); ?></b>
                        </a>
                    </td>
                </tr>
                </tbody>
            </table>
        </div>
        <?php

        $html = ob_get_clean();

        if ( $echo ) {
            echo $html;
        } else {
            return $html;
        }
    }
}
