<?php
/**
 * A class for analytics data sharing with the BE server.
 *
 * Sends a new data set once every 24h.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\Data_Sharing;

use Exception;
use First_Party_Analytics\DB;

/**
 * Class Data_Sharing.
 */
final class Data_Sharing {

    /**
     * Token request endpoint.
     */
    const DATA_SHARING_DOMAIN = 'https://share.firstpartyanalytics.nl/';

    /**
     * Upload datas (file) endpoint.
     */
    const UPLOAD_DATA_ENDPOINT = '/api/upload_datas';

    /**
     * Upload datas (CSV) endpoint.
     */
    const UPLOAD_JSON_DATA_ENDPOINT = '/api/upload_datas_json';

    /**
     * Interval hook name.
     */
    const INTERVAL_HOOK_NAME = 'first_party_analytics_data_sharing';

    /**
     * Interval.
     */
    const INTERVAL = HOUR_IN_SECONDS * 24;

    /**
     * Server request timeout.
     */
    const REQUEST_TIMEOUT = 60;

    /**
     * Constructor.
     */
    public function __construct() {
        if ( Token::get_token() ) { // Start the interval only after the token is retrieved because sending the data without the token will not work.
            // Schedule the action.
            add_action( 'init', [ $this, 'schedule_interval' ] );
        }

        // Un-schedule interval action on plugin deactivation.
        register_deactivation_hook( FIRST_PARTY_ANALYTICS_FILE, [ $this, 'unschedule_interval' ] );

        // Process action.
        add_action( self::INTERVAL_HOOK_NAME, [ $this, 'share_the_data' ] );
    }

    /**
     * Get the callback URL.
     *
     * @param string $endpoint_name The name of the endpoint.
     *
     * @return string Endpoint URL.
     */
    public static function get_endpoint_url( $endpoint_name ) {
        $domain = untrailingslashit(
            isset( $_ENV['FIRST_PARTY_ANALYTICS_DATA_SHARING_DOMAIN'] ) ?
                $_ENV['FIRST_PARTY_ANALYTICS_DATA_SHARING_DOMAIN'] : self::DATA_SHARING_DOMAIN
        );
        return $domain . $endpoint_name;
    }

    /**
     * Schedules the interval action if it isn't scheduled.
     *
     * @throws Exception Exception.
     */
    public static function schedule_interval() {
        if ( ! as_next_scheduled_action( self::INTERVAL_HOOK_NAME ) ) {
            as_schedule_recurring_action(
                time(),
                self::INTERVAL,
                self::INTERVAL_HOOK_NAME
            );
        }
    }

    /**
     * Plugin deactivation handler.
     */
    public static function unschedule_interval() {
        as_unschedule_all_actions( self::INTERVAL_HOOK_NAME );
    }

    /**
     * Shares the data with the BE server.
     */
    public function share_the_data() {
        global $wpdb;

        $tables_not_to_export = [
            'sessions',
            'visitors',
        ];

        $tables_prefix = DB::get_tables_prefix();

        $tables = array_filter(
            DB::get_tables(),
            function( $v ) use ( $tables_not_to_export ) {
                return ! in_array( $v, $tables_not_to_export, true );
            }
        );

        foreach ( $tables as $table ) {
            $table_name = $tables_prefix . $table;

            $temp_file_name = tempnam( sys_get_temp_dir(), 'fpa_' . $table );
            $fp = fopen( $temp_file_name, 'r+' ); 

            fputcsv( $fp, $wpdb->get_col( "SHOW COLUMNS FROM $table_name;" ) ); 
            foreach ( $wpdb->get_results( "SELECT * FROM $table_name", ARRAY_A ) as $row ) { 
                fputcsv( $fp, $row );
            }

            fseek( $fp, 0 );

            $sending_table_name = 'wp_' . DB::TABLES_PREFIX . $table;

            if ( function_exists( 'curl_version' ) ) {
                self::send_as_multipart_file(
                    $sending_table_name,
                    $temp_file_name,
                    $fp
                );
            } else {
                self::send_as_json(
                    $sending_table_name,
                    $temp_file_name,
                    $fp
                );
            }
        }
    }

    /**
     * Sends the data as JSON.
     *
     * @param string   $table_name Table name.
     * @param string   $filename   The name of the file to send.
     * @param resource $fp         File pointer.
     */
    public static function send_as_multipart_file( $table_name, $filename, $fp ) {
        $curl = curl_init(); 

        curl_setopt_array( 
            $curl,
            [
                CURLOPT_URL            => self::get_endpoint_url( self::UPLOAD_DATA_ENDPOINT ),
                CURLOPT_RETURNTRANSFER => 1,
                CURLOPT_TIMEOUT        => self::REQUEST_TIMEOUT,
                CURLOPT_CUSTOMREQUEST  => 'POST',
                CURLOPT_POST           => 1,
                CURLOPT_POSTFIELDS     => [
                    'table' => $table_name,
                    'file'  => new \CURLFile( $filename, 'text/csv', $table_name . '.csv' ),
                ],
                CURLOPT_SAFE_UPLOAD    => true,
                CURLOPT_HTTPHEADER     => [
                    'Content-Type: multipart/form-data',
                    'X-AUTH-TOKEN: ' . Token::get_token(),
                ],
            ]
        );

        curl_exec( $curl ); 

        fclose( $fp ); 
    }

    /**
     * Sends the data as JSON.
     *
     * @param string   $table_name Table name.
     * @param string   $filename   The name of the file to send.
     * @param resource $fp         File pointer.
     */
    public static function send_as_json( $table_name, $filename, $fp ) {
        $params = [
            'csv'   => fread( $fp, filesize( $filename ) ), 
            'table' => $table_name,
        ];

        fclose( $fp ); 

        $headers = [
            'Content-Type' => 'application/ld+json',
            'X-AUTH-TOKEN' => Token::get_token(),
        ];

        wp_remote_post(
            self::get_endpoint_url( self::UPLOAD_JSON_DATA_ENDPOINT ),
            [
                'headers' => $headers,
                'body'    => wp_json_encode( $params ),
                'timeout' => self::REQUEST_TIMEOUT,
            ]
        );
    }
}
