<?php
/**
 * A class to fetch recommendations from the server.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\Data_Sharing;

use Exception;
use First_Party_Analytics\First_Party_Analytics;

/**
 * Class Recommendations_Fetch.
 */
final class Recommendations_Fetch {

    /**
     * The endpoint to get recommendations.
     */
    const ENDPOINT = '/api/recommendations';

    /**
     * Interval hook name.
     */
    const INTERVAL_HOOK_NAME = 'first_party_analytics_fetch_recommendations';

    /**
     * Interval.
     */
    const INTERVAL = HOUR_IN_SECONDS * 4;

    /**
     * Server request timeout.
     */
    const REQUEST_TIMEOUT = 60;

    /**
     * Constructor.
     */
    public function __construct() {
        if ( Token::get_token() ) { // Start the interval only after the token is retrieved because sending the data without the token will not work.
            // Schedule the action.
            add_action( 'init', [ $this, 'schedule_interval' ] );
        }

        // Un-schedule interval action on plugin deactivation.
        register_deactivation_hook( FIRST_PARTY_ANALYTICS_FILE, [ $this, 'unschedule_interval' ] );

        // Process action.
        add_action( self::INTERVAL_HOOK_NAME, [ $this, 'get_the_data' ] );
    }

    /**
     * Schedules the interval action if it isn't scheduled.
     *
     * @throws Exception Exception.
     */
    public static function schedule_interval() {
        if ( ! as_next_scheduled_action( self::INTERVAL_HOOK_NAME ) ) {
            as_schedule_recurring_action(
                time(),
                self::INTERVAL,
                self::INTERVAL_HOOK_NAME
            );
        }
    }

    /**
     * Plugin deactivation handler.
     */
    public static function unschedule_interval() {
        as_unschedule_all_actions( self::INTERVAL_HOOK_NAME );
    }

    /**
     * Get the recommendations.
     */
    public function get_the_data() {
        $headers = [
            'Accept'       => 'application/json',
            'X-AUTH-TOKEN' => Token::get_token(),
        ];

        $response = wp_remote_get(
            Data_Sharing::get_endpoint_url( self::ENDPOINT ),
            [
                'headers' => $headers,
                'timeout' => self::REQUEST_TIMEOUT,
            ]
        );

        if ( ! is_wp_error( $response ) && 200 === $response['response']['code'] ) {
            $recommendations = json_decode( $response['body'], true );

            $last_saved_recommendation_id     = (int) First_Party_Analytics::$options->get( 'last-saved-recommendation-id' );
            $last_saved_recommendation_id_new = 0;

            $new_recommendations_to_add = [];

            foreach ( $recommendations as $recommendation ) {
                $recommendation_id = (int) $recommendation['id'];
                if ( $recommendation_id > $last_saved_recommendation_id ) {
                    $last_saved_recommendation_id_new = max( $last_saved_recommendation_id_new, $recommendation_id );
                    unset( $recommendation['id'] );
                    $new_recommendations_to_add[ $recommendation_id ] = $recommendation;
                }
            }

            if ( $new_recommendations_to_add ) {
                $recommendations_list = First_Party_Analytics::$options->get( 'first-party-analytics-recommendations' );

                First_Party_Analytics::$options->set( 'last-saved-recommendation-id', $last_saved_recommendation_id_new );
                First_Party_Analytics::$options->set( 'first-party-analytics-recommendations', $recommendations_list + $new_recommendations_to_add );
            }
        }
    }
}
