<?php
/**
 * A class for data sharing token retrieval.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\Data_Sharing;

use First_Party_Analytics\First_Party_Analytics;

/**
 * Class Token.
 */
final class Token {

    /**
     * Token option name.
     */
    const OPTION_NAME = 'data-sharing-token';

    /**
     * Transient name when the token was requested.
     */
    const TOKEN_REQUESTED_TRANSIENT_NAME = 'first-party-analytics-token-requested';

    /**
     * Token request transient expiration.
     */
    const TOKEN_REQUESTED_TRANSIENT_EXPIRATION = MINUTE_IN_SECONDS;

    /**
     * Server request timeout.
     */
    const REQUEST_TIMEOUT = 30;

    /**
     * Token request endpoint.
     */
    const TOKEN_REQUEST_ENDPOINT = '/api/tokens';

    /**
     * Constructor.
     */
    public function __construct() {
        if (
            ! $this->get_token() &&
            ! get_transient( self::TOKEN_REQUESTED_TRANSIENT_NAME )
        ) {
            $this->request_token();
        }
    }

    /**
     * Get the token.
     *
     * @return string Token.
     */
    public static function get_token() {
        return First_Party_Analytics::$options->get( self::OPTION_NAME );
    }

    /**
     * Get the callback URL.
     *
     * Adds the URL params to the domain.
     *
     * @return string The callback URL.
     */
    private static function get_callback_url() {
        $domain = isset( $_ENV['FIRST_PARTY_ANALYTICS_DATA_SHARING_SEND_TOKEN_TO'] ) ?
            untrailingslashit( $_ENV['FIRST_PARTY_ANALYTICS_DATA_SHARING_SEND_TOKEN_TO'] ) : get_home_url();
        return strtok( $domain, '?' ) . '?plugin=first-party-analytics&action=get_data_sharing_token';
    }

    /**
     * Request the token from the BE.
     */
    public static function request_token() {
        set_transient( self::TOKEN_REQUESTED_TRANSIENT_NAME, true, self::TOKEN_REQUESTED_TRANSIENT_EXPIRATION );

        $params = [
            'callBackUrl' => self::get_callback_url(),
        ];

        $headers = [
            'Content-Type' => 'application/ld+json',
            'Accept'       => 'application/ld+json',
        ];

        wp_remote_post(
            Data_Sharing::get_endpoint_url( self::TOKEN_REQUEST_ENDPOINT ),
            [
                'headers' => $headers,
                'body'    => wp_json_encode( $params ),
                'timeout' => self::REQUEST_TIMEOUT,
            ]
        );
    }
}
