<?php
/**
 * A class for Page_Visits_Finish database table management.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\Databases;

use First_Party_Analytics\Average_Math;
use First_Party_Analytics\Unique_Visits;
use First_Party_Analytics\Utils;
use First_Party_Analytics\Web_Vitals;
use First_Party_Analytics\WooCommerce\Products_Tracking_Data;
use First_Party_Analytics_Premium\Revision_Specific_Data;

/**
 * Class Page_Visits_Finish.
 */
final class Page_Visits_Finish extends Database {

    /**
     * Table name.
     */
    const NAME = 'page_visits_on_finish';

    /**
     * Meta name of post requests sequence.
     */
    const REQUESTS_ENQUEUE_POST_META_NAME = 'first_party_analytics_' . self::NAME;

    /**
     * How many requests to retain per post maximum.
     */
    const POST_RETAIN_MAX_REQUESTS = 1000;

    /**
     * Converts requests sequence into DB population.
     *
     * @param array $requests_sequence     Requests sequence.
     * @param array $all_requests_sequence Requests sequence, with even parsed fields. Used for percentiles.
     */
    public static function convert_sequence_into_db_population( $requests_sequence, $all_requests_sequence ) {
        $all_load_times = array_map( 'floatval', array_column( $all_requests_sequence, 'loadTime' ) );
        $all_view_times = array_map( 'floatval', array_column( $all_requests_sequence, 'viewTime' ) );
        foreach ( $requests_sequence as $request ) {
            if (
                ! array_key_exists( 'bfCachePresent', $request ) || // Do not process data from the old (incompatible) versions.
                // Do not record incomplete data.
                ! isset( $request['post_id'] ) ||
                ! $request['post_id']
            ) {
                continue;
            }

            $day                        = gmdate( 'Y-m-d', strtotime( $request['time'] ) );
            $url                        = $request['url'];
            $post_id                    = $request['post_id'];
            $post_revision_id           = $request['postRevisionId'];
            $revision_id                = (int) $request['revisionId'];
            $load_time                  = (float) $request['loadTime'];
            $view_time                  = (float) $request['viewTime'];
            $asset_less_page_size       = (int) $request['assetLessPageSize'];
            $website_data_download_size = (int) $request['websiteDataDownloadSize'];

            // Do not convert. The values below because they potentially can be null (i.e. '') and therefore not written to the DB.

            $web_vitals = [
                'ttfb'             => $request['TTFB'],
                'fcp'              => $request['FCP'],
                'lcp'              => $request['LCP'],
                'fid'              => $request['FID'],
                'cls'              => $request['CLS'],
                'inp'              => $request['INP'],
                'si'               => $request['SI'],
                'tbt'              => $request['TBT'],
                'tti'              => $request['TTI'],
                'web_vitals_score' => $request['webVitalsScore'],
            ];

            $viewport_completed_loading_time = $request['viewportCompletedLoadingTime'];
            $is_mobile_device                = $request['isMobileDevice'];
            $bf_cache_present                = $request['bfCachePresent'];

            self::write(
                $day,
                $url,
                $post_id,
                $revision_id,
                $post_revision_id,
                $load_time,
                $view_time,
                $all_load_times,
                $all_view_times,
                $asset_less_page_size,
                $website_data_download_size,
                $web_vitals,
                $viewport_completed_loading_time,
                $is_mobile_device,
                $bf_cache_present
            );

            // Visitor details.

            if ( Unique_Visits::is_request_unique( $request, 'visitor-details-on-visit-finish', 'visitor-details' ) ) {
                Visitors::save_for_the_request( $request, 'visitor-details-on-visit-finish', 'visitor-details' );

                $visitor_details = [
                    'adblockActive',
                    'analyticsBlocked',
                ];

                foreach ( $visitor_details as $visitor_detail ) {
                    if ( isset( $request[ $visitor_detail ] ) ) {
                        Visitor_Details::write( $day, $visitor_detail, $request[ $visitor_detail ] );
                    }
                }
            }

            // Viewed products.

            if ( isset( $request['viewedProducts'] ) ) {
                foreach ( $request['viewedProducts'] as $viewed_product ) {
                    Revision_Specific_Data::post_meta_increment_counter( (int) $viewed_product, Products_Tracking_Data::VIEWED_IN_CATALOG_COUNT_TRACKER_META_KEY );
                }
            }
        }
    }

    /**
     * Writes to the DB.
     *
     * @param string $day                  Day.
     * @param string $url                  Visited URL.
     * @param int    $post_id              ID of the post.
     * @param int    $revision_id          Revision ID.
     * @param int    $post_revision_id     ID of the post revision.
     * @param float  $load_time            Page load time.
     * @param float  $view_time            Page view time.
     * @param array  $all_load_times       All load times for the current sequence block.
     * @param array  $all_view_times       All view times for the current sequence block.
     * @param int    $asset_less_page_size Asset-less page size (in bytes).
     * @param int    $website_data_download_size Website data download size (without assets of external resources).
     * @param array  $web_vitals               The list of web vitals.
     * @param string $viewport_completed_loading_time Loading time for the viewport.
     * @param bool   $is_mobile_device                Whether the request came from the mobile device (used for web vitals and their scores recording).
     * @param bool   $bf_cache_present                If BF cache is present.
     */
    public static function write(
        $day,
        $url,
        $post_id,
        $revision_id,
        $post_revision_id,
        $load_time,
        $view_time,
        $all_load_times,
        $all_view_times,
        $asset_less_page_size,
        $website_data_download_size,
        $web_vitals,
        $viewport_completed_loading_time,
        $is_mobile_device,
        $bf_cache_present
    ) {
        global $wpdb;

        $table_name = self::get_table_name();

        // 1. Add / update the row.

        $query = $wpdb->prepare( // Update "total_visits" at the end as it's more convenient for me to do incremental averaging with the previous total.
            "
            INSERT INTO $table_name 
                (a_day, a_url, post_id, total_visits, revision_id, avg_asset_less_page_size, avg_website_data_download_size, post_revision_id)
            VALUES (%s, %s, %d, 1, %d, %d, %d, %d) ON DUPLICATE KEY UPDATE

            avg_asset_less_page_size = (avg_asset_less_page_size * total_visits + VALUES(avg_asset_less_page_size)) / (total_visits + 1),
            avg_website_data_download_size = (avg_website_data_download_size * total_visits + VALUES(avg_website_data_download_size)) / (total_visits + 1),
            total_visits  = total_visits + 1,
            a_url = VALUES(a_url); /* In case the URL changes. */
        ",
            $day,
            $url,
            $post_id,
            $revision_id,
            $asset_less_page_size,
            $website_data_download_size,
            $post_revision_id
        );

        $wpdb->query( $query ); 

        // 2. Write values which pass the percentiles.

        // 2.1. Load time.

        $is_load_time_in_percentile        = in_array(
            $load_time,
            Average_Math::filter_out_by_percentile( $all_load_times ),
            true
        );
        $is_load_time_not_in_10_percentile = ! in_array(
            $load_time,
            Average_Math::filter_out_by_percentile( $all_load_times, 10 ),
            true
        );

        if (
            $is_load_time_in_percentile &&
            $is_load_time_not_in_10_percentile
        ) {
            $query = $wpdb->prepare(
                "
                    UPDATE $table_name
                    SET
                        avg_load_time = (avg_load_time * load_times_count + %f) / (load_times_count + 1),
                        max_load_time = GREATEST(%f, max_load_time),
                        min_load_time = LEAST(%f, min_load_time),
                        load_times_count = load_times_count + 1
                    WHERE
                        a_day = %s AND
                        post_id = %d AND
                        revision_id = %d AND
                        post_revision_id = %d
                 ",
                $load_time,
                $load_time,
                $load_time,
                $day,
                $post_id,
                $revision_id,
                $post_revision_id
            );

            $wpdb->query( $query ); ;
        }

        // 2.2. View time.

        $is_view_time_in_percentile        = in_array(
            $view_time,
            Average_Math::filter_out_by_percentile( $all_view_times ),
            true
        );
        $is_view_time_not_in_10_percentile = ! in_array(
            $view_time,
            Average_Math::filter_out_by_percentile( $all_view_times, 10 ),
            true
        );

        if (
            $is_view_time_in_percentile &&
            $is_view_time_not_in_10_percentile
        ) {
            $query = $wpdb->prepare(
                "
                    UPDATE $table_name
                    SET
                        avg_view_time = (avg_view_time * view_times_count + %f) / (view_times_count + 1),
                        view_times_count = view_times_count + 1
                    WHERE
                        a_day = %s AND
                        post_id = %d AND
                        revision_id = %d AND
                        post_revision_id = %d
                 ",
                $view_time,
                $day,
                $post_id,
                $revision_id,
                $post_revision_id
            );

            $wpdb->query( $query ); ;
        }

        // 3. Add optional values (which are not null (i.e. empty string)).

        // 3.1. Viewport completed loading time.

        if ( null !== $viewport_completed_loading_time ) {
            $query = $wpdb->prepare(
                "
                    UPDATE $table_name
                    SET
                        avg_viewport_completed_loading_time = 
                            (avg_viewport_completed_loading_time * viewport_completed_loading_time_count + %f) / (viewport_completed_loading_time_count + 1), 
                        viewport_completed_loading_time_count = viewport_completed_loading_time_count + 1
                    WHERE
                        a_day = %s AND
                        post_id = %d AND
                        revision_id = %d AND
                        post_revision_id = %d
                 ",
                $viewport_completed_loading_time,
                $day,
                $post_id,
                $revision_id,
                $post_revision_id
            );

            $wpdb->query( $query ); 
        }

        /**
         * 3.2. Web vitals.
         *
         * Get & update post requests enqueue.
         */

        foreach ( $web_vitals as $web_vital_name => $web_vital_value ) {
            $web_vital_name .= $is_mobile_device ? '_mobile' : '';

            if ( null !== $web_vital_value ) {

                // Save the value to the meta.

                $meta_name = self::REQUESTS_ENQUEUE_POST_META_NAME . '_' . $web_vital_name;

                $post_requests_enqueue = get_post_meta( $post_id, $meta_name, true );
                if ( ! $post_requests_enqueue ) {
                    $post_requests_enqueue = [];
                }

                $filtered_out_post_requests_enqueue = [
                    [
                        'day' => $day,
                        'val' => $web_vital_value,
                    ],
                ];

                $i = 1;
                foreach ( $post_requests_enqueue as $post_request ) {
                    if ( $i <= 10 || ( $post_request['day'] === $day && $i <= 1000 ) ) { // Retain 10 anyway. But if for the current day, then up to 1k.
                        $filtered_out_post_requests_enqueue[] = $post_request;
                    } else {
                        break;
                    }
                    $i ++;
                }

                $post_requests_count_for_the_current_day = count(
                    array_filter(
                        $filtered_out_post_requests_enqueue,
                        function ( $post_request ) use ( $day ) {
                            return $post_request['day'] === $day;
                        }
                    )
                );

                $are_all_posts_in_the_enqueue_for_the_current_day = count( $filtered_out_post_requests_enqueue ) === $post_requests_count_for_the_current_day;

                $post_requests_enqueue = $filtered_out_post_requests_enqueue;

                update_post_meta( $post_id, $meta_name, $post_requests_enqueue );

                // Get percentile and save the value.

                $percentiled_value = Average_Math::get_percentile( array_column( $post_requests_enqueue, 'val' ), Web_Vitals::PERCENTILE );
                $grade             = Web_Vitals::get_rating( $percentiled_value, $web_vital_name );

                /**
                 * When doing percentiles with considering the previous day(s), also add to the buckets of the
                 * requests that took the part in the percentiles calculation for previous day(s); and when 10 are reached,
                 * reset the buckets. So it records percentile and the buckets for the same data.
                 */

                if ( $are_all_posts_in_the_enqueue_for_the_current_day ) {
                    $good_query              = $wpdb->prepare( "{$web_vital_name}_good + %d", 'good' === $grade ); 
                    $needs_improvement_query = $wpdb->prepare( "{$web_vital_name}_needs_improvement + %d", 'needs-improvement' === $grade ); 
                    $poor_query              = $wpdb->prepare( "{$web_vital_name}_poor + %d", 'poor' === $grade ); 
                } else {
                    $good_query              = 0;
                    $needs_improvement_query = 0;
                    $poor_query              = 0;

                    foreach ( array_column( $post_requests_enqueue, 'val' ) as $a_val ) {
                        $a_grade = Web_Vitals::get_rating( $a_val, $web_vital_name );

                        if ( 'good' === $a_grade ) {
                            $good_query ++;
                        } elseif ( 'needs-improvement' === $a_grade ) {
                            $needs_improvement_query ++;
                        } else {
                            $poor_query ++;
                        }
                    }
                }

                $query = $wpdb->prepare(
                    "
                    UPDATE $table_name
                    SET
                        $web_vital_name                     = %f,
                        {$web_vital_name}_good              = $good_query,
                        {$web_vital_name}_needs_improvement = $needs_improvement_query,
                        {$web_vital_name}_poor              = $poor_query
                    WHERE
                        a_day = %s AND
                        post_id = %d AND
                        revision_id = %d AND
                        post_revision_id = %d
                    ",
                    $percentiled_value,
                    $day,
                    $post_id,
                    $revision_id,
                    $post_revision_id
                );

                $wpdb->query( $query ); 
            }
        }

        // 3.3. BF cache.

        if ( null !== $bf_cache_present ) {
            $col_to_up = $bf_cache_present ? 'bf_cache_on' : 'bf_cache_off';

            $query = $wpdb->prepare(
                "
                    UPDATE $table_name
                    SET
                        $col_to_up = $col_to_up + 1
                    WHERE
                        a_day = %s AND
                        post_id = %d AND
                        revision_id = %d AND
                        post_revision_id = %d
                    ",
                $day,
                $post_id,
                $revision_id,
                $post_revision_id
            );

            $wpdb->query( $query ); 
        }
    }

    /**
     * Get bounce rate.
     *
     * @param int      $post_id          The ID of the post to get the bounce rate.
     * @param int      $post_revision_id For which revision to get the data.
     * @param null|int $days_range       For how many days to get the data. If null then no date range (get all-time).
     *
     * @return string The size, human-formatted, or "-" if no data.
     */
    public static function get_avg_website_data_download_size( $post_id, $post_revision_id = 0, $days_range = null ) {
        global $wpdb;

        $table_name = self::get_table_name();

        // Dates.

        $days_where_clause = '';
        if ( $days_range ) {
            $time      = time();
            $day_today = gmdate( 'Y-m-d', $time );
            $day_from  = gmdate( 'Y-m-d', $time - DAY_IN_SECONDS * $days_range );

            $days_where_clause = $wpdb->prepare( 'AND a_day BETWEEN %s AND %s', $day_from, $day_today );
        }

        // FPA revision.

        $revision_where_clause = '';
        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision );
        }

        // Post revision.

        if ( ! $post_revision_id && ! isset( $revision ) ) {
            $post_revision_id = Utils::get_post_latest_revision_id( $post_id );
        }

        $post_revision_where_clause = '';
        if ( $post_revision_id ) { 
            $post_revision_where_clause = $wpdb->prepare( 'AND post_revision_id = %d', $post_revision_id );
        }

        $result = $wpdb->get_row(
            $wpdb->prepare(
                "
                SELECT ROUND(SUM(total_visits * avg_website_data_download_size) / SUM(total_visits)) FROM $table_name
                WHERE post_id = %d
                $days_where_clause
                $revision_where_clause
                $post_revision_where_clause;
            ",
                $post_id
            ),
            ARRAY_N
        );

        return ( $result && (int) $result[0] ) ? size_format( $result[0] ) : '-';
    }

    /**
     * Get bounce rate.
     *
     * @param int      $post_id          The ID of the post to get the bounce rate.
     * @param int      $post_revision_id For which revision to get the data.
     * @param null|int $days_range       For how many days to get the data. If null then no date range (get all-time).
     *
     * @return string The size, human-formatted, or "-" if no data.
     */
    public static function get_avg_asset_less_page_size( $post_id, $post_revision_id = 0, $days_range = null ) {
        global $wpdb;

        $table_name = self::get_table_name();

        // Dates.

        $days_where_clause = '';
        if ( $days_range ) {
            $time      = time();
            $day_today = gmdate( 'Y-m-d', $time );
            $day_from  = gmdate( 'Y-m-d', $time - DAY_IN_SECONDS * $days_range );

            $days_where_clause = $wpdb->prepare( 'AND a_day BETWEEN %s AND %s', $day_from, $day_today );
        }

        // FPA revision.

        $revision_where_clause = '';
        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision );
        }

        // Post revision.

        if ( ! $post_revision_id && ! isset( $revision ) ) {
            $post_revision_id = Utils::get_post_latest_revision_id( $post_id );
        }

        $post_revision_where_clause = '';
        if ( $post_revision_id ) { 
            $post_revision_where_clause = $wpdb->prepare( 'AND post_revision_id = %d', $post_revision_id );
        }

        $result = $wpdb->get_row(
            $wpdb->prepare(
                "
                SELECT ROUND(SUM(total_visits * avg_asset_less_page_size) / SUM(total_visits)) FROM $table_name
                WHERE post_id = %s
                $days_where_clause
                $revision_where_clause
                $post_revision_where_clause;
            ",
                $post_id
            ),
            ARRAY_N
        );

        return ( $result && (int) $result[0] ) ? size_format( $result[0] ) : '-';
    }
}
