<?php
/**
 * A class for Page_Visits database table management.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\Databases;

use First_Party_Analytics\First_Party_Analytics;
use First_Party_Analytics\Unique_Visits;
use WhichBrowser\Parser as User_Agent_Parser;

/**
 * Class Page_Visits.
 */
final class Page_Visits extends Database {

    /**
     * Table name.
     */
    const NAME = 'page_visits';

    /**
     * Converts requests sequence into DB population.
     *
     * @param array $requests_sequence Requests sequence.
     */
    public static function convert_sequence_into_db_population( $requests_sequence ) {
        foreach ( $requests_sequence as $request ) {
            $day         = gmdate( 'Y-m-d', strtotime( $request['time'] ) );
            $url         = trailingslashit( explode( '?', $request['url'] )[0] ); // Delete URL params.
            $post_id     = $request['post_id'];
            $post_type   = $request['postType'];
            $revision_id = (int) $request['revisionId'];
            $is_unique   = Unique_Visits::is_request_unique( $request );

            $is_bot = false;
            if ( isset( $request['userAgent'] ) ) {
                $ua        = $request['userAgent'];
                $parsed_ua = new User_Agent_Parser( $ua );
                $is_bot    = $parsed_ua->getType() === 'bot';
            }

            $is_internal = null;
            if ( isset( $request['referer'] ) && $request['referer'] ) {
                $referer_host = wp_parse_url( $request['referer'], PHP_URL_HOST );
                $site_host    = wp_parse_url( home_url(), PHP_URL_HOST );
                $is_internal  = $referer_host === $site_host;
            }

            self::write( $day, $url, $post_id, $post_type, $is_unique, $is_bot, $is_internal, $revision_id );

            // Save visitor URL to visitors table.

            Visitors::save_for_the_request( $request );
        }
    }

    /**
     * Writes to the DB.
     *
     * @param string $day         Day.
     * @param string $url         Visited URL.
     * @param string $post_id     Post ID.
     * @param string $post_type   Post type.
     * @param bool   $is_unique   If the visit is unique.
     * @param bool   $is_bot      If the visit is for bot.
     * @param bool   $is_internal If the visit is internal.
     * @param int    $revision_id Revision ID.
     */
    public static function write( $day, $url, $post_id, $post_type, $is_unique, $is_bot, $is_internal, $revision_id ) {
        global $wpdb;

        $table_name = self::get_table_name();

        $unique_visit   = (int) $is_unique;
        $bot_visit      = (int) $is_bot;
        $internal_visit = (int) ( null !== $is_internal ? $is_internal : 0 );
        $incoming_visit = (int) ( null !== $is_internal ? ! $is_internal : 0 );

        $query = $wpdb->prepare( 
            "
            INSERT INTO $table_name (a_day, a_url, post_id, post_type, unique_visits, bot_visits, total_visits, internal_visits, incoming_visits, revision_id)
            VALUES (%s, %s, %s, %s, %d, %d, 1, %d, %d, %d) ON DUPLICATE KEY UPDATE

            post_id         = GREATEST(post_id, VALUES(post_id)),
            post_type       = GREATEST(post_type, VALUES(post_type)),
            unique_visits   = unique_visits   + VALUES(unique_visits), 
            bot_visits      = bot_visits      + VALUES(bot_visits),
            total_visits    = total_visits    + 1,
            internal_visits = internal_visits + VALUES(internal_visits),
            incoming_visits = incoming_visits + VALUES(incoming_visits),
            a_url = VALUES(a_url); /* In case the URL changes. */
        ",
            $day,
            $url,
            $post_id,
            $post_type,
            $unique_visit,
            $bot_visit,
            $internal_visit,
            $incoming_visit,
            $revision_id
        );

        $wpdb->query( $query ); 
    }

    /**
     * Get the page by URL and day.
     *
     * @param string $a_url URL.
     * @param string $a_day Day.
     *
     * @return array|null Page data if it's found or false if not.
     */
    public static function get_by_url_and_day( $a_url, $a_day ) {
        global $wpdb;

        $table_name = self::get_table_name();

        $revision_where_clause = '';
        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision );
        }

        return $wpdb->get_row(
            $wpdb->prepare(
                "
                    SELECT * FROM $table_name
                    WHERE a_url = %s AND a_day = %s $revision_where_clause;
                ",
                $a_url,
                $a_day
            ),
            ARRAY_A
        );
    }

    /**
     * Get unique visits of the page ID.
     *
     * @param int|string $id ID of the page.
     *
     * @return int
     */
    public static function get_unique_visits_count_by_id( $id ) {
        global $wpdb;

        $table_name = self::get_table_name();

        $revision_where_clause = '';
        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision );
        }

        return (int) $wpdb->get_var(
            $wpdb->prepare(
                "
                    SELECT SUM(unique_visits) FROM $table_name
                    WHERE post_id = %d
                    $revision_where_clause;
                ",
                (int) $id
            )
        );
    }

    /**
     * Get post visits count.
     *
     * Searches by post ID first, then, if nothing found, searches by post URL.
     *
     * @param int      $post_id    The ID of the post to get the visits.
     * @param null|int $days_range For how many days to get the data. If null then no date range (get all-time).
     *
     * @return int Post visits count.
     */
    public static function get_post_visits_count( $post_id, $days_range = null ) {
        global $wpdb;

        $table_name = self::get_table_name();

        $days_where_clause = '';
        if ( $days_range ) {
            $time      = time();
            $day_today = gmdate( 'Y-m-d', $time );
            $day_from  = gmdate( 'Y-m-d', $time - DAY_IN_SECONDS * $days_range );

            $days_where_clause = $wpdb->prepare( 'AND a_day BETWEEN %s AND %s', $day_from, $day_today );
        }

        $revision_where_clause = '';
        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision );
        }

        $result = $wpdb->get_row(
            $wpdb->prepare(
                "
                    SELECT SUM(unique_visits) FROM $table_name
                    WHERE post_id = %s
                    $days_where_clause
                    $revision_where_clause;
                ",
                $post_id
            ),
            ARRAY_N
        );

        return (int) $result[0];
    }

    /**
     * Get post popularity ranking.
     *
     * @param int      $post_id         The ID of the post to get the visits.
     * @param null|int $days_range For how many days to get the data. If null then no date range (get all-time).
     *
     * @return string|int
     */
    public static function get_popularity_ranking( $post_id, $days_range = null ) {
        global $wpdb;

        $post_id    = (int) $post_id;
        $days_range = (int) $days_range;

        $table_name = self::get_table_name();

        $where_clause = '';
        if ( $days_range ) {
            $time      = time();
            $day_today = gmdate( 'Y-m-d', $time );
            $day_from  = gmdate( 'Y-m-d', $time - DAY_IN_SECONDS * $days_range );

            $where_clause .= 'WHERE ' . $wpdb->prepare( 'a_day BETWEEN %s AND %s', $day_from, $day_today );
        }

        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $where_clause .= ( $where_clause ? ' AND ' : 'WHERE ' ) . $wpdb->prepare( 'revision_id = %d', $revision );
        }

        $results = $wpdb->get_results(
            "
                SELECT SUM(unique_visits), a_url, post_id FROM $table_name
                $where_clause
                GROUP BY a_url
                ORDER BY SUM(unique_visits) DESC;
            ",
            ARRAY_A
        );

        /**
         * Try to find by post ID.
         *
         * In case can't find by post id, then try to find by URL.
         */

        $post_id_match = array_search( (string) $post_id, array_column( $results, 'post_id' ), true );

        if ( false !== $post_id_match ) {
            return $post_id_match + 1;
        } elseif ( $post_permalink = get_permalink( $post_id ) ) { 
            $url_match = array_search( $post_permalink, array_column( $results, 'a_url' ), true );
            if ( false !== $url_match ) {
                return $url_match + 1;
            }
        }

        return '-';
    }

    /**
     * Get bounce rate.
     *
     * @param int      $post_id    The ID of the post to get the bounce rate.
     * @param null|int $days_range For how many days to get the data. If null then no date range (get all-time).
     *
     * @return float|string Bounce rate or "-" if 0 sessions for this page.
     */
    public static function get_bounce_rate( $post_id, $days_range = null ) {
        global $wpdb;

        $table_name = self::get_table_name();

        $days_where_clause = '';
        if ( $days_range ) {
            $time      = time();
            $day_today = gmdate( 'Y-m-d', $time );
            $day_from  = gmdate( 'Y-m-d', $time - DAY_IN_SECONDS * $days_range );

            $days_where_clause = $wpdb->prepare( 'AND a_day BETWEEN %s AND %s', $day_from, $day_today );
        }

        $revision_where_clause = '';
        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision );
        }

        $result = $wpdb->get_row(
            $wpdb->prepare(
                "
                    SELECT SUM(sessions), ROUND(AVG(bounce_rate) * 100) FROM $table_name
                    WHERE post_id = %s
                    $days_where_clause
                    $revision_where_clause;
                ",
                $post_id
            ),
            ARRAY_N
        );

        return ( $result && (int) $result[0] ) ? $result[1] . '%' : '-';
    }

    /**
     * Visits for all products.
     *
     * @param bool|string $post_type Post type.
     *
     * @return array Array, where key is post_id and value is number of visits.
     */
    public static function get_visits_for_all_pages( $post_type = false ) {
        global $wpdb;

        $table_name = self::get_table_name();

        $where_clause = '';
        if ( 'product' === $post_type ) {
            $where_clause = "WHERE post_type = '$post_type'";
        }

        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $where_clause .= ( $where_clause ? ' AND' : 'WHERE' ) . $wpdb->prepare( ' revision_id = %d', $revision );
        }

        $results = $wpdb->get_results(
            "
                SELECT post_id, SUM(unique_visits) FROM $table_name
                $where_clause
                GROUP BY post_id;
            ",
            ARRAY_N
        );

        $neatly_formatted_array = array_column( $results, 1, 0 );

        return array_map( 'intval', $neatly_formatted_array );
    }
}
