<?php
/**
 * A class for Search_Words database table management.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\Databases;

use First_Party_Analytics\Stop_Words;

/**
 * Class Search_Words.
 */
final class Search_Words extends Database {

    /**
     * Table name.
     */
    const NAME = 'search_words';

    /**
     * Handles the search, breaks it into words and saves each one of them.
     *
     * @param string $search_text Search text.
     * @param string $page        Page URL on which the search has been triggered.
     * @param string $selector    Selector of the search input that triggered the search.
     * @param string $time        Time of the search.
     * @param int    $revision_id Revision ID.
     */
    public static function handle_search( $search_text, $page, $selector, $time, $revision_id ) {
        $search_text = trim( $search_text );

        if ( ! $search_text ) {
            return;
        }

        self::write( $search_text, $page, $selector, true, $time, $revision_id );

        $search = preg_replace( '#[[:punct:]]#', '', $search_text ); // Remove punctuation marks.
        $words  = array_map( 'strtolower', array_map( 'trim', explode( ' ', $search ) ) );

        foreach ( $words as $word ) {
            self::write( $word, $page, $selector, false, $time, $revision_id );
        }
    }

    /**
     * Writes to the DB.
     *
     * @param string $word        A word.
     * @param string $page        Page URL on which the search has been triggered.
     * @param string $selector    Selector of the search input that triggered the search.
     * @param bool   $full_search Full search or just a word of the search.
     * @param string $time        Time of the search.
     * @param int    $revision_id Revision ID.
     */
    public static function write( $word, $page, $selector, $full_search, $time, $revision_id ) {
        global $wpdb;

        $table_name = self::get_table_name();

        $query = $wpdb->prepare( 
            "
            INSERT INTO $table_name (word, page_url, input_selector, full_search, `time`, revision_id) VALUES (%s, %s, %s, %d, %s, %d);
        ",
            $word,
            $page,
            $selector,
            (int) $full_search,
            $time,
            $revision_id
        );

        $wpdb->query( $query ); 
    }

    /**
     * Returns all search words with their count.
     *
     * @param int $limit How many words to get.
     *
     * @return array|bool Data, or false if not data.
     */
    public static function get_with_count( $limit = 100 ) {
        global $wpdb;

        static $results;
        if ( null === $results ) { // Cache.
            $table_name = self::get_table_name();

            $stop_words = Stop_Words::get_stop_words_for_site_languages();

            $placeholders = implode( ',', array_fill( 0, count( $stop_words ), '%s' ) );

            $revision_where_clause = '';
            if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
                $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
                $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision );
            }

            $results = $wpdb->get_results( 
                $wpdb->prepare( 
                    "
                        SELECT
                            word, COUNT(*) as `count`
                        FROM
                            $table_name
                        WHERE
                            word NOT IN ($placeholders)
                            $revision_where_clause
                        GROUP BY
                            word
                        ORDER BY
                            `count` DESC 
                        LIMIT %d;
                    ",
                    ...$stop_words,
                    ...[ $limit ]
                ),
                ARRAY_A
            );
        }

        if ( ! $results ) {
            return false;
        }

        return $results;
    }

    /**
     * Get search words for word cloud.
     *
     * Filters out the stop-words.
     *
     * @return array|bool Data, or false if not data.
     */
    public static function get_for_word_cloud() {
        $results = self::get_with_count();

        return $results ? $results : false;
    }
}
