<?php
/**
 * A class for Sessions database table management.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\Databases;

use First_Party_Analytics\Sessions as Sessions_Class;

/**
 * Class Sessions.
 */
final class Sessions extends Database {

    /**
     * Table name.
     */
    const NAME = 'sessions';

    /**
     * Converts requests sequence into DB population.
     *
     * @param array $requests_sequence Requests sequence.
     * @param bool  $new_page_visit    If the request is for a new page visit or not (e.g. on page visit finish).
     */
    public static function convert_sequence_into_db_population( $requests_sequence, $new_page_visit = true ) {
        foreach ( $requests_sequence as $request ) {
            $visitor_id = $request['id'];
            $time       = gmdate( 'Y-m-d H:i:s', strtotime( $request['time'] ) );

            self::write( $new_page_visit, $visitor_id, $time, $request );
        }
    }

    /**
     * Get sessions count.
     *
     * @return int
     */
    public static function get_count() {
        global $wpdb;

        static $results;
        if ( null !== $results ) { // Cache.
            return $results;
        }

        $table_name = self::get_table_name();

        $revision_where_clause = '';
        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $revision_where_clause = $wpdb->prepare( 'WHERE revision_id = %d', $revision );
        }

        $results = (int) $wpdb->get_var(
            "
                SELECT COUNT(*)
                FROM $table_name
                $revision_where_clause;
            "
        );

        return $results;
    }

    /**
     * Get session from the DB for visitor_id and visit_time.
     *
     * @param bool   $get_with_timeout If the session should be within SESSION_TIMEOUT time.
     * @param string $visitor_id       Visitor ID.
     * @param string $time             Time ID.
     * @param int    $revision_id      Revision ID.
     *
     * @return array|false
     */
    private static function get( $get_with_timeout, $visitor_id, $time, $revision_id ) {
        global $wpdb;

        $table_name = self::get_table_name();

        $revision_where_clause = '';
        if ( $revision_id ) { 
            $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision_id );
        }

        if ( $get_with_timeout ) {
            $min_time = gmdate( 'Y-m-d H:i:s', strtotime( $time ) - Sessions_Class::SESSION_TIMEOUT );

            $query = $wpdb->prepare(
                "
                    SELECT * FROM $table_name
                    WHERE visitor_id = %s AND last_time > %s $revision_where_clause
                    ORDER BY `id` DESC LIMIT 1;
                ",
                $visitor_id,
                $min_time
            );
        } else {
            $query = $wpdb->prepare(
                "
                    SELECT * FROM $table_name
                    WHERE visitor_id = %s $revision_where_clause
                    ORDER BY `id` DESC LIMIT 1;
                ",
                $visitor_id
            );
        }

        $result = $wpdb->get_row( $query, ARRAY_A ); 

        return $result ? $result : false;
    }

    /**
     * Writes to the DB.
     *
     * @param bool   $new_page_visit If the request is for a new page visit or not (e.g. on page visit finish).
     * @param string $visitor_id     Visitor ID.
     * @param string $time           Time ID.
     * @param array  $request        Request.
     */
    public static function write( $new_page_visit, $visitor_id, $time, $request ) {
        global $wpdb;

        $table_name = self::get_table_name();

        $revision_id = (int) $request['revisionId'];

        $prev_request = self::get( $new_page_visit, $visitor_id, $time, $revision_id );

        if ( $new_page_visit ) {
            if ( $prev_request ) {
                $query = $wpdb->prepare(
                    "
                        UPDATE $table_name SET
                        
                        visited_pages = visited_pages + 1,
                        bounced = 0,
                        last_time = GREATEST(%s, last_time)

                        WHERE id = %s;
                    ",
                    $time,
                    $prev_request['id']
                );
            } else {
                $query = $wpdb->prepare(
                    "
                        INSERT INTO $table_name (visitor_id, visited_pages, first_time, last_time, revision_id)
                        VALUES (%s, 1, %s, %s, %d);
                    ",
                    $visitor_id,
                    $time,
                    $time,
                    $revision_id
                );
            }
        } elseif ( $prev_request ) {
            $last_time               = strtotime( $prev_request['last_time'] );
            $seconds_since_last_time = strtotime( $time ) - $last_time;
            $time                    = gmdate(
                'Y-m-d H:i:s',
                $last_time + min( Sessions_Class::SESSION_TIMEOUT, $seconds_since_last_time )
            );

            $query = $wpdb->prepare(
                "
                    UPDATE $table_name SET

                    last_time = GREATEST(%s, last_time),
                    bounced = LEAST(bounced, %d)

                    WHERE id = %s;
                ",
                $time,
                (int) $request['bounced'],
                $prev_request['id']
            );
        }

        if ( isset( $query ) ) {
            $wpdb->query( $query ); 
        }
    }
}
