<?php
/**
 * A class for Special_Url_Params database table management.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\Databases;

use First_Party_Analytics\Utils;
use Pdp\SyntaxError;

/**
 * Class Special_Url_Params.
 */
final class Special_Url_Params extends Database {

    /**
     * Table name.
     */
    const NAME = 'special_url_params';

    /**
     * The list of URL params to record.
     *
     * Record only when at least one of those is present.
     */
    const URL_PARAMS = [
        'utm_medium',
        'utm_source',
        'utm_campaign',
        'utm_term',
        'utm_content',
    ];

    /**
     * Converts requests sequence into DB population.
     *
     * @param array $requests_sequence Requests sequence.
     */
    public static function convert_sequence_into_db_population( $requests_sequence ) {
        foreach ( $requests_sequence as $request ) {
            $day     = gmdate( 'Y-m-d', strtotime( $request['time'] ) );
            $url     = $request['url'];
            $query   = wp_parse_url( $url, PHP_URL_QUERY );
            $referer = isset( $request['referer'] ) ? $request['referer'] : '';
            if ( $query && $referer ) {
                parse_str( $query, $params );

                $params = array_filter(
                    $params,
                    function( $v, $k ) {
                        return in_array( $k, self::URL_PARAMS, true );

                    },
                    ARRAY_FILTER_USE_BOTH
                );

                if ( $params ) {
                    $missing_params = array_diff( self::URL_PARAMS, array_keys( $params ) );
                    foreach ( $missing_params as $missing_param ) {
                        $params[ $missing_param ] = '';
                    }

                    extract( $params );

                    $referer      = preg_replace( '@^http(s)?:\/\/(www\.)?@', '', $referer ); // Delete protocol and www.
                    $referer_host = untrailingslashit( explode( '/', $referer )[0] );

                    try {
                        $referer_main_domain = Utils::get_domain_data( $referer_host )['second_level_domain'];
                    } catch ( SyntaxError $e ) { // When the domain can't be parsed. For example, it's a local domain (127.0.0.1).
                        $referer_main_domain = '';
                    }

                    $revision_id = (int) $request['revisionId'];

                    self::write( $day, $referer_host, $referer_main_domain, $utm_medium, $utm_source, $utm_campaign, $utm_term, $utm_content, $revision_id );
                }
            }
        }
    }

    /**
     * Writes to the DB.
     *
     * @param string $day                 Day.
     * @param string $referer_host        Referer host (e.g. "google.com").
     * @param string $referer_main_domain Referer main domain (e.g. "google").
     * @param string $utm_medium          UTM medium.
     * @param string $utm_source          UTM source.
     * @param string $utm_campaign        UTM campaign.
     * @param string $utm_term            UTM term.
     * @param string $utm_content         UTM content.
     * @param int    $revision_id         Revision ID.
     */
    public static function write( $day, $referer_host, $referer_main_domain, $utm_medium, $utm_source, $utm_campaign, $utm_term, $utm_content, $revision_id ) {
        global $wpdb;

        $table_name = self::get_table_name();

        $query = $wpdb->prepare( 
            "
            INSERT INTO $table_name (a_day, referer_host, referer_main_domain, utm_medium, utm_source, utm_campaign, utm_term, utm_content, revision_id)
            VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %d);
        ",
            $day,
            $referer_host,
            $referer_main_domain,
            $utm_medium,
            $utm_source,
            $utm_campaign,
            $utm_term,
            $utm_content,
            $revision_id
        );

        $wpdb->query( $query ); 
    }
}
