<?php
/**
 * A class for Visitor_Details database table management.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\Databases;

use First_Party_Analytics\First_Party_Analytics;
use First_Party_Analytics\GeoLocation;
use First_Party_Analytics\Unique_Visits;
use First_Party_Analytics\Utils;
use WhichBrowser\Parser as User_Agent_Parser;
use Pdp\SyntaxError;

/**
 * Class Visitor_Details.
 */
final class Visitor_Details extends Database {

    /**
     * Table name.
     */
    const NAME = 'visitor_details';

    /**
     * ID of the revision.
     *
     * @var int
     */
    public static $revision_id = 0;

    /**
     * Converts requests sequence into DB population.
     *
     * @param array $requests_sequence Requests sequence.
     */
    public static function convert_sequence_into_db_population( $requests_sequence ) {
        foreach ( $requests_sequence as $request ) {
            if ( ! Unique_Visits::is_request_unique( $request, 'visitor-details-main', 'visitor-details' ) ) {
                continue;
            }
            Visitors::save_for_the_request( $request, 'visitor-details-main', 'visitor-details' );

            $day = gmdate( 'Y-m-d', strtotime( $request['time'] ) );

            self::$revision_id = (int) $request['revisionId'];

            // User-Agent-based data.

            if ( isset( $request['userAgent'] ) ) {
                $ua = $request['userAgent'];

                $parsed_ua = new User_Agent_Parser( $ua );

                self::write( $day, 'user-agent', $ua );

                $browser_name = ! empty( $parsed_ua->browser->alias ) ?
                    $parsed_ua->browser->alias : ( ! empty( $parsed_ua->browser->name ) ? $parsed_ua->browser->name : '' );

                self::write( $day, 'browser', $browser_name );
                self::write( $day, 'browserFull', $parsed_ua->browser->toString() );

                self::write( $day, 'os', $parsed_ua->os->getName() );
                self::write( $day, 'osFull', $parsed_ua->os->toString() );

                self::write( $day, 'device', ucfirst( $parsed_ua->getType() ) );
            }

            // Screen resolution.

            if ( isset( $request['screenResolution'] ) ) {
                self::write( $day, 'screen-resolution', $request['screenResolution'] );
            }

            // Country.

            $country = isset( $request['country'] ) && $request['country'] ?
                $request['country'] : // Country from the request headers.
                GeoLocation::get_ip_country( $request['ip'] );

            self::write( $day, 'country', strtoupper( $country ) );

            /**
             * Save language.
             *
             * @param string $language Language to save.
             * @param string $key      For which key to save.
             */
            $save_language = function( $language, $key = 'language' ) use ( $day ) {
                self::write( $day, $key, $language );

                if ( str_contains( $language, '-' ) ) { // Save base language also (e.g., for 'en-US' saves 'en' also (in addition to 'en-US').
                    self::write( $day, $key, explode( '-', $language )[0] );
                }
            };

            // Language.

            if ( isset( $request['language'] ) ) {
                $save_language( $request['language'] );
            }

            // Supported languages.

            if ( isset( $request['languages'] ) ) {
                foreach ( $request['languages'] as $language ) {
                    $save_language( $language, 'supportedLanguage' );
                }
            }

            // Is device touch only (smartphone, tablet).

            if ( isset( $request['isDeviceTouchOnly'] ) ) {
                self::write( $day, 'isDeviceTouchOnly', $request['isDeviceTouchOnly'] );
            }

            // Pointer type (fine, coarse, none).

            if ( isset( $request['pointerType'] ) ) {
                self::write( $day, 'pointerType', $request['pointerType'] );
            }

            // Has touch (yes, no).

            if ( isset( $request['hasTouch'] ) ) {
                self::write( $day, 'hasTouch', $request['hasTouch'] );
            }

            // Interaction capabilities (pointer type + if has touch).

            if ( isset( $request['pointerType'] ) && isset( $request['hasTouch'] ) ) {
                self::write( $day, 'interactionCapabilities', $request['pointerType'] . ' / ' . ( $request['hasTouch'] ? '+' : '-' ) );
            }

            /*
             * JS / NO-JS request.
             *
             * If "client" then JS, if "noscript" then NO-JS.
             *
             * "server" is unrecorded.
             */

            if ( 'server' !== $request['type'] ) {
                self::write( $day, 'noscript', 'noscript' === $request['type'] ? '1' : '0' );
            }

            /*
             * Breakpoint and orientation.
             *
             * Bootstrap 5 breakpoint. @see https://getbootstrap.com/docs/5.0/layout/breakpoints/#available-breakpoints
             *
             * E.g. "xxl-landscape"
             */

            if ( isset( $request['breakpointAndOrientation'] ) ) {
                self::write( $day, 'breakpointAndOrientation', $request['breakpointAndOrientation'] );
            }

            /*
             * DNT (Do not track).
             */

            if (
                isset( $request['dnt'] ) &&
                'fpa-null-val' !== $request['dnt'] // When wasn't able to supplement with server-side request, and can't determine client-side, then do not write.
            ) {
                self::write( $day, 'dnt', $request['dnt'] );
            }

            /*
             * Referer-based data..
             */

            if ( isset( $request['referer'] ) && $request['referer'] ) {
                $referer = preg_replace( '@^http(s)?:\/\/(www\.)?@', '', $request['referer'] ); // Delete protocol and www.
                self::write( $day, 'referer-full', untrailingslashit( $referer ) );

                $referer_host = untrailingslashit( explode( '/', $referer )[0] );
                self::write( $day, 'referer-host', $referer_host );

                try {
                    $domain_data = Utils::get_domain_data( $referer_host );
                    self::write( $day, 'referer-without-subdomain', $domain_data['registrable_domain'] );
                    self::write( $day, 'referer-second-level-domain', $domain_data['second_level_domain'] );
                } catch ( SyntaxError $e ) { // When the domain can't be parsed. For example, it's a local domain (127.0.0.1).
                    self::write( $day, 'referer-without-subdomain', $referer_host );
                }

                /*
                 * Text fragment.
                 */

                if (
                    isset( $request['textFragment'] ) &&
                    $request['textFragment'] &&
                    str_starts_with( $request['referer'], 'https://www.google.' )
                ) {
                    $text_fragment = urldecode( $request['textFragment'] );
                    self::write( $day, 'google-featured-snippet-highlighted-text', $text_fragment );
                }
            } else {
                self::write( $day, 'referer-host', '' ); // Also write when no referer for "Direct visits" sources row.
            }

            /*
             * Connection protocol.
             */

            if ( isset( $request['connectionProtocol'] ) ) {
                self::write( $day, 'connectionProtocol', $request['connectionProtocol'] );
            }
        }
    }

    /**
     * Writes to the DB.
     *
     * @param string $day          Day.
     * @param string $key          Key.
     * @param string $value        Value.
     */
    public static function write( $day, $key, $value ) {
        global $wpdb;

        $table_name = self::get_table_name();

        $query = $wpdb->prepare( 
            "
            INSERT INTO $table_name (a_day, a_key, a_value, visits_count, revision_id) VALUES (%s, %s, %s, 1, %d) ON DUPLICATE KEY UPDATE

            visits_count = visits_count + 1;
        ",
            $day,
            $key,
            $value,
            self::$revision_id
        );

        $wpdb->query( $query ); 
    }

    /**
     * Returns total aggregated views per hour.
     *
     * @param string $key                   For which key to get the params.
     * @param bool   $fancy_array           Return data in fancy array or not.
     * @param bool   $binary_value          Value is binary, can be yes or no. "yes" is always '1', "no" doesn't matter.
     * @param bool   $inverse_binary_values Whether to return inversed binary values.
     *
     * @return array|bool Data, or false if not data.
     */
    public static function get_data( $key, $fancy_array = true, $binary_value = false, $inverse_binary_values = false ) {
        global $wpdb;

        static $results = [];
        if ( ! isset( $results[ $key ] ) ) { // Cache.
            $table_name = self::get_table_name();

            $revision_where_clause = '';
            if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
                $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
                $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision );
            }

            $results[ $key ] = $wpdb->get_results(
                "
                SELECT
                    a_value as `value`, SUM(visits_count) as `visits`
                FROM
                    $table_name
                WHERE
                    a_key = '$key'
                    $revision_where_clause
                GROUP BY
                    a_value
                ORDER BY `visits` DESC
            ",
                ARRAY_A
            );

            if ( ! $results[ $key ] ) {
                return false;
            }

            $results[ $key ] = array_map(
                function( $x ) {
                    $x['visits'] = (int) $x['visits']; // Type cast visits to int.
                    return $x;
                },
                $results[ $key ]
            );
        }

        if ( $fancy_array ) {
            $values = array_column( $results[ $key ], 'value' );

            if ( $binary_value ) {
                $yes_index = array_search( '1', $values, true );
                if ( false !== $yes_index ) {
                    $values[ $yes_index ] = ! $inverse_binary_values ? __( 'Yes', 'first-party-analytics' ) : __( 'No', 'first-party-analytics' );
                }
                $values[ false !== $yes_index ? (int) ! $yes_index : 0 ] = ! $inverse_binary_values ?
                    __( 'No', 'first-party-analytics' ) : __( 'Yes', 'first-party-analytics' );
            } else {
                $empty_value_key = array_search( '', $values, true );
                if ( false !== $empty_value_key ) {
                    $values[ $empty_value_key ] = __( 'Unknown', 'first-party-analytics' );
                }
            }

            return [
                'value'  => self::values_conversion( $key, $values ),
                'visits' => array_column( $results[ $key ], 'visits' ),
                'type'   => $binary_value ? 'binary' : 'standard',
            ];
        } else {
            return array_column( $results[ $key ], 'visits', 'value' );
        }
    }

    /**
     * Converts between different values.
     *
     * Example: valuesConversion( 'connectionProtocol', [ 'h2' ] ) -> HTTP/2
     *
     * @param string $key    Key.
     * @param array  $values Values.
     *
     * @return array Values.
     */
    public static function values_conversion( $key, $values ) {
        $conversions = [
            'connectionProtocol' => [
                /**
                 * Conversion values.
                 *
                 * Only exception is for "Unknown"
                 *
                 * Supported values:
                 *
                 * - to_upper - convert all array values to upper case.
                 */
                '_mode'        => 'to_upper',

                /**
                 * The list of conversion:
                 *
                 * FROM -> TO
                 */
                '_conversions' => [
                    'h2'  => 'HTTP/2',
                    'h2c' => 'HTTP/2 over TCP',
                ],
            ],
        ];

        if ( isset( $conversions[ $key ] ) ) {
            foreach ( $conversions[ $key ]['_conversions'] as $replace_from => $replace_to ) {
                $i = array_search( $replace_from, $values, true );

                if ( false !== $i ) {
                    $values[ $i ] = $replace_to;
                }
            }

            if ( isset( $conversions[ $key ]['_mode'] ) ) {
                if ( 'to_upper' === $conversions[ $key ]['_mode'] ) {
                    $unknown_val = __( 'Unknown', 'first-party-analytics' );
                    $values      = array_map(
                        function( $v ) use ( $unknown_val ) {
                            return $unknown_val === $v ? $v : strtoupper( $v );
                        },
                        $values
                    );
                }
            }
        }

        return $values;
    }
}
