<?php
/**
 * A class for Visitors database table management.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\Databases;

/**
 * Class Visitors.
 */
final class Visitors extends Database {

    /**
     * Table name.
     */
    const NAME = 'visitors';

    /**
     * Default type.
     */
    const DEFAULT_TYPE = 'page-visit';

    /**
     * Returns the row for the day.
     *
     * @param string $time       Time.
     * @param int    $visitor_id Visitor ID.
     * @param string $value      Visited URL or false if any.
     * @param string $type       Desired type.
     *
     * @return array|false
     */
    public static function get_row_for_day( $time, $visitor_id, $value, $type = self::DEFAULT_TYPE ) {
        global $wpdb;

        $table_name = self::get_table_name();

        if ( ! $type ) {
            $type = self::DEFAULT_TYPE;
        }

        $result = $wpdb->get_row(
            $wpdb->prepare( 
                "
                SELECT *
                FROM $table_name WHERE DATE(`time`) = DATE(%s) and visitor_id = %s and `value` = %s AND type = %s;
            ",
                $time,
                $visitor_id,
                $value,
                $type
            )
        );

        return $result ? $result : false;
    }

    /**
     * Deletes visitors for the previous day (WHERE time < time() - 24h).
     */
    public static function remove_visitors_for_the_previous_day() {
        global $wpdb;

        $table_name = self::get_table_name();

        $day = gmdate( 'Y-m-d H:i:s', time() - DAY_IN_SECONDS );

        $query = $wpdb->prepare( 
            "
                    DELETE FROM $table_name WHERE `time` < %s;
                ",
            $day
        );

        $wpdb->query( $query ); 
    }

    /**
     * Writes to the DB.
     *
     * @param string      $time       Time.
     * @param int         $visitor_id Visitor ID.
     * @param string      $value      Value.
     * @param bool|string $type       Type. If not specified, uses default type.
     */
    public static function write( $time, $visitor_id, $value, $type ) {
        global $wpdb;

        if ( ! $type ) {
            $type = self::DEFAULT_TYPE;
        }

        $table_name = self::get_table_name();

        $query = $wpdb->prepare( 
            "
            INSERT IGNORE INTO $table_name (`time`, visitor_id, `value`, type) VALUES (%s, %s, %s, %s);
        ",
            $time,
            $visitor_id,
            $value,
            $type
        );

        $wpdb->query( $query ); 
    }

    /**
     * Writes data to the "visitors" database for the request.
     *
     * @param array       $request Request.
     * @param bool|string $value   Custom value.
     * @param bool|string $type    Custom type.
     */
    public static function save_for_the_request( $request, $value = false, $type = false ) {
        $time       = gmdate( 'Y-m-d H:i:s', strtotime( $request['time'] ) );
        $visitor_id = $request['id'];
        $url        = $request['url'];
        self::write( $time, $visitor_id, $value ? $value : $url, $type );
    }
}
