<?php
/**
 * The MaxMind database management class file.
 *
 * @important This is almost entirely copied from WooCommerce MaxMind integration.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\MaxMind;

use Exception;
use First_Party_Analytics\First_Party_Analytics;
use PharData;
use WP_Error;

/**
 * The service class responsible for interacting with MaxMind databases.
 */
class Database {

    /**
     * The name of the MaxMind database to utilize.
     */
    const DATABASE = 'GeoLite2-Country';

    /**
     * The extension for the MaxMind database.
     */
    const DATABASE_EXTENSION = '.mmdb';

    /**
     * The single instance of the class.
     *
     * @var object
     */
    protected static $instance;

    /**
     * Uploads directory.
     *
     * @var string
     */
    private $uploads_dir;

    /**
     * Get the main Instance.
     *
     * @return object Main instance.
     */
    public static function get_instance() {
        if ( ! isset( self::$instance ) ) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    /**
     * Constructor.
     */
    public function __construct() {
        $this->uploads_dir = trailingslashit( wp_upload_dir()['basedir'] ) . 'first_party_analytics_uploads/';
    }

    /**
     * Fetches the path that the database should be stored.
     *
     * @return string The local database path.
     */
    public function get_database_path() {
        return $this->uploads_dir . self::DATABASE . self::DATABASE_EXTENSION;
    }

    /**
     * Fetches the path that the database should be stored.
     *
     * @return string Last update time, or false if no such time.
     */
    public function get_last_update_time() {
        $file_path = $this->get_database_path();
        $mtime     = First_Party_Analytics::$fs->mtime( $file_path );
        return $mtime ? $mtime : '';
    }

    /**
     * Fetches the database from the MaxMind service.
     *
     * @param string $license_key The license key to be used when downloading the database.
     *
     * @return string|WP_Error The path to the database file or an error if invalid.
     *
     * @throws Exception Exception.
     */
    public function download_database( $license_key ) {
        $download_uri = add_query_arg(
            [
                'edition_id'  => self::DATABASE,
                'license_key' => rawurlencode( sanitize_text_field( $license_key ) ),
                'suffix'      => 'tar.gz',
            ],
            'https://download.maxmind.com/app/geoip_download'
        );

        // Needed for the download_url call right below.
        require_once ABSPATH . 'wp-admin/includes/file.php';

        $tmp_archive_path = download_url( esc_url_raw( $download_uri ) );
        if ( is_wp_error( $tmp_archive_path ) ) {
            // Transform the error into something more informative.
            $error_data = $tmp_archive_path->get_error_data();
            if ( isset( $error_data['code'] ) ) {
                switch ( $error_data['code'] ) {
                    case 401:
                        return new WP_Error(
                            'first_party_analytics_maxmind_geolocation_database_license_key',
                            __( 'The MaxMind license key is invalid. If you have recently created this key, you may need to wait for it to become active.', 'first-party-analytics' )
                        );
                }
            }

            return new WP_Error( 'first_party_analytics_maxmind_geolocation_database_download', __( 'Failed to download the MaxMind database.', 'first-party-analytics' ) );
        }

        // Extract the database from the archive.
        try {
            $file = new PharData( $tmp_archive_path );

            $tmp_database_path = trailingslashit( dirname( $tmp_archive_path ) ) . trailingslashit( $file->current()->getFilename() ) . self::DATABASE . self::DATABASE_EXTENSION;

            $file->extractTo(
                dirname( $tmp_archive_path ),
                trailingslashit( $file->current()->getFilename() ) . self::DATABASE . self::DATABASE_EXTENSION,
                true
            );
        } catch ( Exception $exception ) {
            return new WP_Error( 'first_party_analytics_maxmind_geolocation_database_archive', $exception->getMessage() );
        } finally {
            // Remove the archive since we only care about a single file in it.
            unlink( $tmp_archive_path );
        }

        return $tmp_database_path;
    }

    /**
     * Updates the database used for geolocation queries.
     *
     * @param bool $with_interval Whether the DB is updated with interval.
     *
     * @throws Exception Exception.
     */
    public function update_database( $with_interval = true ) {
        if ( $with_interval && self::is_wc_maxmind_integration_found() ) { // Do not update the DB when there is WC MaxMind integration found.
            return;
        }

        // Remove any existing archives to comply with the MaxMind TOS.
        $target_database_path = $this->get_database_path();

        // If there's no database path, we can't store the database.
        if ( empty( $target_database_path ) ) {
            return;
        }

        if ( First_Party_Analytics::$fs->exists( $target_database_path ) ) {
            First_Party_Analytics::$fs->delete( $target_database_path );
        }

        if ( ! First_Party_Analytics::$fs->exists( $this->uploads_dir ) ) {
            First_Party_Analytics::$fs->mkdir( $this->uploads_dir );
        }

        if ( isset( $new_database_path ) ) {
            $tmp_database_path = $new_database_path;
        } else {
            // We can't download a database if there's no license key configured.
            $license_key = First_Party_Analytics::$options->get( 'maxmind-license-key' );
            if ( empty( $license_key ) ) {
                return;
            }

            $tmp_database_path = $this->download_database( $license_key );
            if ( is_wp_error( $tmp_database_path ) ) {
                return;
            }
        }

        // Move the new database into position.
        First_Party_Analytics::$fs->move( $tmp_database_path, $target_database_path, true );
        First_Party_Analytics::$fs->delete( dirname( $tmp_database_path ) );
    }

    /**
     * Fetches the ISO country code associated with an IP address.
     *
     * @param string $ip_address The IP address to find the country code for.
     * @return string The country code for the IP address, or empty if not found.
     */
    public function get_iso_country_code_for_ip( $ip_address ) {
        $country_code = '';

        if ( ! class_exists( 'MaxMind\Db\Reader' ) ) {
            return $country_code;
        }

        $database_path = $this->get_database_path();
        if ( ! file_exists( $database_path ) ) {
            return $country_code;
        }

        try {
            $reader = new \MaxMind\Db\Reader( $database_path );
            $data   = $reader->get( $ip_address );

            if ( isset( $data['country']['iso_code'] ) ) {
                $country_code = $data['country']['iso_code'];
            }

            $reader->close();
        } catch ( Exception $e ) {
            return $country_code;
        }

        return $country_code;
    }

    /**
     * Whether WooCommerce integration is found.
     *
     * @return bool
     */
    public static function is_wc_maxmind_integration_found() {
        return class_exists( 'WC_Integration_MaxMind_Geolocation' );
    }
}
