<?php
/**
 * A class to add post info to the post publish box.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\Representation;

use First_Party_Analytics\Databases\Page_Visits;
use First_Party_Analytics\Databases\Page_Visits_Finish;
use First_Party_Analytics\First_Party_Analytics;
use First_Party_Analytics\Utils;
use First_Party_Analytics\Web_Vitals;
use First_Party_Analytics\WooCommerce\Pages_Tracking_Data;
use First_Party_Analytics\WooCommerce\Products_Tracking_Data;
use WP_Post;

/**
 * Class Post_Publish_Box.
 */
final class Publish_Box {

    /**
     * Constructor.
     */
    public function __construct() {
        add_action( 'add_meta_boxes', [ $this, 'add_custom_box' ] );
    }

    /**
     * Adds custom box.
     */
    public function add_custom_box() {
        add_meta_box(
            'first-party-analytics-standard-editor',
            __( 'First Party Analytics', 'first-party-analytics' ),
            [ $this, 'print_meta_box_content' ],
            [ 'post', 'page', 'product' ],
            'side',
            'high',
            [ '__back_compat_meta_box' => true ]
        );
    }

    /**
     * Adds publish-box text.
     *
     * @param WP_Post $post The current post object.
     */
    public function print_meta_box_content( WP_Post $post ) {
        $post_latest_revision_id = Utils::get_post_latest_revision_id( $post->ID );

        ?>
        <div class="misc-pub-section first-party-analytics-publish-box" id="first-party-analytics-visits">
            <div class="first-party-analytics-publish-box-image"></div>
            <div class="first-party-analytics-publish-box-content">
                <?php esc_attr_e( 'Unique visits (last 7 days):', 'first-party-analytics' ); ?>
                <b><?php echo esc_attr( Page_Visits::get_post_visits_count( $post->ID, 7 ) ); ?></b>
            </div>
        </div>
        <div class="misc-pub-section first-party-analytics-publish-box" id="first-party-analytics-popularity">
            <div class="first-party-analytics-publish-box-image"></div>
            <div class="first-party-analytics-publish-box-content">
                <?php esc_attr_e( 'Popularity ranking (last 7 days):', 'first-party-analytics' ); ?>
                <b><?php echo esc_attr( Page_Visits::get_popularity_ranking( $post->ID, 7 ) ); ?></b>
            </div>
        </div>
        <div class="misc-pub-section first-party-analytics-publish-box" id="first-party-analytics-bounce-rate">
            <div class="first-party-analytics-publish-box-image"></div>
            <div class="first-party-analytics-publish-box-content">
                <?php esc_attr_e( 'Bounce rate (last 7 days):', 'first-party-analytics' ); ?>
                <b><?php echo esc_attr( Page_Visits::get_bounce_rate( $post->ID, 7 ) ); ?></b>
            </div>
        </div>
        <div class="misc-pub-section first-party-analytics-publish-box" id="first-party-analytics-avg-website-data-download-size">
            <div class="first-party-analytics-publish-box-image"></div>
            <div class="first-party-analytics-publish-box-content">
                <?php esc_attr_e( 'Avg. download size of website data from the page:', 'first-party-analytics' ); ?>
                <b><?php echo esc_attr( Page_Visits_Finish::get_avg_website_data_download_size( $post->ID, $post_latest_revision_id ) ); ?></b>
            </div>
        </div>
        <div class="misc-pub-section first-party-analytics-publish-box" id="first-party-analytics-avg-asset-less-page-size">
            <div class="first-party-analytics-publish-box-image"></div>
            <div class="first-party-analytics-publish-box-content">
                <?php esc_attr_e( 'Avg. page download size (assets-less):', 'first-party-analytics' ); ?>
                <b><?php echo esc_attr( Page_Visits_Finish::get_avg_asset_less_page_size( $post->ID, $post_latest_revision_id ) ); ?></b>
            </div>
        </div>
        <?php Web_Vitals::get_web_vitals_table( $post->ID, true ); ?>
        <?php
        if ( First_Party_Analytics::$premium_enabled ) :
            ?>
            <div class="misc-pub-section first-party-analytics-publish-box" id="first-party-analytics-page-conversions-value">
                <div class="first-party-analytics-publish-box-image"></div>
                <div class="first-party-analytics-publish-box-content">
                    <?php
                    $currency = Utils::get_currency();
                    echo esc_html(
                        __( 'Conversion value', 'first-party-analytics' ) . ( $currency ? ' (' . $currency . '):' : ':' )
                    );
                    ?>
                    <b><?php echo esc_attr( Pages_Tracking_Data::get_conversions_value( $post->ID ) ); ?></b>
                </div>
            </div>
            <div class="misc-pub-section first-party-analytics-publish-box" id="first-party-analytics-page-average-conversion-depth">
                <div class="first-party-analytics-publish-box-image"></div>
                <div class="first-party-analytics-publish-box-content">
                    <?php esc_attr_e( 'Average conversion depth:', 'first-party-analytics' ); ?>
                    <b><?php echo esc_attr( Pages_Tracking_Data::get_average_conversion_depth( $post->ID ) ); ?></b>
                </div>
            </div>
            <?php
        endif;
        ?>
        <?php
        if ( 'product' === $post->post_type ) :
            ?>
            <div class="misc-pub-section first-party-analytics-publish-box" id="first-party-analytics-viewed-in-catalog-counter">
                <div class="first-party-analytics-publish-box-image"></div>
                <div class="first-party-analytics-publish-box-content">
                    <?php esc_attr_e( 'Times product viewed in catalog:', 'first-party-analytics' ); ?>
                    <b><?php echo esc_attr( Products_Tracking_Data::get_viewed_in_catalog_count( $post->ID ) ); ?></b>
                </div>
            </div>
            <div class="misc-pub-section first-party-analytics-publish-box" id="first-party-analytics-product-views-counter">
                <div class="first-party-analytics-publish-box-image"></div>
                <div class="first-party-analytics-publish-box-content">
                    <?php esc_attr_e( 'Times product page viewed:', 'first-party-analytics' ); ?>
                    <b><?php echo esc_attr( Products_Tracking_Data::get_product_all_time_unique_views_count( $post->ID ) ); ?></b>
                </div>
            </div>
            <div class="misc-pub-section first-party-analytics-publish-box" id="first-party-analytics-product-added-to-cart">
                <div class="first-party-analytics-publish-box-image"></div>
                <div class="first-party-analytics-publish-box-content">
                    <?php esc_attr_e( 'Times added to cart:', 'first-party-analytics' ); ?>
                    <b><?php echo esc_attr( Products_Tracking_Data::get_added_to_cart_count( $post->ID ) ); ?></b>
                </div>
            </div>
            <div class="misc-pub-section first-party-analytics-publish-box" id="first-party-analytics-product-purchased">
                <div class="first-party-analytics-publish-box-image"></div>
                <div class="first-party-analytics-publish-box-content">
                    <?php esc_attr_e( 'Times purchased:', 'first-party-analytics' ); ?>
                    <b><?php echo esc_attr( Products_Tracking_Data::get_purchased_count( $post->ID ) ); ?></b>
                </div>
            </div>
            <div class="misc-pub-section first-party-analytics-publish-box" id="first-party-analytics-product-purchased">
                <div class="first-party-analytics-publish-box-image"></div>
                <div class="first-party-analytics-publish-box-content">
                    <?php esc_attr_e( 'All-time purchased (including when the plugin was not enabled):', 'first-party-analytics' ); ?>
                    <b><?php echo esc_attr( Products_Tracking_Data::get_all_time_purchased_count( $post->ID ) ); ?></b>
                </div>
            </div>
            <?php
        endif;
        ?>
        <hr>
        <div class="misc-pub-section first-party-analytics-publish-box">
            <div class="first-party-analytics-publish-box-image"></div>
            <div class="first-party-analytics-publish-box-content">
                <a href="<?php echo esc_url( admin_url( 'admin.php?page=first-party-analytics' ) ); ?>">
                    <?php esc_attr_e( 'Plugin page', 'first-party-analytics' ); ?>
                </a>
            </div>
        </div>
        <?php
    }
}
