<?php
/**
 * A class for the average view time data retrieval.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\Representation\Graphs;

use First_Party_Analytics\Databases\Page_Visits_Finish as Page_Visits_Finish_DB;
use First_Party_Analytics\Utils;

/**
 * Class Average_View_time.
 */
final class Average_View_Time {

    /**
     * Returns all-time data.
     *
     * @return float|bool
     */
    public static function get_all_time() {
        global $wpdb;

        static $result;
        if ( null !== $result ) { // Cache.
            return $result ? (float) $result[0] : false;
        }

        $table_name = Page_Visits_Finish_DB::get_table_name();

        $revision_where_clause = '';
        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $revision_where_clause = $wpdb->prepare( 'WHERE revision_id = %d', $revision );
        }

        $result = $wpdb->get_row(
            "
                SELECT SUM(avg_view_time * total_visits) / SUM(total_visits)
                FROM $table_name
                $revision_where_clause;
            ",
            ARRAY_N
        );

        return $result ? (float) $result[0] : false;
    }

    /**
     * Returns data per days.
     *
     * @param int $days_range For how many days to display pages.
     *
     * @return array
     */
    public static function get_days_data( $days_range = 1 ) {
        global $wpdb;

        $table_name = Page_Visits_Finish_DB::get_table_name();

        $time      = time();
        $day_today = gmdate( 'Y-m-d', $time );
        $day_from  = gmdate( 'Y-m-d', $time - DAY_IN_SECONDS * ( $days_range - 1 ) );

        $revision_where_clause = '';
        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision );
        }

        $results = $wpdb->get_results(
            $wpdb->prepare(
                "
                SELECT a_day, SUM(avg_view_time * total_visits) / SUM(total_visits) FROM $table_name
                WHERE a_day BETWEEN %s AND %s $revision_where_clause
                GROUP BY a_day
                ORDER BY a_day;
            ",
                $day_from,
                $day_today
            ),
            ARRAY_N
        );

        return Utils::convert_array_column_to_float( $results, 2 );
    }

    /**
     * Returns data per weeks.
     *
     * @param int $weeks_range For how many weeks to display pages.
     *
     * @return array
     */
    public static function get_weeks_data( $weeks_range = 1 ) {
        global $wpdb;

        $table_name = Page_Visits_Finish_DB::get_table_name();

        $time      = time();
        $day_today = gmdate( 'Y-m-d', $time );
        $day_from  = gmdate( 'Y-m-d', $time - WEEK_IN_SECONDS * ( $weeks_range - 1 ) );

        $revision_where_clause = '';
        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision );
        }

        $results = $wpdb->get_results(
            $wpdb->prepare(
                "
                SELECT a_day, SUM(avg_view_time * total_visits) / SUM(total_visits) FROM $table_name
                WHERE a_day BETWEEN %s AND %s $revision_where_clause
                GROUP BY YEARWEEK(a_day)
                ORDER BY a_day;
            ",
                $day_from,
                $day_today
            ),
            ARRAY_N
        );

        return Utils::convert_array_column_to_float( $results, 2 );
    }

    /**
     * Returns data per months.
     *
     * @param int $months_range For how many months to display pages.
     *
     * @return array
     */
    public static function get_months_data( $months_range = 1 ) {
        global $wpdb;

        $table_name = Page_Visits_Finish_DB::get_table_name();

        $time      = time();
        $day_today = gmdate( 'Y-m-d', $time );
        $day_from  = gmdate( 'Y-m-d', $time - MONTH_IN_SECONDS * ( $months_range - 1 ) );

        $revision_where_clause = '';
        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision );
        }

        $results = $wpdb->get_results(
            $wpdb->prepare(
                "
                SELECT a_day, SUM(avg_view_time * total_visits) / SUM(total_visits) FROM $table_name
                WHERE a_day BETWEEN %s AND %s $revision_where_clause
                GROUP BY MONTH(a_day)
                ORDER BY a_day;
            ",
                $day_from,
                $day_today
            ),
            ARRAY_N
        );

        return Utils::convert_array_column_to_float( $results, 2 );
    }

    /**
     * Returns general stats.
     *
     * @return array
     */
    public static function get_general_stats() {
        return [
            'allTime' => self::get_all_time(),
        ];
    }

    /**
     * Returns graph stats data.
     *
     * @return array
     */
    public static function get_data() {
        return [
            'general' => [
                'stats' => self::get_general_stats(),
            ],
            'day'     => [
                'unit'   => 'day',
                'fields' => [
                    'period',
                    'value',
                ],
                'data'   => self::get_days_data( 30 ),
            ],
            'week'    => [
                'unit'   => 'week',
                'fields' => [
                    'period',
                    'value',
                ],
                'data'   => self::get_weeks_data( 14 ),
            ],
            'month'   => [
                'unit'   => 'month',
                'fields' => [
                    'period',
                    'value',
                ],
                'data'   => self::get_months_data( 12 ),
            ],
        ];
    }
}
