<?php
/**
 * A class for the graph data retrieval.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\Representation\Graphs;

use First_Party_Analytics\Databases\Page_Visits as Page_Visits_DB;

/**
 * Class General_Views.
 */
final class General_Views {

    /**
     * Returns views for today.
     *
     * @return int|bool
     */
    public static function get_today_views() {
        global $wpdb;

        static $result;
        if ( null !== $result ) { // Cache.
            return $result ? (int) $result[0] : false;
        }

        $table_name = Page_Visits_DB::get_table_name();

        $day = gmdate( 'Y-m-d' );

        $revision_where_clause = '';
        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision );
        }

        $result = $wpdb->get_row(
            $wpdb->prepare(
                "
                    SELECT SUM(unique_visits) FROM $table_name
                    WHERE a_day = %s $revision_where_clause;
                ",
                $day
            ),
            ARRAY_N
        );

        return $result ? (int) $result[0] : false;
    }

    /**
     * Returns views for today.
     *
     * @return int|bool
     */
    public static function get_all_views() {
        global $wpdb;

        static $result;
        if ( null !== $result ) { // Cache.
            return $result ? (int) $result[0] : false;
        }

        $table_name = Page_Visits_DB::get_table_name();

        $revision_where_clause = '';
        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $revision_where_clause = $wpdb->prepare( 'WHERE revision_id = %d', $revision );
        }

        $result = $wpdb->get_row(
            "
                SELECT SUM(unique_visits)
                FROM $table_name
                $revision_where_clause;
            ",
            ARRAY_N
        );

        return $result ? (int) $result[0] : false;
    }

    /**
     * Returns best day data.
     *
     * @return array
     */
    public static function get_best_day_data() {
        global $wpdb;

        static $result;
        if ( null !== $result ) { // Cache.
            return $result;
        }

        $table_name = Page_Visits_DB::get_table_name();

        $revision_where_clause = '';
        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $revision_where_clause = $wpdb->prepare( 'WHERE revision_id = %d', $revision );
        }

        $result = $wpdb->get_row(
            "
                SELECT a_day, SUM(unique_visits) as `visits`
                FROM $table_name
                $revision_where_clause
                GROUP BY a_day
                ORDER BY `visits` DESC
                LIMIT 1;
            ",
            ARRAY_A
        );

        return $result;
    }

    /**
     * Returns views per date.
     *
     * @param int $days_range For how many days to display pages.
     *
     * @return array
     */
    public static function get_days_total_views( $days_range = 1 ) {
        global $wpdb;

        static $results;
        if ( null !== $results ) { // Cache.
            return $results;
        }

        $table_name = Page_Visits_DB::get_table_name();

        $time      = time();
        $day_today = gmdate( 'Y-m-d', $time );
        $day_from  = gmdate( 'Y-m-d', $time - DAY_IN_SECONDS * ( $days_range - 1 ) );

        $revision_where_clause = '';
        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision );
        }

        $results = $wpdb->get_results(
            $wpdb->prepare(
                "
                SELECT a_day, SUM(unique_visits) FROM $table_name
                WHERE a_day BETWEEN %s AND %s $revision_where_clause
                GROUP BY a_day
                ORDER BY a_day;
            ",
                $day_from,
                $day_today
            ),
            ARRAY_N
        );

        return $results;
    }

    /**
     * Returns views per date.
     *
     * @param int $weeks_range For how many weeks to display pages.
     *
     * @return array
     */
    public static function get_weeks_total_views( $weeks_range = 1 ) {
        global $wpdb;

        static $results;
        if ( null !== $results ) { // Cache.
            return $results;
        }

        $table_name = Page_Visits_DB::get_table_name();

        $time      = time();
        $day_today = gmdate( 'Y-m-d', $time );
        $day_from  = gmdate( 'Y-m-d', $time - WEEK_IN_SECONDS * ( $weeks_range - 1 ) );

        $revision_where_clause = '';
        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision );
        }

        $results = $wpdb->get_results(
            $wpdb->prepare(
                "
                SELECT a_day, SUM(unique_visits) FROM $table_name
                WHERE a_day BETWEEN %s AND %s $revision_where_clause
                GROUP BY YEARWEEK(a_day)
                ORDER BY a_day;
            ",
                $day_from,
                $day_today
            ),
            ARRAY_N
        );

        return $results;
    }

    /**
     * Returns views per date.
     *
     * @param int $months_range For how many months to display pages.
     *
     * @return array
     */
    public static function get_months_total_views( $months_range = 1 ) {
        global $wpdb;

        static $results;
        if ( null !== $results ) { // Cache.
            return $results;
        }

        $table_name = Page_Visits_DB::get_table_name();

        $time      = time();
        $day_today = gmdate( 'Y-m-d', $time );
        $day_from  = gmdate( 'Y-m-d', $time - MONTH_IN_SECONDS * ( $months_range - 1 ) );

        $revision_where_clause = '';
        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision );
        }

        $results = $wpdb->get_results(
            $wpdb->prepare(
                "
                SELECT a_day, SUM(unique_visits) FROM $table_name
                WHERE a_day BETWEEN %s AND %s $revision_where_clause
                GROUP BY MONTH(a_day)
                ORDER BY a_day;
            ",
                $day_from,
                $day_today
            ),
            ARRAY_N
        );

        return $results;
    }

    /**
     * Returns general stats.
     *
     * @return array
     */
    public static function get_general_stats() {
        $best_day = self::get_best_day_data();

        return [
            'views_today'          => self::get_today_views(),
            'views'                => self::get_all_views(),
            'views_best_day'       => $best_day ? $best_day['a_day'] : false,
            'views_best_day_total' => $best_day ? (int) $best_day['visits'] : false,
        ];
    }

    /**
     * Returns graph stats data.
     *
     * @return array
     */
    public static function get_data() {
        return [
            'general' => [
                'stats' => self::get_general_stats(),
            ],
            'day'     => [
                'unit'   => 'day',
                'fields' => [
                    'period',
                    'views',
                ],
                'data'   => self::get_days_total_views( 30 ),
            ],
            'week'    => [
                'unit'   => 'week',
                'fields' => [
                    'period',
                    'views',
                ],
                'data'   => self::get_weeks_total_views( 14 ),
            ],
            'month'   => [
                'unit'   => 'month',
                'fields' => [
                    'period',
                    'views',
                ],
                'data'   => self::get_months_total_views( 12 ),
            ],
        ];
    }
}
