<?php
/**
 * A class for Google featured snippet highlighted text table representation.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\Representation\Tables;

use First_Party_Analytics\Databases\Visitor_Details as DB;
use First_Party_Analytics\Utils;

/**
 * Class Google_Featured_Snippet_Highlighted_Text.
 */
final class Google_Featured_Snippet_Highlighted_Text extends Table {

    /**
     * Returns the list of texts.
     *
     * @param int $page For which page to get the stuff.
     *
     * @return array
     */
    public static function get_texts_list( $page = 1 ) {
        global $wpdb;

        $table_name = DB::get_table_name();

        // Get a bit more pages to know when to add pagination.
        $pages_limit = self::PAGES_LIMIT + 1;

        $offset = ( $page - 1 ) * self::PAGES_LIMIT;

        $revision_where_clause = '';
        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision );
        }

        $results = $wpdb->get_results(
                $wpdb->prepare(
                    "
                        SELECT SUM(visits_count) as `visits_count`, a_value
                        FROM $table_name
                        WHERE a_key = 'google-featured-snippet-highlighted-text' $revision_where_clause
                        GROUP BY a_value
                        ORDER BY `visits_count` DESC
                        LIMIT %d OFFSET %d;
                    ",
                    $pages_limit,
                    $offset
                ),
            ARRAY_A
        );

        return $results;
    }

    /**
     * Get the most visited pages table.
     *
     * @param bool $return Return or not.
     * @param int  $page   For which page to get the stuff.
     *
     * @return string|void
     */
    public static function print_table( $return = false, $page = 1 ) {
        $content = self::get_table_content(
            array_column( self::get_texts_list( $page ), 'visits_count', 'a_value' ),
            $page
        );

        if ( $return ) {
            return $content;
        } else {
            echo Utils::esc_html_before_output( $content );
        }
    }

    /**
     * Returns the table content.
     *
     * @param array $texts The list of texts, where key is visits count and the value is the text.
     * @param int   $page  For which page to get the stuff.
     */
    public static function get_table_content( $texts, $page = 1  ) {
        $add_load_more_btn = count( $texts ) > self::PAGES_LIMIT;
        $texts             = array_slice( $texts, 0, self::PAGES_LIMIT );

        ob_start();
        if ( $texts ) :
            ?>
            <div class="first-party-analytics-table-content">
                <table>
                    <thead>
                    <tr>
                        <th><?php esc_html_e( '#', 'first-party-analytics' ); ?></th>
                        <th><?php esc_html_e( 'Text', 'first-party-analytics' ); ?></th>
                        <th><?php esc_html_e( 'Visits', 'first-party-analytics' ); ?></th>
                    </tr>
                    </thead>
                    <tbody>
                <?php
                $i = ( $page - 1 ) * self::PAGES_LIMIT;;
                foreach ( $texts as $text => $visits ) :
                    $i++;
                    ?>
                        <tr>
                            <td><?php echo esc_html( $i ); ?></td>
                            <td><?php echo esc_html( str_replace( ',', ' <...> ', $text ) ); ?></a></td>
                            <td><?php echo esc_html( $visits ); ?></td>
                        </tr>
                    <?php endforeach; ?>
                    </tbody>
                </table>
            </div>
                <?php if ( $add_load_more_btn ) : ?>
                <button data-next-page="<?php echo esc_attr( $page + 1 ); ?>" class="first-party-analytics-load-more-btn">
                    <?php esc_attr_e( 'Load more', 'first-party-analytics' ); ?>
                </button>
            <?php endif; ?>
                    <?php
            else :
                ?>
            <p class="no-data-here-yet-block">
                <?php esc_attr_e( 'No data yet.', 'first-party-analytics' ); ?>
            </p>
                <?php
            endif;
            return ob_get_clean();
    }
}
