<?php
/**
 * A class for pages performance representation.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\Representation\Tables;

use First_Party_Analytics\Databases\Page_Visits_Finish as DB;
use First_Party_Analytics\Utils;
use First_Party_Analytics\Web_Vitals;

/**
 * Class Pages_Performance.
 */
final class Pages_Performance extends Table {

    /**
     * Default request type.
     */
    const DEFAULT_DATE = 1;

    /**
     * Prints dates.
     */
    public static function print_dates() {
        $dates = [
            1          => __( 'Last day', 'first-party-analytics' ),
            7          => __( 'Last 7 days', 'first-party-analytics' ),
            30         => __( 'Last 30 days', 'first-party-analytics' ),
            'all-time' => __( 'All-time', 'first-party-analytics' ),
        ];

        foreach ( $dates as $date_slug => $date_name ) {
            ?>
            <span class="setting-entity <?php echo self::DEFAULT_DATE === $date_slug ? 'selected default' : ''; ?>" data-val="<?php echo esc_attr( $date_slug ); ?>">
            <?php echo esc_attr( $date_name ); ?>
            </span>
            <?php
        }
    }

    /**
     * Returns the list of referers.
     *
     * @param int    $page For which page to get the stuff.
     * @param string $date For which date to get the data.
     *
     * @return array
     */
    public static function get_requests_list( $page, $date ) {
        global $wpdb;

        $table_name = DB::get_table_name();

        // Get a bit more pages to know when to add pagination.
        $pages_limit = self::PAGES_LIMIT + 1;

        $offset = ( $page - 1 ) * self::PAGES_LIMIT;

        if ( 'all-time' === $date ) {
            $where_clause = '';
        } else {
            $time         = time();
            $days_range   = (int) $date;
            $day_today    = gmdate( 'Y-m-d', $time );
            $day_from     = 1 === $days_range ? $day_today : gmdate( 'Y-m-d', $time - DAY_IN_SECONDS * ( $days_range - 1 ) );
            $where_clause = $wpdb->prepare( 'WHERE a_day BETWEEN %s AND %s', $day_from, $day_today );
        }

        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $where_clause .= ( $where_clause ? ' AND' : 'WHERE' ) . $wpdb->prepare( ' revision_id = %d', $revision );
        }

        $results = $wpdb->get_results(
                $wpdb->prepare(
                    "
                        SELECT `a_url`,
                               `post_id`,
                               ROUND(SUM(viewport_completed_loading_time_count * avg_viewport_completed_loading_time) / SUM(viewport_completed_loading_time_count), 2) as `avg_viewport_completed_loading_time`
                        FROM $table_name
                        $where_clause
                        GROUP BY `a_url`
                        ORDER BY `avg_viewport_completed_loading_time` DESC, `id`
                        LIMIT %d OFFSET %d;
                    ",
                    $pages_limit,
                    $offset
                ),
            ARRAY_A
        );

        return $results;
    }

    /**
     * Get the most visited pages table.
     *
     * @param bool   $return Return or not.
     * @param int    $page   For which page to get the stuff.
     *
     * @return string|void
     */
    public static function print_table( $return = false, $page = 1,  $date = self::DEFAULT_DATE ) {
        $data = [];
        foreach ( self::get_requests_list( $page, $date ) as $page_data ) {
            $scores = Web_Vitals::get_web_vitals(
                $page_data['post_id'], (int) $date, false, true, true
            );

            $data[ $page_data[ 'a_url' ] ] = [
                'avg_viewport_completed_loading_time' => $page_data['avg_viewport_completed_loading_time'],
                'score_mobile'                        => $scores['WEB_VITALS_SCORE_MOBILE'][0],
                'score_desktop'                       => $scores['WEB_VITALS_SCORE'][0]
            ];
        }

        $content = self::get_table_content(
            $data,
            $page
        );

        if ( $return ) {
            return $content;
        } else {
            echo Utils::esc_html_before_output( $content );
        }
    }

    /**
     * Returns the table content.
     *
     * @param array $pages The list of pages, where key is page URL and the value is the data.
     * @param int   $page  For which page to get the stuff.
     */
    public static function get_table_content( $pages, $page = 1  ) {
        $add_load_more_btn = count( $pages ) > self::PAGES_LIMIT;
        $pages             = array_slice( $pages, 0, self::PAGES_LIMIT );

        ob_start();
        if ( $pages ) :
            ?>
            <div class="first-party-analytics-table-content">
                <table>
                    <thead>
                    <tr>
                        <th><?php esc_html_e( '#', 'first-party-analytics' ); ?></th>
                        <th><?php esc_html_e( 'URL', 'first-party-analytics' ); ?></th>
                        <th><?php esc_html_e( 'Avg. time to load viewport area', 'first-party-analytics' ); ?></th>
                        <th><?php esc_html_e( 'Performance Score (Mobile / Desktop)', 'first-party-analytics' ); ?></th>
                    </tr>
                    </thead>
                    <tbody>
                <?php
                $i = ( $page - 1 ) * self::PAGES_LIMIT;;
                foreach ( $pages as $url => $data ) :
                    $i++;
                    ?>
                        <tr>
                            <td><?php echo esc_html( $i ); ?></td>
                            <td><a href="<?php echo esc_url( $url ); ?>" target=”_blank”><?php echo esc_html( $url ); ?></a></td>
                            <td>
                            <?php
                                /* translators: %s is the number of seconds. */
                                echo esc_html( $data['avg_viewport_completed_loading_time'] ? sprintf( '%s sec.', $data['avg_viewport_completed_loading_time'] ) : '-' );
                            ?>
                            </td>
                            <td>
                                <a href="#" data-page="<?php echo esc_attr( $url ); ?>"
                                    class="show-web-vitals-btn">
                                    <?php echo esc_html( $data['score_mobile'] . ' / ' . $data['score_desktop'] ); ?>
                                </a>
                            </td>
                    </tr>
                    <?php endforeach; ?>
                    </tbody>
                </table>
            </div>
                <?php if ( $add_load_more_btn ) : ?>
                <button data-next-page="<?php echo esc_attr( $page + 1 ); ?>" class="first-party-analytics-load-more-btn">
                    <?php esc_attr_e( 'Load more', 'first-party-analytics' ); ?>
                </button>
                    <?php endif; ?>
        <?php
            else :
        ?>
            <p class="no-data-here-yet-block">
                <?php esc_attr_e( 'No data yet.', 'first-party-analytics' ); ?>
            </p>
                <?php
        endif;
        return ob_get_clean();
    }
}
