<?php
/**
 * A class for most visited pages table representation.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\Representation\Tables;

use First_Party_Analytics\Databases\Page_Visits as DB;
use First_Party_Analytics\First_Party_Analytics;
use First_Party_Analytics\Utils;
use First_Party_Analytics\WooCommerce\Pages_Tracking_Data;
use First_Party_Analytics\WooCommerce\Products_Tracking_Data;

/**
 * Class Most_Visited_Pages.
 */
final class Most_Visited_Pages extends Table {

    /**
     * Returns the list of the most visited pages.
     *
     * @param string $post_type  For which post type to get pages.
     * @param string $visit_type For which visit type to get pages.
     * @param int    $days_range For how many days to display pages.
     * @param int    $page       For which page to get the stuff.
     *
     * @return array
     */
    public static function get_most_visited_pages_list( $post_type, $visit_type, $days_range, $page ) {
        global $wpdb;

        $days_range = (int) $days_range;

        $visit_type = $visit_type . '_visits';

        $table_name = DB::get_table_name();

        $time      = time();
        $day_today = gmdate( 'Y-m-d', $time );
        $day_from  = 1 === $days_range ? $day_today : gmdate( 'Y-m-d', $time - DAY_IN_SECONDS * ( $days_range - 1 ) );

        // Get a bit more pages to know when to add pagination.
        $pages_limit = self::PAGES_LIMIT + 1;

        $post_type_query_string = $post_type ? $wpdb->prepare( 'post_type = %s AND', $post_type ) : '';

        $offset = ( $page - 1 ) * self::PAGES_LIMIT;

        $revision_where_clause = '';
        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision );
        }

        $results = $wpdb->get_results(
            $wpdb->prepare(
                "
                        SELECT SUM(%1s) as `visits_count`, a_url, post_id
                        FROM $table_name
                        WHERE
                            $post_type_query_string
                            a_day BETWEEN %s AND %s
                            $revision_where_clause
                        GROUP BY a_url
                        HAVING `visits_count` > 0
                        ORDER BY `visits_count` DESC
                        LIMIT %d OFFSET %d;
                    ",
                $visit_type,
                $day_from,
                $day_today,
                $pages_limit,
                $offset
            ),
            ARRAY_A
        );

        return $results;
    }

    /**
     * Prints dates.
     */
    public static function print_dates() {
        $dates = [
            1   => __( 'Last day', 'first-party-analytics' ),
            7   => __( 'Last 7 days', 'first-party-analytics' ),
            30  => __( 'Last 30 days', 'first-party-analytics' ),
            365 => __( 'Last 365 days', 'first-party-analytics' ),
        ];

        $default = 1;

        foreach ( $dates as $date_slug => $date_name ) {
            ?>
            <span class="setting-entity <?php echo $default === $date_slug ? 'selected default' : ''; ?>" data-val="<?php echo esc_attr( $date_slug ); ?>">
            <?php echo esc_attr( $date_name ); ?>
            </span>
            <?php
        }
    }

    /**
     * Prints post types.
     */
    public static function print_post_types() {

        $types = [
            'all'  => __( 'All Types' ),
            'post' => __( 'Post' ),
            'page' => __( 'Page' ),
        ];

        if ( class_exists( 'WooCommerce' ) ) {
            $types['product'] = __( 'Product', 'woocommerce' );
        }

        $default = 'all';

        foreach ( $types as $type_slug => $type_name ) {
            ?>
            <span class="setting-entity <?php echo $default === $type_slug ? 'selected default' : ''; ?>" data-val="<?php echo esc_attr( $type_slug ); ?>">
            <?php echo esc_attr( $type_name ); ?>
            </span>
            <?php
        }
    }

    /**
     * Prints visit types.
     */
    public static function print_visit_types() {

        $types = [
            'unique'   => __( 'Unique', 'first-party-analytics' ),
            'total'    => __( 'Any', 'first-party-analytics' ),
            'bot'      => __( 'Bot', 'first-party-analytics' ),
            'internal' => __( 'Internal', 'first-party-analytics' ),
            'incoming' => __( 'External', 'first-party-analytics' ),
        ];

        $default = 'unique';

        foreach ( $types as $type_slug => $type_name ) {
            ?>
            <span class="setting-entity <?php echo $default === $type_slug ? 'selected default' : ''; ?>" data-val="<?php echo esc_attr( $type_slug ); ?>">
            <?php echo esc_attr( $type_name ); ?>
            </span>
            <?php
        }
    }

    /**
     * Get the most visited pages table.
     *
     * @param bool   $return     Return or not.
     * @param string $post_type  For which post type to get pages.
     * @param string $visit_type For which visit type to get pages.
     * @param int    $days_range For how many days to display pages.
     * @param int    $page       For which page to get the stuff.
     *
     * @return string|void
     */
    public static function print_table( $return = false, $post_type = false, $visit_type = 'unique', $days_range = 1, $page = 1 ) {
        $pages_raw = self::get_most_visited_pages_list( $post_type, $visit_type, $days_range, $page );

        $additional_columns_data = [];

        if ( First_Party_Analytics::$premium_enabled ) {
            foreach ( $pages_raw as $page_raw ) {
                $additional_columns_data[ $page_raw['a_url'] ] = [ // Values when no ID.
                    'conversionsDepth' => '-',
                    'conversionsValue' => '-',
                ];

                if ( $page_raw['post_id'] ) {
                    $page_id = (int) $page_raw['post_id'];

                    $avg_conv_val = (string) Pages_Tracking_Data::get_average_conversion_depth( $page_id );

                    $additional_columns_data[ $page_raw['a_url'] ] = [
                        'conversionsDepth' =>
                            is_numeric( $avg_conv_val ) ?
                                ( str_contains( $avg_conv_val, '.' ) ? round( $avg_conv_val, 1 ) : $avg_conv_val . '.0' ) : $avg_conv_val,
                        'conversionsValue' => Pages_Tracking_Data::get_conversions_value( $page_id ),
                    ];
                }
            }
        }

        if ( 'product' === $post_type ) {
            foreach ( $pages_raw as $page_raw ) {
                if ( $page_raw['post_id'] ) {
                    $page_id = (int) $page_raw['post_id'];

                    $additional_columns_data[ $page_raw['a_url'] ] = array_merge(
                        isset( $additional_columns_data[ $page_raw['a_url'] ] ) ? $additional_columns_data[ $page_raw['a_url'] ] : [],
                        Products_Tracking_Data::get_all( $page_id )
                    );
                }
            }
        }

        $pages = array_column( $pages_raw, 'visits_count', 'a_url' );

        $content = self::get_table_content( $pages, $page, $post_type, $additional_columns_data );

        if ( $return ) {
            return $content;
        } else {
            echo Utils::esc_html_before_output( $content );
        }
    }

    /**
     * Returns the table content.
     *
     * @param array  $pages                   The list of pages, where key is page URL and the value is count.
     * @param int    $page                    For which page to get the stuff.
     * @param string $post_type               For which post type to print pages.
     * @param array  $additional_columns_data Additional columns data.
     */
    public static function get_table_content( $pages, $page = 1, $post_type = false, $additional_columns_data = [] ) {
        $add_load_more_btn = count( $pages ) > self::PAGES_LIMIT;
        $pages             = array_slice( $pages, 0, self::PAGES_LIMIT );

        ob_start();
        if ( $pages ) :
            ?>
            <div class="first-party-analytics-table-content">
                <table>
                    <thead>
                    <tr>
                        <th><?php esc_html_e( '#', 'first-party-analytics' ); ?></th>
                        <th><?php esc_html_e( 'URL', 'first-party-analytics' ); ?></th>
                        <th><?php esc_html_e( 'Visits', 'first-party-analytics' ); ?></th>
                        <?php
                        if ( First_Party_Analytics::$premium_enabled ) :
                            ?>
                            <th><?php esc_html_e( 'All-time average conversion depth', 'first-party-analytics' ); ?></th>
                            <th>
                                <?php
                                $currency = Utils::get_currency();
                                echo esc_html(
                                    __( 'All-time conversion value', 'first-party-analytics' ) . ( $currency ? ' (' . $currency . ')' : '' )
                                );
                                ?>
                            </th>
                            <?php
                        endif;
                        ?>
                        <?php
                        if ( 'product' === $post_type ) :
                            ?>
                            <th><?php esc_html_e( 'Times product viewed in catalog', 'first-party-analytics' ); ?></th>
                            <th><?php esc_html_e( 'Times product page viewed', 'first-party-analytics' ); ?></th>
                            <th><?php esc_html_e( 'Times added to cart', 'first-party-analytics' ); ?></th>
                            <th><?php esc_html_e( 'Times purchased', 'first-party-analytics' ); ?></th>
                            <th><?php esc_html_e( 'All-time purchased (including when the plugin was not enabled)', 'first-party-analytics' ); ?></th>
                            <?php
                        endif;
                        ?>
                    </tr>
                    </thead>
                    <tbody>
                <?php
                $i = ( $page - 1 ) * self::PAGES_LIMIT;

                foreach ( $pages as $url => $visits ) :
                    $i++;
                    ?>
                        <tr>
                            <td><?php echo esc_html( $i ); ?></td>
                            <td><a href="<?php echo esc_url( $url ); ?>" target=”_blank”><?php echo esc_html( $url ); ?></a></td>
                            <td><?php echo esc_html( $visits ); ?></td>
                            <?php
                            if ( First_Party_Analytics::$premium_enabled ) :
                                ?>
                                    <td><?php echo esc_html( $additional_columns_data[ $url ]['conversionsDepth'] ); ?></td>
                                    <td><?php echo esc_html( $additional_columns_data[ $url ]['conversionsValue'] ); ?></td>
                                <?php
                            endif;
                            ?>
                            <?php
                            if ( 'product' === $post_type ) :
                                ?>
                                <td><?php echo esc_html( $additional_columns_data[ $url ]['viewedInCatalog'] ); ?></td>
                                <td><?php echo esc_html( $additional_columns_data[ $url ]['productViewsCount'] ); ?></td>
                                <td><?php echo esc_html( $additional_columns_data[ $url ]['addedToCartCount'] ); ?></td>
                                <td><?php echo esc_html( $additional_columns_data[ $url ]['purchasedCount'] ); ?></td>
                                <td><?php echo esc_html( $additional_columns_data[ $url ]['purchasedAllTimeCount'] ); ?></td>
                                <?php
                            endif;
                            ?>
                        </tr>
                    <?php endforeach; ?>
                    </tbody>
                </table>
            </div>
                <?php if ( $add_load_more_btn ) : ?>
                <button data-next-page="<?php echo esc_attr( $page + 1 ); ?>" class="first-party-analytics-load-more-btn">
                    <?php esc_attr_e( 'Load more', 'first-party-analytics' ); ?>
                </button>
            <?php endif; ?>
                    <?php
            else :
                ?>
            <p class="no-data-here-yet-block">
                <?php esc_attr_e( 'No data yet.', 'first-party-analytics' ); ?>
            </p>
                <?php
            endif;
            return ob_get_clean();
    }
}
