<?php
/**
 * A class for referers table representation.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\Representation\Tables;

use First_Party_Analytics\Databases\Visitor_Details as DB;
use First_Party_Analytics\Utils;

/**
 * Class Referers.
 */
final class Referers extends Table {

    /**
     * Returns the list of referers.
     *
     * @param int    $page         For which page to get the stuff.
     * @param string $display_type Display type.
     *
     * @return array
     */
    public static function get_referers_list( $page = 1, $display_type = 'full' ) {
        global $wpdb;

        $table_name = DB::get_table_name();

        // Get a bit more pages to know when to add pagination.
        $pages_limit = self::PAGES_LIMIT + 1;

        $offset = ( $page - 1 ) * self::PAGES_LIMIT;

        $revision_where_clause = '';
        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision );
        }

        $results = $wpdb->get_results(
                $wpdb->prepare(
                    "
                        SELECT SUM(visits_count) as `visits_count`, a_value
                        FROM $table_name
                        WHERE a_key = 'referer-$display_type' AND a_value != '' $revision_where_clause
                        GROUP BY a_value
                        ORDER BY `visits_count` DESC
                        LIMIT %d OFFSET %d;
                    ",
                    $pages_limit,
                    $offset
                ),
            ARRAY_A
        );

        return $results;
    }

    /**
     * Prints referer display types.
     */
    public static function print_display_types() {
        $types = [
            'full'                => __( 'Full', 'first-party-analytics' ),
            'host'                => __( 'Domain', 'first-party-analytics' ),
            'without-subdomain'   => __( 'Domain without subdomain', 'first-party-analytics' ),
            'second-level-domain' => __( 'Main domain', 'first-party-analytics' ),
        ];

        $default = 'full';

        foreach ( $types as $type_slug => $type_name ) {
            ?>
            <span class="setting-entity <?php echo $default === $type_slug ? 'selected default' : ''; ?>" data-val="<?php echo esc_attr( $type_slug ); ?>">
            <?php echo esc_attr( $type_name ); ?>
            </span>
            <?php
        }
    }

    /**
     * Get the most visited pages table.
     *
     * @param bool   $return       Return or not.
     * @param int    $page         For which page to get the stuff.
     * @param string $display_type Display type.
     *
     * @return string|void
     */
    public static function print_table( $return = false, $page = 1, $display_type = 'full' ) {
        $content = self::get_table_content(
            array_column(  self::get_referers_list( $page, $display_type ), 'visits_count', 'a_value' ),
            $page
        );

        if ( $return ) {
            return $content;
        } else {
            echo Utils::esc_html_before_output( $content );
        }
    }

    /**
     * Returns the table content.
     *
     * @param array $referers The list of referers, where key is visits count and the value is the referer.
     * @param int   $page     For which page to get the stuff.
     */
    public static function get_table_content( $referers, $page = 1  ) {
        $add_load_more_btn = count( $referers ) > self::PAGES_LIMIT;
        $referers          = array_slice( $referers, 0, self::PAGES_LIMIT );

        ob_start();
        if ( $referers ) :
            ?>
            <div class="first-party-analytics-table-content">
                <table>
                    <thead>
                    <tr>
                        <th><?php esc_html_e( '#', 'first-party-analytics' ); ?></th>
                        <th><?php esc_html_e( 'Referrer', 'first-party-analytics' ); ?></th>
                        <th><?php esc_html_e( 'Visits', 'first-party-analytics' ); ?></th>
                    </tr>
                    </thead>
                    <tbody>
                <?php
                $i = ( $page - 1 ) * self::PAGES_LIMIT;;
                foreach ( $referers as $referer => $visits ) :
                    $i++;
                    ?>
                        <tr>
                            <td><?php echo esc_html( $i ); ?></td>
                            <td><?php echo esc_html( $referer ); ?></a></td>
                            <td><?php echo esc_html( $visits ); ?></td>
                        </tr>
                    <?php endforeach; ?>
                    </tbody>
                </table>
            </div>
                <?php if ( $add_load_more_btn ) : ?>
                <button data-next-page="<?php echo esc_attr( $page + 1 ); ?>" class="first-party-analytics-load-more-btn">
                    <?php esc_attr_e( 'Load more', 'first-party-analytics' ); ?>
                </button>
            <?php endif; ?>
                    <?php
            else :
                ?>
            <p class="no-data-here-yet-block">
                <?php esc_attr_e( 'No data yet.', 'first-party-analytics' ); ?>
            </p>
                <?php
            endif;
            return ob_get_clean();
    }
}
