<?php
/**
 * A class for searches table representation.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\Representation\Tables;

use First_Party_Analytics\Databases\Search_Words as DB;
use First_Party_Analytics\Utils;

/**
 * Class Searches.
 */
final class Searches extends Table {

    /**
     * Returns the list of searches.
     *
     * @param int    $page        For which page to get the stuff.
     * @param string $search_type For which search type to get pages.
     *
     * @return array
     */
    public static function get_searches_list( $page = 1, $search_type = false ) {
        global $wpdb;

        $table_name = DB::get_table_name();

        // Get a bit more pages to know when to add pagination.
        $pages_limit = self::PAGES_LIMIT + 1;

        $offset = ( $page - 1 ) * self::PAGES_LIMIT;

        $search_type_string = ( $search_type && 'all' !== $search_type ) ?
            'WHERE full_search = ' . ( ( 'words' !== $search_type ) ? '1' : '0' ) : '';

        $select_string = 'individual' !== $search_type ?
            'SELECT word, COUNT(*) AS `count`' :
            'SELECT time, word, page_url, input_selector';

        $group_by_string = 'individual' !== $search_type ?
            'GROUP BY word' :
            '';

        $order_by_string = 'individual' !== $search_type ?
            'ORDER BY `count` DESC' :
            'ORDER BY `time` DESC';

        $revision_where_clause = '';
        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $revision_where_clause = ( $search_type_string ? 'AND' : 'WHERE' ) . $wpdb->prepare( ' revision_id = %d', $revision );
        }

        $results = $wpdb->get_results(
            $wpdb->prepare(
                "
                        $select_string
                        FROM $table_name
                        $search_type_string
                        $revision_where_clause
                        $group_by_string
                        $order_by_string
                        LIMIT %d OFFSET %d;
                    ",
                $pages_limit,
                $offset
            ),
            ARRAY_A
        );

        return $results;
    }

    /**
     * Prints search types (all/internal/external).
     */
    public static function print_search_types() {
        $types = [
            'all'        => __( 'All', 'first-party-analytics' ),
            'words'      => __( 'Words', 'first-party-analytics' ),
            'full_text'  => __( 'Full text', 'first-party-analytics' ),
            'individual' => __( 'Individual', 'first-party-analytics' ),
        ];

        $default = 'all';

        foreach ( $types as $type_slug => $type_name ) {
            ?>
            <span class="setting-entity <?php echo $default === $type_slug ? 'selected default' : ''; ?>" data-val="<?php echo esc_attr( $type_slug ); ?>">
            <?php echo esc_attr( $type_name ); ?>
            </span>
            <?php
        }
    }

    /**
     * Get the most visited pages table.
     *
     * @param bool   $return           Return or not.
     * @param int    $page             For which page to get the stuff.
     * @param string $search_type      For which search type to get pages.
     *
     * @return string|void
     */
    public static function print_table( $return = false, $page = 1, $search_type = false ) {
        $searches_list = self::get_searches_list( $page, $search_type );

        $formatted_searches = 'individual' !== $search_type
            ? array_column( $searches_list, 'count', 'word' ) : $searches_list;

        $content = self::get_table_content(
            $formatted_searches,
            $page,
            $search_type
        );

        if ( $return ) {
            return $content;
        } else {
            echo Utils::esc_html_before_output( $content );
        }
    }

    /**
     * Returns the table content.
     *
     * @param array  $searches    The list of searches, where key is count and the value is the word.
     * @param int    $page        For which page to get the stuff.
     * @param string $search_type For which search type to get pages.
     */
    public static function get_table_content( $searches, $page = 1, $search_type = false ) {
        $add_load_more_btn = count( $searches ) > self::PAGES_LIMIT;
        $searches          = array_slice( $searches, 0, self::PAGES_LIMIT );

        ob_start();
        if ( $searches ) :
            ?>
            <div class="first-party-analytics-table-content mt-0">
                <table>
                    <thead>
                    <tr>
                        <th><?php esc_html_e( '#', 'first-party-analytics' ); ?></th>
                            <?php
                            if ( 'individual' === $search_type ) :
                                ?>
                                <th><?php esc_html_e( 'Time', 'first-party-analytics' ); ?></th>
                                <?php
                            endif;
                            ?>
                            <th>
                                <?php
                                if ( 'full_text' === $search_type ) {
                                    esc_html_e( 'Search text', 'first-party-analytics' );
                                } elseif ( 'words' === $search_type ) {
                                    esc_html_e( 'Search word', 'first-party-analytics' );
                                } else {
                                    esc_html_e( 'Search', 'first-party-analytics' );
                                }
                                ?>
                            </th>
                            <?php
                            if ( 'individual' !== $search_type ) :
                                ?>
                                <th><?php esc_html_e( 'Count', 'first-party-analytics' ); ?></th>
                                <?php
                            else :
                                ?>
                                <th><?php esc_html_e( 'Search source', 'first-party-analytics' ); ?></th>
                                <?php
                                endif;
                            ?>
                    </tr>
                    </thead>
                    <tbody>
                        <?php
                        $i = ( $page - 1 ) * self::PAGES_LIMIT;

                        if ( 'individual' !== $search_type ) :
                            foreach ( $searches as $word => $count ) :
                                $i++;
                                ?>
                                    <tr>
                                        <td><?php echo esc_html( $i ); ?></td>
                                        <td><?php echo esc_html( $word ); ?></a></td>
                                        <td><?php echo esc_html( $count ); ?></td>
                                    </tr>
                                <?php
                            endforeach;
                        else :
                            foreach ( $searches as $search_data ) :
                                $i++;
                                ?>
                                <tr data-page="<?php echo esc_attr( $search_data['page_url'] ); ?>"
                                    data-input-selector="<?php echo esc_attr( $search_data['input_selector'] ); ?>">
                                        <td><?php echo esc_html( $i ); ?></td>
                                        <td><?php echo esc_html( $search_data['time'] ); ?></a></td>
                                        <td><?php echo esc_html( $search_data['word'] ); ?></td>
                                        <td class="first-party-analytics-view-search-source-btn">
                                            <?php esc_attr_e( 'View search source', 'first-party-analytics' ); ?>
                                        </td>
                                </tr>
                                <?php
                            endforeach;
                        endif;
                        ?>
                    </tbody>
                </table>
            </div>
            <?php if ( $add_load_more_btn ) : ?>
                <button data-next-page="<?php echo esc_attr( $page + 1 ); ?>" class="first-party-analytics-load-more-btn">
                    <?php esc_attr_e( 'Load more', 'first-party-analytics' ); ?>
                </button>
            <?php endif; ?>
            <?php else : ?>
                <p class="no-data-here-yet-block">
                    <?php esc_attr_e( 'No data yet.', 'first-party-analytics' ); ?>
                </p>
                <?php
            endif;
            return ob_get_clean();
    }
}
