<?php
/**
 * A class for user-agents table representation.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\Representation\Tables;

use First_Party_Analytics\Databases\Visitor_Details as DB;
use First_Party_Analytics\Utils;

/**
 * Class User_Agents.
 */
final class User_Agents extends Table {

    /**
     * Returns the list of User-Agents.
     *
     * @param int $page For which page to get the stuff.
     *
     * @return array
     */
    public static function get_user_agents_list( $page = 1 ) {
        global $wpdb;

        $table_name = DB::get_table_name();

        // Get a bit more pages to know when to add pagination.
        $pages_limit = self::PAGES_LIMIT + 1;

        $offset = ( $page - 1 ) * self::PAGES_LIMIT;

        $revision_where_clause = '';
        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision );
        }

        $results = $wpdb->get_results(
                $wpdb->prepare(
                    "
                        SELECT SUM(visits_count) as `visits_count`, a_value
                        FROM $table_name
                        WHERE a_key = 'user-agent' $revision_where_clause
                        GROUP BY a_value
                        ORDER BY `visits_count` DESC
                        LIMIT %d OFFSET %d;
                    ",
                    $pages_limit,
                    $offset
                ),
            ARRAY_A
        );

        return $results;
    }

    /**
     * Get the most visited pages table.
     *
     * @param bool $return Return or not.
     * @param int  $page   For which page to get the stuff.
     *
     * @return string|void
     */
    public static function print_table( $return = false, $page = 1 ) {
        $content = self::get_table_content(
            array_column( self::get_user_agents_list( $page ), 'visits_count', 'a_value' ),
            $page
        );

        if ( $return ) {
            return $content;
        } else {
            echo Utils::esc_html_before_output( $content );
        }
    }

    /**
     * Returns the table content.
     *
     * @param array $user_agents The list of user agents, where key is visits count and the value is the user agent.
     * @param int   $page        For which page to get the stuff.
     */
    public static function get_table_content( $user_agents, $page = 1  ) {
        $add_load_more_btn = count( $user_agents ) > self::PAGES_LIMIT;
        $user_agents       = array_slice( $user_agents, 0, self::PAGES_LIMIT );

        ob_start();
        if ( $user_agents ) :
            ?>
            <div class="first-party-analytics-table-content">
                <table>
                    <thead>
                    <tr>
                        <th><?php esc_html_e( '#', 'first-party-analytics' ); ?></th>
                        <th><?php esc_html_e( 'User-Agent', 'first-party-analytics' ); ?></th>
                        <th><?php esc_html_e( 'Visits', 'first-party-analytics' ); ?></th>
                    </tr>
                    </thead>
                    <tbody>
                <?php
                $i = ( $page - 1 ) * self::PAGES_LIMIT;;
                foreach ( $user_agents as $user_agent => $visits ) :
                    $i++;
                    ?>
                        <tr>
                            <td><?php echo esc_html( $i ); ?></td>
                            <td><?php echo esc_html( $user_agent ); ?></a></td>
                            <td><?php echo esc_html( $visits ); ?></td>
                        </tr>
                    <?php endforeach; ?>
                    </tbody>
                </table>
            </div>
                <?php if ( $add_load_more_btn ) : ?>
                <button data-next-page="<?php echo esc_attr( $page + 1 ); ?>" class="first-party-analytics-load-more-btn">
                    <?php esc_attr_e( 'Load more', 'first-party-analytics' ); ?>
                </button>
            <?php endif; ?>
                    <?php
            else :
                ?>
            <p class="no-data-here-yet-block">
                <?php esc_attr_e( 'No data yet.', 'first-party-analytics' ); ?>
            </p>
                <?php
        endif;
        return ob_get_clean();
    }
}
