<?php
/**
 * A class for visited links table representation.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\Representation\Tables;

use First_Party_Analytics\Databases\Link_Visits as DB;
use First_Party_Analytics\Utils;

/**
 * Class Visited_Links.
 */
final class Visited_Links extends Table {

    /**
     * Returns the list of the visited links.
     *
     * @param string $dst_type   For which dst type to get pages.
     * @param int    $days_range For how many days to display pages.
     * @param int    $page       For which page to get the stuff.
     *
     * @return array
     */
    public static function get_visited_links_list( $dst_type, $days_range, $page ) {
        global $wpdb;

        $days_range = (int) $days_range;

        $table_name = DB::get_table_name();

        $time      = time();
        $time_now  = gmdate( 'Y-m-d H:i:s', $time );
        $time_from = gmdate( 'Y-m-d H:i:s', $time - DAY_IN_SECONDS * $days_range );

        // Get a bit more pages to know when to add pagination.
        $pages_limit = self::PAGES_LIMIT + 1;

        $type_query_string = 'all' !== $dst_type ? $wpdb->prepare( 'external_dst = %d AND', (int) ( 'external' === $dst_type ) ) : '';

        $offset = ( $page - 1 ) * self::PAGES_LIMIT;

        $revision_where_clause = '';
        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision );
        }

        $results = $wpdb->get_results(
            $wpdb->prepare(
                "
                        SELECT `time`, `source`, href
                        FROM $table_name
                        WHERE
                            $type_query_string
                            `time` BETWEEN %s AND %s
                            $revision_where_clause
                        ORDER BY `time` DESC
                        LIMIT %d OFFSET %d;
                    ",
                $time_from,
                $time_now,
                $pages_limit,
                $offset
            ),
            ARRAY_A
        );

        return $results;
    }

    /**
     * Prints dates.
     */
    public static function print_dates() {
        $dates = [
            1   => __( 'Last 24h', 'first-party-analytics' ),
            7   => __( 'Last 7 days', 'first-party-analytics' ),
            30  => __( 'Last 30 days', 'first-party-analytics' ),
            365 => __( 'Last 365 days', 'first-party-analytics' ),
        ];

        $default = 1;

        foreach ( $dates as $date_slug => $date_name ) {
            ?>
            <span class="setting-entity <?php echo $default === $date_slug ? 'selected default' : ''; ?>" data-val="<?php echo esc_attr( $date_slug ); ?>">
            <?php echo esc_attr( $date_name ); ?>
            </span>
            <?php
        }
    }

    /**
     * Prints link destination types (all/internal/external).
     */
    public static function print_destination_types() {
        $types = [
            'all'      => __( 'All', 'first-party-analytics' ),
            'internal' => __( 'Internal', 'first-party-analytics' ),
            'external' => __( 'External', 'first-party-analytics' ),
        ];

        $default = 'all';

        foreach ( $types as $type_slug => $type_name ) {
            ?>
            <span class="setting-entity <?php echo $default === $type_slug ? 'selected default' : ''; ?>" data-val="<?php echo esc_attr( $type_slug ); ?>">
            <?php echo esc_attr( $type_name ); ?>
            </span>
            <?php
        }
    }

    /**
     * Get the most visited pages table.
     *
     * @param bool   $return     Return or not.
     * @param string $dst_type   For which dst type to get pages.
     * @param int    $days_range For how many days to display pages.
     * @param int    $page       For which page to get the stuff.
     *
     * @return string|void
     */
    public static function print_table( $return = false, $dst_type = false, $days_range = 1, $page = 1 ) {
        $content = self::get_table_content(
            self::get_visited_links_list( $dst_type, $days_range, $page ),
            $page
        );

        if ( $return ) {
            return $content;
        } else {
            echo Utils::esc_html_before_output( $content );
        }
    }

    /**
     * Returns the table content.
     *
     * @param array $visited_links The list of visited links.
     * @param int   $page          For which page to get the stuff.
     */
    public static function get_table_content( $visited_links, $page = 1 ) {
        $add_load_more_btn = count( $visited_links ) > self::PAGES_LIMIT;
        $visited_links     = array_slice( $visited_links, 0, self::PAGES_LIMIT );

        ob_start();
        if ( $visited_links ) :
            ?>
            <div class="first-party-analytics-table-content">
                <table>
                    <thead>
                    <tr>
                        <th><?php esc_html_e( '#', 'first-party-analytics' ); ?></th>
                        <th><?php esc_html_e( 'Time', 'first-party-analytics' ); ?></th>
                        <th><?php esc_html_e( 'Source', 'first-party-analytics' ); ?></th>
                        <th><?php esc_html_e( 'Destination', 'first-party-analytics' ); ?></th>
                    </tr>
                    </thead>
                    <tbody>
                    <?php
                    $i = ( $page - 1 ) * self::PAGES_LIMIT;

                    foreach ( $visited_links as $visited_link ) :
                        $i++;
                        ?>
                        <tr>
                            <td><?php echo esc_html( $i ); ?></td>
                            <td><?php echo esc_html( $visited_link['time'] ); ?></td>
                            <td>
                                <a href="<?php echo esc_url( $visited_link['source'] ); ?>" target=”_blank”>
                                    <?php echo esc_html( $visited_link['source'] ); ?>
                                </a>
                            </td>
                            <td>
                                <a href="<?php echo esc_url( $visited_link['href'] ); ?>" target=”_blank”>
                                    <?php echo esc_html( $visited_link['href'] ); ?>
                                </a>
                            </td>
                        </tr>
                    <?php endforeach; ?>
                    </tbody>
                </table>
            </div>
            <?php if ( $add_load_more_btn ) : ?>
            <button data-next-page="<?php echo esc_attr( $page + 1 ); ?>" class="first-party-analytics-load-more-btn">
                <?php esc_attr_e( 'Load more', 'first-party-analytics' ); ?>
            </button>
                <?php
        endif;
        else :
            ?>
            <p class="no-data-here-yet-block">
                <?php esc_attr_e( 'No data yet.', 'first-party-analytics' ); ?>
            </p>
            <?php
        endif;
        return ob_get_clean();
    }
}
