<?php
/**
 * A class for visits by sources table representation.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\Representation\Tables;

use First_Party_Analytics\Databases\Visitor_Details as Visitor_Details_DB;
use First_Party_Analytics\Databases\Special_Url_Params as Special_Url_Param_DB;
use First_Party_Analytics\Sources;
use RecursiveArrayIterator;
use RecursiveIteratorIterator;

/**
 * Class Visits_By_Sources.
 */
final class Visits_By_Sources extends Table {

    /**
     * Default selector option.
     */
    const DEFAULT_DATE_RANGE = 'all-time';

    /**
     * Returns the list.
     *
     * @param bool|array $sources                                     Sources.
     * @param string     $date_range                                  Date range.
     * @param bool       $get_utm_tags                                Get UTM tags instead of referers.
     * @param bool       $accumulate_in_parents_children_referers     Whether to accumulate in parent children referers.
     * @param bool       $do_not_return_empty_values_for_visits_list  Do not return empty values.
     * @param bool       $add_direct_visits                           If to add direct visits.
     *
     * @return array Referers OR UTM tags.
     */
    public static function get_the_list(
            $sources = false,
            $date_range = self::DEFAULT_DATE_RANGE,
            $get_utm_tags = false,
            $accumulate_in_parents_children_referers = true,
            $do_not_return_empty_values_for_visits_list = true,
            $add_direct_visits = false
    ) {
        global $wpdb;

        $args = wp_json_encode( func_get_args() );

        // Cache.
        static $cache = [];
        if ( isset( $cache[ $args ] ) ) {
            return $cache[ $args ];
        }

        $visitor_details_table = Visitor_Details_DB::get_table_name();

        $list         = [];
        $sources_list = Sources::get_sources_list( $sources, $accumulate_in_parents_children_referers, [], false, true );

        $all_time_txt = __( 'All-time', 'first-party-analytics' );

        // Get the list of dates.

        if ( 'all-time' !== $date_range ) {
            $date_period = str_contains( $date_range, 'day' ) ? DAY_IN_SECONDS : ( str_contains( $date_range, 'week' ) ? WEEK_IN_SECONDS : MONTH_IN_SECONDS );
            $range       = (int) filter_var( $date_range, FILTER_SANITIZE_NUMBER_INT );

            $time      = time();
            $day_today = gmdate( 'Y-m-d', $time );
            $day_from  = gmdate( 'Y-m-d', $time - $date_period * $range );

            // Do year-week for weeks to sync the dates.
            // And month without a day.
            $day_select = str_contains( $date_range, 'day' ) ? 'a_day' :
                ( str_contains( $date_range, 'week' ) ? "STR_TO_DATE(CONCAT(YEARWEEK(a_day), 'Sunday'), '%X%V %W')" : 'DATE_FORMAT(a_day, "%Y-%m")' );

            $group_by = str_contains( $date_range, 'day' ) ? 'day' : ( str_contains( $date_range, 'week' ) ? 'YEARWEEK(day)' : 'day' );

            static $dates_were_fetched = false;
            static $dates              = [];

            if ( ! $dates_were_fetched ) {
                $revision_where_clause = '';
                if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
                    $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
                    $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision );
                }

                $dates              = $wpdb->get_results(
                    $wpdb->prepare(
                        "
                            SELECT $day_select as `day`
                            FROM $visitor_details_table
                            WHERE 
                                (
                                    a_key = 'referer-host' OR
                                    a_key = 'referer-second-level-domain'
                                ) AND
                                a_day BETWEEN %s AND %s $revision_where_clause
                            GROUP BY $group_by
                            ORDER BY `day`;
                            ",
                        $day_from,
                        $day_today
                    ),
                    ARRAY_N
                );
                $dates_were_fetched = true;

                $dates_array = [];
                foreach ( $dates as $date ) {
                    $dates_array[ $date[0] ] = [];
                }
                $dates = $dates_array;
            }

            if ( ! $dates ) {
                $cache[ $args ] = [];
                return [];
            }
        }
        $special_url_params_table_name = Special_Url_Param_DB::get_table_name();

        // Get referers or UTM tags.

        foreach ( $sources_list as $source_referers_list ) {
            if ( $source_referers_list ) {

                // Get source referers per host and per main domain (that are ending like google.*).

                $source_referers_host        = [];
                $source_referers_main_domain = [];

                foreach ( $source_referers_list as $referer ) {
                    if ( ! str_ends_with( $referer, '.*' ) ) {
                        $source_referers_host[] = $referer;
                    } else {
                        $source_referers_main_domain[] = substr( $referer, 0, -2 );
                    }
                }

                $placeholders_host = count( $source_referers_host ) ?
                    implode( ',', array_fill( 0, count( $source_referers_host ), '%s' ) ) : 'NULL';

                $placeholders_main_domain = count( $source_referers_main_domain ) ?
                    implode( ',', array_fill( 0, count( $source_referers_main_domain ), '%s' ) ) : 'NULL';

                if ( 'all-time' === $date_range ) {
                    if ( ! $get_utm_tags ) { // Get referers.
                        $revision_where_clause = '';
                        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
                            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
                            $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision );
                        }

                        $result = $wpdb->get_var( 
                            $wpdb->prepare(
                                "
                            SELECT SUM(visits_count)
                            FROM $visitor_details_table
                            WHERE 
                                (
                                    (
                                        a_key = 'referer-host' AND
                                        a_value IN ($placeholders_host)
                                    )
                                    OR
                                    (
                                        a_key = 'referer-second-level-domain' AND
                                        a_value IN ($placeholders_main_domain)
                                    )
                                )
                                $revision_where_clause;
                            ",
                                ...$source_referers_host,
                                ...$source_referers_main_domain
                            )
                        );
                        $list[] = [ $all_time_txt => (int) $result ];
                    } else { // Get UTM tags.
                        $revision_where_clause = '';
                        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
                            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
                            $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision );
                        }

                        $utm_tags = $wpdb->get_var( 
                            $wpdb->prepare(
                                "SELECT CONCAT('{', GROUP_CONCAT(J5), '}')
        FROM (
            SELECT CONCAT('\"', utm_medium, '\":{', GROUP_CONCAT(J4), '}') as J5
            FROM (
                SELECT utm_medium, CONCAT('\"', utm_source, '\":{', GROUP_CONCAT(J3), '}') as J4
                FROM (
                    SELECT utm_medium, utm_source, CONCAT('\"', utm_campaign, '\":{', GROUP_CONCAT(J2), '}') as J3
                        FROM (
                        SELECT utm_medium, utm_source, utm_campaign, CONCAT('\"', utm_term, '\":{', GROUP_CONCAT(J1), '}') as J2
                        FROM (
                            SELECT utm_medium, utm_source, utm_campaign, utm_term, CONCAT('\"', utm_content, '\":', COUNT(*) ) as J1
                            FROM $special_url_params_table_name
                            WHERE referer_host IN ($placeholders_host) OR
                                  referer_main_domain IN ($placeholders_main_domain)
                                  $revision_where_clause
                            GROUP BY utm_medium, utm_source, utm_campaign, utm_term, utm_content
                        ) as Q1
                        GROUP BY utm_medium, utm_source, utm_campaign, utm_term
                    ) AS Q2
                    GROUP BY utm_medium, utm_source, utm_campaign
                ) AS Q3
                GROUP BY utm_medium, utm_source
            ) Q4
            GROUP BY utm_medium
        ) Q5;",
                                ...$source_referers_host,
                                ...$source_referers_main_domain
                            )
                        );

                        $list[] = $utm_tags ? json_decode( $utm_tags, true ) : [];
                    }
                } else {
                    $a_list = [];

                    if ( ! $get_utm_tags ) { // Get referers.
                        $revision_where_clause = '';
                        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
                            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
                            $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision );
                        }

                        $results = array_column(
                            $wpdb->get_results(
                                $wpdb->prepare(
                                    "
                                SELECT $day_select as `day`, SUM(visits_count)
                                FROM $visitor_details_table
                                WHERE
                                    (
                                        (
                                            a_key = 'referer-host' AND
                                            a_value IN ($placeholders_host)
                                        )
                                        OR
                                        (
                                            a_key = 'referer-second-level-domain' AND
                                            a_value IN ($placeholders_main_domain)
                                        )
                                    ) AND
                                    a_day BETWEEN %s AND %s
                                    $revision_where_clause
                                GROUP BY $group_by
                                ORDER BY `day`;
                            ",
                                    ...$source_referers_host,
                                    ...$source_referers_main_domain,
                                    ...[ $day_from, $day_today ]
                                ),
                                ARRAY_N
                            ),
                            1,
                            0
                        );

                        foreach ( array_keys( $dates ) as $date ) {
                            $a_list[ $date ] = isset( $results[ $date ] ) ? (int) $results[ $date ] : 0;
                        }
                    } else { // Get UTM tags.
                        $revision_where_clause = '';
                        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
                            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
                            $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision );
                        }

                        $utm_tags_per_date = array_column(
                            $wpdb->get_results( 
                                $wpdb->prepare(
                                    "SELECT `day`, CONCAT('{', GROUP_CONCAT(J5), '}')
            FROM (
                SELECT `day`, CONCAT('\"', utm_medium, '\":{', GROUP_CONCAT(J4), '}') as J5
                FROM (
                    SELECT `day`, utm_medium, CONCAT('\"', utm_source, '\":{', GROUP_CONCAT(J3), '}') as J4
                    FROM (
                        SELECT `day`, utm_medium, utm_source, CONCAT('\"', utm_campaign, '\":{', GROUP_CONCAT(J2), '}') as J3
                            FROM (
                            SELECT `day`, utm_medium, utm_source, utm_campaign, CONCAT('\"', utm_term, '\":{', GROUP_CONCAT(J1), '}') as J2
                            FROM (
                                SELECT $day_select as `day`, utm_medium, utm_source, utm_campaign, utm_term, CONCAT('\"', utm_content, '\":', COUNT(*) ) as J1
                                FROM $special_url_params_table_name
                                WHERE (referer_host IN ($placeholders_host) OR
                                      referer_main_domain IN ($placeholders_main_domain)) AND
                                      a_day BETWEEN %s AND %s
                                      $revision_where_clause
                                GROUP BY a_day, utm_medium, utm_source, utm_campaign, utm_term, utm_content
                                ORDER BY `day`
                            ) as Q1
                            GROUP BY $group_by, utm_medium, utm_source, utm_campaign, utm_term
                        ) AS Q2
                        GROUP BY $group_by, utm_medium, utm_source, utm_campaign
                    ) AS Q3
                    GROUP BY $group_by, utm_medium, utm_source
                ) Q4
                GROUP BY $group_by, utm_medium
            ) Q5
            GROUP BY $group_by;",
                                    ...$source_referers_host,
                                    ...$source_referers_main_domain,
                                    ...[ $day_from, $day_today ]
                                ),
                                ARRAY_N
                            ),
                            1,
                            0
                        );

                        foreach ( array_keys( $dates ) as $date ) {
                            $a_list[ $date ] = isset( $utm_tags_per_date[ $date ] ) ? json_decode( $utm_tags_per_date[ $date ], true ) : [];
                        }
                    }
                    $list[] = $a_list;
                }
            } else {
                if ( 'all-time' === $date_range ) {
                    $list[] = ! $get_utm_tags ? [ $all_time_txt => 0 ] : [];
                } else {
                    $a_list = [];

                    foreach ( array_keys( $dates ) as $date ) {
                        $a_list[ $date ] = ! $get_utm_tags ? 0 : [];
                    }

                    $list[] = $a_list;
                }
            }
        }

        // Do not return empty values (where all dates are zero).

        $list = array_filter(
            $list,
            function( $list_item ) use ( $get_utm_tags ) {
                if ( ! $get_utm_tags ) {
                    return 0 !== array_sum( $list_item );
                } else {
                    return 0 !== count( $list_item );
                }
            }
        );

        $cache[ $args ] = $list;
        return $list;
    }

    /**
     * Get the most visited pages table.
     *
     * @param bool|array $sources    Sources.
     * @param string     $date_range Date range.
     * @param bool       $get_utm_tags Get UTM tags instead of referers.
     *
     * @return array|false|string
     */
    public static function print_table( $sources = false, $date_range = self::DEFAULT_DATE_RANGE, $get_utm_tags = true ) {
        $visits_list = self::get_the_list( $sources, $date_range, false, true, true, true );

        $utm_tags = self::get_the_list( false, $date_range, $get_utm_tags, false );

        /*
         * Do not show UTM tags that do not have visitors in visitor-details table (for some unknown reason).
         *
         * So there can be UTM tags data (special_url_params) that do not have any related referers in visitor_details table.
         *
         * The reason of this behavior is that because we do not record visitor details for duplicate visits (not-unique), but do record URL params for them.
         *
         * Probably this part of logic can be improved because we are missing some data to show (when no referer recorded for the second visit but URL params are).
         */
        foreach ( $utm_tags as $num => $data ) {
            if ( ! array_key_exists( $num, $visits_list ) ) {
                unset( $utm_tags[ $num ] );
            }
        }

        /*
         * Use recursive iterator to build the list of UTM-tags.
         *
         * Contains an array like:
         *
         * [
         *   5                         => [ 'All-time' => 13 ], <- Standard value from visits count.
         *  '5 / Ads'                  => [ 'All-time' => 5 ],  <- Based on UTM tags.
         *  '5 / Ads / cpc'            => [ 'All-time' => 5 ],
         *  '5 / Ads / cpc / facebook' => [ 'All-time' => 5 ]
         * ]
         *
         */

        if ( ! $visits_list ) {
            return self::get_table_content( [] );
        }

        // Visits for UTM tags for all indexes (dates).
        $the_visits_with_utm_tags = [];

        $dates = array_keys( $visits_list[0] );

        foreach ( $dates as $index ) {
            $visits_with_utm_tags = [];
            $array                = self::DEFAULT_DATE_RANGE !== $date_range ? array_column( $utm_tags, $index ) : $utm_tags;
            $iterator             = new RecursiveIteratorIterator(
                new RecursiveArrayIterator( $array ),
                RecursiveIteratorIterator::SELF_FIRST
            );
            foreach ( $iterator as $v ) {
                for (
                    $path_elements = [], $i = 0, $z = $iterator->getDepth();
                    $i <= $z;
                    $i ++
                ) {
                    if ( $v && 1 === $i ) {
                        $path_elements[]               = 'Ads';
                        $path                          = implode( ' / ', $path_elements );
                        $visits_with_utm_tags[ $path ] = [ $index => 0 ];
                    }

                    $path_elements[] = $iterator->getSubIterator( $i )->key();
                }
                $path = implode( ' / ', $path_elements );

                $visits_with_utm_tags[ $path ] = $iterator->hasChildren() ? [ $index => 0 ] : [ $index => $v ];
            }

            // Sum counters of children to parents.

            foreach ( array_reverse( $visits_with_utm_tags, true ) as $k => $v ) {
                $path_elements = explode( ' / ', $k );
                $depth         = count( $path_elements );

                if ( $depth > 2 ) {
                    $parent = implode( ' / ', array_slice( $path_elements, 0, -1 ) );

                    $visits_with_utm_tags[ $parent ][ $index ] =
                        $visits_with_utm_tags[ $parent ][ $index ] + $visits_with_utm_tags[ $k ][ $index ];
                }
            }

            $the_visits_with_utm_tags = array_replace_recursive( $visits_with_utm_tags, $the_visits_with_utm_tags );
        }

        // Add zero values for missing dates, for UTM tags.

        if ( self::DEFAULT_DATE_RANGE !== $date_range ) {
            foreach ( $the_visits_with_utm_tags as &$a_list ) {
                foreach ( $dates as $date ) {
                    if ( ! isset( $a_list[ $date ] ) ) {
                        $a_list[ $date ] = 0;
                    }
                }
            }
        }

        // Get.

        $the_list = array_replace_recursive( $the_visits_with_utm_tags, $visits_list );
        ksort( $the_list );

        // Do not return empty values (where all dates are zero).

        $the_list = array_filter(
            $the_list,
            function( $v, $k ) {
                return ( ! is_int( $k ) ) || // Do not filter-out UTM-stuff.
                    ( 0 !== array_sum( (array) $v ) );
            },
            ARRAY_FILTER_USE_BOTH
        );

        return self::get_table_content( $the_list );
    }

    /**
     * Get HTML
     *
     * @param bool|array $source     Sources.
     * @param string     $date_range Date range.
     * @param bool       $get_utm_tags Get UTM tags instead of referers.
     *
     * @return string
     */
    public static function get_html( $source = false, $date_range = self::DEFAULT_DATE_RANGE, $get_utm_tags = true ) {
        ob_start();
        ?>
        <div class="first-party-analytics-column first-party-analytics-table-options-list">
            <div class="first-party-analytics-column first-party-analytics-setting-entities-block mt-m11">
                <?php Sources::display_sources_list( $source, true, $get_utm_tags, $date_range, true ); ?>
            </div>
        </div>
        <div class="first-party-analytics-column first-party-analytics-table-content-container">
            <?php echo self::print_table( $source, $date_range, $get_utm_tags );  ?>
        </div>
        <?php
        return ob_get_clean();
    }

    /**
     * Returns the table content.
     *
     * @param array|bool $visits_per_source Visits per source..
     */
    public static function get_table_content( $visits_per_source ) {
        ob_start();
        if ( $visits_per_source ) :
            $top_values = $visits_per_source[0];

            ?>
            <div class="first-party-analytics-table-content mt-50">
                <table>
                    <thead>
                    <tr>
                        <?php
                        foreach ( $visits_per_source[0] as $date => $visits_count ) :
                            ?>
                            <th><?php echo esc_attr( $date ); ?></th>
                            <?php
                        endforeach;
                        ?>
                    </tr>
                    </thead>
                    <tbody>
                    <?php
                    foreach ( $visits_per_source as $i => $source ) :
                        ?>
                        <tr>
                            <?php
                            foreach ( $source as $ii => $visits_count ) :
                                ?>
                                <td>
                                    <?php echo esc_attr( $visits_count ); ?>
                                    <?php // Show percents.
                                    if ( 0 !== $i && 0 !== $visits_count  ) {
                                        echo esc_attr( '(' . round( $visits_count / $top_values[ $ii ] * 100, 2 ) . '%)' );
                                    }
                                    ?>
                                </td>
                                <?php
                            endforeach;
                            ?>
                        </tr>
                        <?php
                    endforeach;
                    ?>
                    </tbody>
                </table>
            </div>
            <?php
        else :
            ?>
            <p class="no-data-here-yet-block">
                <?php esc_attr_e( 'No data yet.', 'first-party-analytics' ); ?>
            </p>
            <?php
        endif;
        return ob_get_clean();
    }
}
