<?php
/**
 * NoScript tracker standalone script.
 *
 * @package First-Party-Analytics
 */

/**
 * Constants for expressing human-readable intervals
 * in their respective number of seconds.
 *
 * Please note that these values are approximate and are provided for convenience.
 * For example, MONTH_IN_SECONDS wrongly assumes every month has 30 days and
 * YEAR_IN_SECONDS does not take leap years into account.
 *
 * If you need more accuracy please consider using the DateTime class (https://www.php.net/manual/en/class.datetime.php).
 */
define( 'MINUTE_IN_SECONDS', 60 );
define( 'HOUR_IN_SECONDS', 60 * MINUTE_IN_SECONDS );
define( 'DAY_IN_SECONDS', 24 * HOUR_IN_SECONDS );
define( 'WEEK_IN_SECONDS', 7 * DAY_IN_SECONDS );
define( 'MONTH_IN_SECONDS', 30 * DAY_IN_SECONDS );
define( 'YEAR_IN_SECONDS', 365 * DAY_IN_SECONDS );


/**
 * Removes trailing forward slashes and backslashes if they exist.
 *
 * The primary use of this is for paths and thus should be used for paths. It is
 * not restricted to paths and offers no specific path support.
 *
 * @param string $string What to remove the trailing slashes from.
 * @return string String without the trailing slashes.
 */
function untrailingslashit( $string ) {
    return rtrim( $string, '/\\' );
}

/**
 * Appends a trailing slash.
 *
 * Will remove trailing forward and backslashes if it exists already before adding
 * a trailing forward slash. This prevents double slashing a string or path.
 *
 * The primary use of this is for paths and thus should be used for paths. It is
 * not restricted to paths and offers no specific path support.
 *
 * @param string $string What to add the trailing slash to.
 * @return string String with trailing slash added.
 */
function trailingslashit( $string ) {
    return untrailingslashit( $string ) . '/';
}

/**
 * Maps a function to all non-iterable elements of an array or an object.
 *
 * This is similar to `array_walk_recursive()` but acts upon objects too.
 *
 * @since 4.4.0
 *
 * @param mixed    $value    The array, object, or scalar.
 * @param callable $callback The function to map onto $value.
 * @return mixed The value with the callback applied to all non-arrays and non-objects inside it.
 */
function map_deep( $value, $callback ) {
    if ( is_array( $value ) ) {
        foreach ( $value as $index => $item ) {
            $value[ $index ] = map_deep( $item, $callback );
        }
    } elseif ( is_object( $value ) ) {
        $object_vars = get_object_vars( $value );
        foreach ( $object_vars as $property_name => $property_value ) {
            $value->$property_name = map_deep( $property_value, $callback );
        }
    } else {
        $value = call_user_func( $callback, $value );
    }

    return $value;
}

/**
 * Callback function for `stripslashes_deep()` which strips slashes from strings.
 *
 * @since 4.4.0
 *
 * @param mixed $value The array or string to be stripped.
 * @return mixed The stripped value.
 */
function stripslashes_from_strings_only( $value ) {
    return is_string( $value ) ? stripslashes( $value ) : $value;
}

/**
 * Navigates through an array, object, or scalar, and removes slashes from the values.
 *
 * @since 2.0.0
 *
 * @param mixed $value The value to be stripped.
 * @return mixed Stripped value.
 */
function stripslashes_deep( $value ) {
    return map_deep( $value, 'stripslashes_from_strings_only' );
}

/**
 * Removes slashes from a string or recursively removes slashes from strings within an array.
 *
 * This should be used to remove slashes from data passed to core API that
 * expects data to be unslashed.
 *
 * @since 3.6.0
 *
 * @param string|array $value String or array of data to unslash.
 * @return string|array Unslashed `$value`.
 */
function wp_unslash( $value ) {
    return stripslashes_deep( $value );
}

/**
 * Sanitize text field.
 *
 * Different in logic from WP's sanitize_text_field() function: this one just removes tags from the string.
 *
 * @param string $string The string.
 *
 * @return string Sanitized string.
 */
function sanitize_text_field( $string ) {
    return strip_tags( $string ); 
}

/**
 * A wrapper for PHP's parse_url() function that handles consistency in the return
 * values across PHP versions.
 *
 * PHP 5.4.7 expanded parse_url()'s ability to handle non-absolute url's, including
 * schemeless and relative url's with :// in the path. This function works around
 * those limitations providing a standard output on PHP 5.2~5.4+.
 *
 * Secondly, across various PHP versions, schemeless URLs starting containing a ":"
 * in the query are being handled inconsistently. This function works around those
 * differences as well.
 *
 * @link https://www.php.net/manual/en/function.parse-url.php
 *
 * @param string $url       The URL to parse.
 *                          Defaults to -1 (= return all parts as an array).
 * @return mixed False on parse failure; Array of URL components on success;
 *               When a specific component has been requested: null if the component
 *               doesn't exist in the given URL; a string or - in the case of
 *               PHP_URL_PORT - integer when it does. See parse_url()'s return values.
 */
function wp_parse_url( $url ) {
    $to_unset = array();
    $url      = (string) $url;

    if ( '//' === substr( $url, 0, 2 ) ) {
        $to_unset[] = 'scheme';
        $url        = 'placeholder:' . $url;
    } elseif ( '/' === substr( $url, 0, 1 ) ) {
        $to_unset[] = 'scheme';
        $to_unset[] = 'host';
        $url        = 'placeholder://placeholder' . $url;
    }

    $parts = parse_url( $url );

    if ( false === $parts ) {
        // Parsing failure.
        return $parts;
    }

    // Remove the placeholder values.
    foreach ( $to_unset as $key ) {
        unset( $parts[ $key ] );
    }

    return $parts;
}

/**
 * The name of the type of analytics.
 *
 * @var string
 */
const NAME = 'server_side_noscript_analytics';

/**
 * Fields of this type of analytics.
 *
 * @important should be at sync with noscript-tracker.php.
 */
const FIELDS = [
    'time',
    'id',
    'ip',
    'post_id',
    'url',
    'requestMethod',
    'userAgent',
    'referer',
    'decimate',
    'postType',
    'searchText',
    'country',
    'dnt',
    'cookies',
    'revisionId',
];


/**
 * The name of the cookie.
 */
const COOKIE_NAME = 'first_party_analytics_uid';

/**
 * Identifier cookies length.
 */
const COOKIE_LENGTH = 20;

/**
 * Identifier cookies expiration, in seconds.
 */
const COOKIE_EXPIRATION = YEAR_IN_SECONDS;


/**
 * Sends "no-cache" headers.
 */
function send_no_cache_headers() {
    header( 'Cache-Control: no-cache, no-store, must-revalidate' ); // HTTP 1.1.
    header( 'Pragma: no-cache' ); // HTTP 1.0.
    header( 'Expires: 0' ); // Proxies.
}

/**
 * Sends the image.
 */
function send_image() {
    header( 'Content-Type: image/png' );
    readfile( realpath( __DIR__ . '/../../media/tracking-pixel-image.png' ) ); 
    flush();
}

/**
 * Generates cookie value.
 *
 * @return string Cookie value.
 */
function generate_cookie_value() {
    $output = rand( 1, 9 ); 

    for ( $i = 1; $i < COOKIE_LENGTH; $i++ ) {
        $output .= rand( 0, 9 ); 
    }

    return $output;
}

/**
 * Returns the visitor tracking cookie value.
 *
 * And if no cookie, and then current request is for noscript tracking image, then sets it.
 * Otherwise, it's expected that the image is set by the client with JS.
 *
 * @return string Cookie value or empty string if cookie is not set.
 */
function get_cookie() {
    $cookie_value = isset( $_COOKIE[ COOKIE_NAME ] ) ? $_COOKIE[ COOKIE_NAME ] : generate_cookie_value();

    setcookie( COOKIE_NAME, $cookie_value, time() + COOKIE_EXPIRATION, '/' ); // Set the tracking cookie or update its expiration.
    return $cookie_value;
}

/**
 * Returns the visitor IP.
 *
 * Supports CloudFlare.
 *
 * @return string
 */
function get_visitor_ip() {
    if ( isset( $_SERVER['HTTP_CF_CONNECTING_IP'] ) ) { // CloudFlare.
        return sanitize_text_field( wp_unslash( $_SERVER['HTTP_CF_CONNECTING_IP'] ) ); 
    } elseif ( isset( $_SERVER['HTTP_X_REAL_IP'] ) ) {
        return sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_REAL_IP'] ) ); 
    } elseif ( isset( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) { // Forwarded.
        // Proxy servers can send through this header like this: X-Forwarded-For: client1, proxy1, proxy2
        // Make sure we always only send through the first IP in the list which should always be the client IP.
        return trim( current( preg_split( '/,/', sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) ) ) ); 
    } elseif ( isset( $_SERVER['REMOTE_ADDR'] ) ) {
        return sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ); 
    } else {
        return '';
    }
}

/**
 * Fetches the country code from the request headers, if one is available.
 *
 * @return string The country code pulled from the headers, or empty string if one was not found.
 *
 * Inspired by @see \WC_Geolocation::get_country_code_from_headers
 */
function get_country_code_from_headers() {
    $country_code = '';

    $headers = [
        'MM_COUNTRY_CODE',
        'GEOIP_COUNTRY_CODE',
        'HTTP_CF_IPCOUNTRY',
        'HTTP_X_COUNTRY_CODE',
    ];

    foreach ( $headers as $header ) {
        if ( empty( $_SERVER[ $header ] ) ) {
            continue;
        }

        $country_code = strtoupper( sanitize_text_field( wp_unslash( $_SERVER[ $header ] ) ) ); 
        break;
    }

    return $country_code;
}

/**
 * Get filepath for raw analytics data file.
 *
 * @return string Raw analytics data filepath.
 */
function get_filepath() {
    $analytics_dir = trailingslashit( sys_get_temp_dir() );
    $real_filename = md5( NAME );
    return $analytics_dir . $real_filename;
}

/**
 * Adds specified content to the raw analytics file.
 *
 * @param string $content Content to write.
 */
function write_to_file( $content ) {
    file_put_contents( get_filepath(), $content . PHP_EOL, FILE_APPEND | LOCK_EX ); 
}

/**
 * Writes fields to the file.
 *
 * Checks that all fields are in place.
 *
 * @param mixed ...$fields The list of fields with their values.
 *
 * @throws Exception Exception.
 */
function write_fields( ...$fields ) {
    $how_many_fields_passed   = count( $fields );
    $how_many_fields_expected = count( FIELDS );
    if ( $how_many_fields_passed !== $how_many_fields_expected ) {
        throw new Exception( "Passed $how_many_fields_passed fields but expected $how_many_fields_expected." );
    }
    $fields = array_map(
        function( $field ) {
            /**
             * Replace tabs to spaces.
             *
             * @param $string
             *
             * @return string
             */
            $un_tab = function( $string ) {
                return preg_replace( '/\t/', '', $string );
            };

            if ( is_array( $field ) ) {
                // If the field is an array, then json_encode it.
                return json_encode( array_map( $un_tab, $field ) ); 
            } else {
                return $un_tab( $field );
            }
        },
        $fields
    );
    write_to_file( implode( "\t", $fields ) );
}

/**
 * Standardize the URL (if it has query part, then remove trailing slash from the path, otherwise add it).
 *
 * @param string $url URL to standardize.
 *
 * @return string
 */
function standardize_url( $url ) {
    $url_parts = wp_parse_url( $url );
    return $url_parts['scheme'] . '://' . $url_parts['host'] .
        ( array_key_exists( 'port', $url_parts ) ? ':' . $url_parts['port'] : '' ) .
        ( array_key_exists( 'path', $url_parts ) ?
            ( array_key_exists( 'query', $url_parts ) ? untrailingslashit( $url_parts['path'] ) : trailingslashit( $url_parts['path'] ) )
            : '' ) .
        ( array_key_exists( 'query', $url_parts ) ? '?' . $url_parts['query'] : '' );
}

/**
 * Check if "do not track" header is set.
 *
 * @return bool
 */
function is_dnt_set() {
    return isset( $_SERVER['HTTP_DNT'] ) && '1' === sanitize_text_field( wp_unslash( $_SERVER['HTTP_DNT'] ) );
}

/**
 * Inits server-side analytics.
 */
function record() {
    $time           = gmdate( 'c' );
    $id             = get_cookie();
    $ip             = get_visitor_ip();
    $page_url       = standardize_url( sanitize_text_field( wp_unslash( $_SERVER['HTTP_REFERER'] ) ) );
    $request_method = sanitize_text_field( wp_unslash( $_SERVER['REQUEST_METHOD'] ) );
    $ua             = isset( $_SERVER['HTTP_USER_AGENT'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_USER_AGENT'] ) ) : '';
    $referer        = '';
    $decimate       = 1;
    $post_type      = '';
    $search_text    = isset( $_GET['searchText'] ) ? sanitize_text_field( wp_unslash( $_GET['searchText'] ) ) : ''; 
    $revision_id    = isset( $_GET['revisionId'] ) ? (int) sanitize_text_field( wp_unslash( $_GET['revisionId'] ) ) : 0; 

    write_fields(
        $time,
        $id,
        $ip,
        '', // Do not paste Post ID.
        $page_url,
        $request_method,
        $ua,
        $referer,
        $decimate,
        $post_type,
        $search_text,
        get_country_code_from_headers(),
        is_dnt_set(),
        $_COOKIE,
        $revision_id
    );
}

/**
 * Act decisively.
 */
function act() {
    send_no_cache_headers();
    send_image();
    record();
}
act();
