<?php
/**
 * A class for handling links click tracking ping requests.
 *
 * Handles the data from <a ping=""> attributes on their click.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\Tracking;

use Exception;
use First_Party_Analytics\First_Party_Analytics;
use First_Party_Analytics\GeoLocation;
use First_Party_Analytics\Requests_Exclusion;
use First_Party_Analytics\Tracking_Cookies;
use First_Party_Analytics\Databases\Link_Visits as Link_Visits_DB;
use First_Party_Analytics\Utils;
use First_Party_Analytics\WooCommerce\Pages_Tracking_Data;
use First_Party_Analytics_Premium\First_Party_Analytics_Premium;
use First_Party_Analytics_Premium\Revision_Specific_Data;
use Pdp\SyntaxError;

/**
 * Class Link_Visits.
 */
final class Link_Visits {

    /**
     * How max time between the requests to dedup.
     *
     * Based on:
     *
     * 1. User ID.
     * 2. Source.
     * 3. Destination.
     */
    const DEDUP_MAX_TIME = 60;

    /**
     * Returns request with the specified fields.
     *
     * @param string $source    The page from which the link was clicked.
     * @param string $href      The href of the clicked link.
     * @param int    $timestamp Timestamp of the request.
     *
     * @return array Request.
     */
    private static function get_request( $source, $href, $timestamp ) {
        return [
            'id'           => Tracking_Cookies::get_cookie(),
            'ip'           => GeoLocation::get_visitor_ip(),
            'cookies'      => $_COOKIE,
            'source'       => $source,
            'href'         => $href,
            'time'         => $timestamp,
            'external_dst' => self::is_dst_external( $href ),
            'revisionId'   => First_Party_Analytics::$premium_enabled ? \First_Party_Analytics_Premium\Revisions\Revisions::get_current_revision_id() : 0,
        ];
    }

    /**
     * Is destination external.
     *
     * @param string $href The href of the clicked link.
     *
     * @return bool Whether destination (href) is external.
     */
    private static function is_dst_external( $href ) {
        $href_host = wp_parse_url( $href, PHP_URL_HOST );
        if ( ! $href_host ) {
            return false; // Relative link, without a host.
        } else {
            $site_host = wp_parse_url( home_url(), PHP_URL_HOST );
            return $site_host !== $href_host;
        }
    }

    /**
     * Determines whether the request should be deduped.
     *
     * Used when the request was duplicated on sending from JS and with "PING" attribute.
     *
     * @param string $visitor_id  Visitor ID.
     * @param string $time        Time of the visit.
     * @param string $source      The page URL from which the visit occurred.
     * @param string $href        The href of the visited URL.
     *
     * @throws Exception Exception.
     */
    private static function should_this_request_be_deduped( $visitor_id, $time, $source, $href ) {
        global $wpdb;

        $table_name = Link_Visits_DB::get_table_name();

        $query = $wpdb->prepare( 
            "
            SELECT `time` FROM $table_name
            WHERE
                visitor_id = %s AND
                `source` = %s AND
                `href` = %s
            ORDER BY `time` DESC
            LIMIT 1;
        ",
            $visitor_id,
            $source,
            $href
        );

        $result = $wpdb->get_var( $query ); 

        if (
            ! $result ||
            // If the difference between the current visit and the last similar visit is greater than dedup_max_time.
            // Then this request is not considered duplicate.
            strtotime( $time ) - strtotime( $result ) > self::DEDUP_MAX_TIME
        ) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * Add one server-side analytics entity.
     *
     * @param string $source    The page from which the link was clicked.
     * @param string $href      The href of the clicked link.
     * @param int    $timestamp Timestamp of the request.
     * @param int    $post_id   ID of the post.
     *
     * @throws Exception Exception.
     */
    public static function handle( $source, $href, $timestamp, $post_id ) {
        if (
            $href &&
            '#' !== $href // Do not record anchors.
        ) {
            $requests = Requests_Exclusion::exclude( [ self::get_request( $source, $href, $timestamp ) ] );
            if ( $requests ) {
                $request = $requests[0];
                $time    = gmdate( 'Y-m-d H:i:s', $request['time'] );
                if ( ! self::should_this_request_be_deduped( $request['id'], $time, $request['source'], $request['href'] ) ) {
                    $href_without_proto = preg_replace( '@^http(s)?:\/\/(www\.)?@', '', $request['href'] ); // Delete protocol and www.
                    $host               = untrailingslashit( explode( '/', $href_without_proto )[0] );

                    try {
                        $main_domain = Utils::get_domain_data( $host )['second_level_domain'];
                    } catch ( SyntaxError $e ) { // When the domain can't be parsed. For example, it's a local domain (127.0.0.1).
                        $main_domain = '';
                    }

                    $url_params = [];
                    $url_part   = explode( '?', $href );
                    if ( 2 === count( $url_part ) ) {
                        parse_str( $url_part[1], $url_params );
                        $url_params_str = wp_json_encode( $url_params );
                    } else {
                        $url_params_str = '';
                    }

                    // Write to link-visits DB.

                    Link_Visits_DB::write(
                        $request['id'],
                        $post_id,
                        $time,
                        $request['source'],
                        $request['href'],
                        $host,
                        $main_domain,
                        $url_params_str,
                        $request['external_dst'],
                        $request['revisionId']
                    );

                    /**
                     * Add the max value of the rules that apply to the product.
                     *
                     * Find a rule with the max value that matches the clicked link, and append it to the conversion value of the page.
                     */

                    $rules = First_Party_Analytics_Premium::$options->get( 'setting-outgoing-links-earnings' );

                    $max_matching_rule_value = 0;

                    foreach ( $rules as $row ) {
                        $domain    = $row['domain'];
                        $url_param = $row['urlParam'];
                        $param_val = $row['paramVal'];
                        $ppc       = (float) $row['ppc'];

                        if (
                            (
                                ! $domain ||
                                (
                                    ! str_ends_with( $domain, '.*' ) &&
                                    $host === $domain
                                ) ||
                                (
                                    str_ends_with( $domain, '.*' ) &&
                                    substr( $domain, 0, -2 ) === $main_domain
                                )
                            ) &&
                            (
                                ! $url_param ||
                                (
                                    $param_val &&
                                    array_key_exists( $url_param, $url_params ) &&
                                    $param_val === (string) $url_params[ $url_param ]
                                ) ||
                                (
                                    ! $param_val &&
                                    array_key_exists( $url_param, $url_params )
                                )
                            )
                        ) {
                            $max_matching_rule_value = max( $max_matching_rule_value, $ppc );
                        }
                    }

                    // Increase the post meta by the max matched value.

                    $currency = Utils::get_currency();

                    if ( $max_matching_rule_value ) {
                        foreach (
                            Revision_Specific_Data::get_passed_and_revision_specific_key( Pages_Tracking_Data::CONVERSIONS_VALUE_META_KEY )
                            as $meta_key
                        ) {
                            $meta_key = $meta_key . '-' . strtolower( $currency );

                            $prev_val = (int) get_post_meta( $post_id, $meta_key, true );

                            update_post_meta(
                                $post_id,
                                $meta_key,
                                $prev_val + $max_matching_rule_value
                            );
                        }
                    }
                }
            }
        }
    }
}
