<?php
/**
 * Server-side analytics data recording.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\Tracking;

use Exception;
use First_Party_Analytics\Fields_Mapping;
use First_Party_Analytics\First_Party_Analytics;
use First_Party_Analytics\GeoLocation;
use First_Party_Analytics\Tracking_Cookies;
use First_Party_Analytics\Utils;

/**
 * Class Server_Side_Analytics.
 */
final class Server_Side {

    /**
     * The name of the type of analytics.
     *
     * @var string
     */
    const NAME = 'server_side_analytics';

    /**
     * Fields of this type of analytics.
     */
    const FIELDS = [
        'time',
        'id',
        'ip',
        'post_id',
        'url',
        'requestMethod',
        'userAgent',
        'referer',
        'decimate',
        'postType',
        'searchText',
        'country',
        'dnt',
        'cookies',
        'revisionId',
    ];

    /**
     * Constructor.
     *
     * @throws Exception Exception.
     */
    public function __construct() {
        if (
            // Record only front-end requests.
            ! is_admin() &&
            // Don't record AJAXes.
            ! wp_doing_cron() &&
            ! wp_doing_ajax() &&
            ! isset( $_GET['wc-ajax'] ) && 
            // Don't record REST API requests.
            ! wp_is_json_request() &&
            // Misc.
            self::is_visit_extension_appropriate() &&
            // Decimate.
            Utils::decimate()
        ) {
            $this->record();
        }
    }

    /**
     * Returns request URL.
     *
     * @return string
     * @throws Exception Exception.
     */
    public static function get_current_request_url() {
        $url = ( ! empty( $_SERVER['HTTPS'] ) && 'off' !== sanitize_text_field( wp_unslash( $_SERVER['HTTPS'] ) ) ? 'https' : 'http' )
            . '://' . sanitize_text_field( wp_unslash( $_SERVER['HTTP_HOST'] ) ) . sanitize_text_field( wp_unslash( $_SERVER['REQUEST_URI'] ) );

        return Utils::standardize_url( $url );
    }

    /**
     * Filter out the visits for the inappropriate extensions (.ico .png) when image can't be found.
     *
     * @return bool
     *
     * @throws Exception Exception.
     */
    private static function is_visit_extension_appropriate() {
        $parsed_url = wp_parse_url( self::get_current_request_url() );

        if ( isset( $parsed_url['path'] ) ) {
            $pathinfo = pathinfo( $parsed_url['path'] );
            if (
                isset( $pathinfo['extension'] ) &&
                'php' !== $pathinfo['extension']
            ) {
                return false;
            }
        }

        return true;
    }

    /**
     * Returns search text.
     *
     * @return string
     *
     * @throws Exception Exception.
     */
    private static function get_search_text() {
        return is_search() ? html_entity_decode( get_search_query() ) : '';
    }

    /**
     * Check if "do not track" header is set.
     *
     * @return bool
     */
    private static function is_dnt_set() {
        return isset( $_SERVER['HTTP_DNT'] ) && '1' === sanitize_text_field( wp_unslash( $_SERVER['HTTP_DNT'] ) );
    }

    /**
     * Add one server-side analytics entity.
     *
     * @throws Exception Exception.
     */
    public static function record() {
        $time     = gmdate( 'c' );
        $id       = Tracking_Cookies::get_cookie();
        $ip       = GeoLocation::get_visitor_ip();
        $page_url = self::get_current_request_url();
        $post_id  = url_to_postid( $page_url );

        if ( 0 === $post_id ) {
            $post_id = Utils::get_currently_requested_post_id();
        }

        if ( ! $post_id ) {
            $post_id = '';
        }

        Fields_Mapping::write(
            self::NAME,
            $time,
            $id,
            $ip,
            $post_id,
            $page_url,
            sanitize_text_field( wp_unslash( $_SERVER['REQUEST_METHOD'] ) ), 
            isset( $_SERVER['HTTP_USER_AGENT'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_USER_AGENT'] ) ) : '', 
            isset( $_SERVER['HTTP_REFERER'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_REFERER'] ) ) : '', 
            First_Party_Analytics::$options->get( 'setting-decimate' ),
            get_post_type( $post_id ),
            self::get_search_text(),
            GeoLocation::get_country_code_from_headers(),
            self::is_dnt_set(),
            $_COOKIE,
            First_Party_Analytics::$premium_enabled ? \First_Party_Analytics_Premium\Revisions\Revisions::get_current_revision_id() : 0
        );
    }
}
