<?php
/**
 * A class for the WooCommerce products tracking data.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\WooCommerce;

use First_Party_Analytics\Databases\Page_Visits as DB;
use First_Party_Analytics_Premium\Revision_Specific_Data;

/**
 * Class Products_Tracking_Data.
 */
final class Products_Tracking_Data {

    /**
     * Viewed in catalog count meta tracker key.
     *
     * @var string
     */
    const VIEWED_IN_CATALOG_COUNT_TRACKER_META_KEY = 'first_party_analytics_viewed_in_catalog';

    /**
     * Added to cart meta tracker key.
     *
     * @var string
     */
    const ADDED_TO_CART_TRACKER_META_KEY = 'first_party_analytics_added_to_cart_counter';

    /**
     * Times product purchased meta key (prefixed by revision, if any).
     *
     * @var string
     */
    const TIMES_PURCHASED_META_KEY = 'first_party_analytics_product_purchased_counter';

    /**
     * Constructor.
     */
    public function __construct() {
        add_filter( 'woocommerce_post_class', [ $this, 'add_product_tracking_class' ], 10, 2 );

        add_action( 'woocommerce_add_to_cart', [ $this, 'added_to_cart_action_tracker' ], 10, 3 );
        add_filter(
            'woocommerce_order_status_completed',
            [
                $this,
                'order_complete_action_tracker',
            ],
            10,
            2
        );
    }

    /**
     * Adds product tracking class.
     *
     * @param array       $classes Product cart item key.
     * @param \WC_Product $product The ID of the product.
     */
    public function add_product_tracking_class( $classes, $product ) {
        $classes[] = 'first-party-analytics-a-product-' . $product->get_id();
        return $classes;
    }

    /**
     * Added to cart product action tracker.
     *
     * @param string $cart_item_key Product cart item key.
     * @param int    $product_id    The ID of the product.
     * @param int    $quantity      Quantity that was added to the cart.
     */
    public function added_to_cart_action_tracker( $cart_item_key, $product_id, $quantity ) {
        Revision_Specific_Data::post_meta_increment_counter( $product_id, self::ADDED_TO_CART_TRACKER_META_KEY, $quantity );
    }

    /**
     * Payment complete action tracker.
     *
     * Updates TIMES_PURCHASED_META_KEY value.
     *
     * @param int       $order_id Order ID.
     * @param \WC_Order $order    WC Order object.
     */
    public function order_complete_action_tracker( $order_id, $order ) {
        $products              = $order->get_items();
        $increment_per_product = [];

        foreach ( $products as $product ) {
            $product_id                           = $product->get_product_id();
            $increment_per_product[ $product_id ] =
                ( isset( $increment_per_product[ $product_id ] ) ? $increment_per_product[ $product_id ] : 0 ) + $product->get_quantity();
        }

        foreach ( $increment_per_product as $product_id => $quantity ) {
            Revision_Specific_Data::post_meta_increment_counter( $product_id, self::TIMES_PURCHASED_META_KEY, $quantity );
        }
    }

    /**
     * Returns product all-time purchased count (including when the plugin was not enabled).
     *
     * @param int $product_id ID of the product (post).
     *
     * @return int Visits count.
     */
    public static function get_all_time_purchased_count( $product_id ) {
        return (int) get_post_meta( $product_id, 'total_sales', true );
    }

    /**
     * Returns product purchased count.
     *
     * @param int $product_id ID of the product (post).
     *
     * @return int Visits count.
     */
    public static function get_purchased_count( $product_id ) {
        $revision = isset( $_REQUEST['fpaRevision'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ) : ''; 

        return (int) get_post_meta( $product_id, self::TIMES_PURCHASED_META_KEY . $revision, true );
    }

    /**
     * Returns the counter of how many times a product was added to cart.
     *
     * @param int $product_id ID of the product (post).
     *
     * @return int
     */
    public static function get_added_to_cart_count( $product_id ) {
        $revision = isset( $_REQUEST['fpaRevision'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ) : ''; 

        return (int) get_post_meta( $product_id, self::ADDED_TO_CART_TRACKER_META_KEY . $revision, true );
    }

    /**
     * Returns the counter of how many times a product was added to cart.
     *
     * @param int $product_id ID of the product (post).
     *
     * @return int
     */
    public static function get_viewed_in_catalog_count( $product_id ) {
        $revision = isset( $_REQUEST['fpaRevision'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ) : ''; 

        return (int) get_post_meta( $product_id, self::VIEWED_IN_CATALOG_COUNT_TRACKER_META_KEY . $revision, true );
    }

    /**
     * Returns the product all-time unique views counter.
     *
     * @param int $product_id ID of the product (post).
     *
     * @return int
     */
    public static function get_product_all_time_unique_views_count( $product_id ) {
        global $wpdb;

        $table_name = DB::get_table_name();

        $page_url = get_permalink( $product_id );

        if ( ! $page_url ) {
            return 0;
        }

        $revision_where_clause = '';
        if ( isset( $_REQUEST['fpaRevision'] ) && '' !== $_REQUEST['fpaRevision'] ) { 
            $revision = sanitize_text_field( wp_unslash( $_REQUEST['fpaRevision'] ) ); 
            $revision_where_clause = $wpdb->prepare( 'AND revision_id = %d', $revision );
        }

        $result = $wpdb->get_var(
            $wpdb->prepare(
                "
                        SELECT SUM(unique_visits)
                        FROM $table_name
                        WHERE
                            post_type = 'product' AND
                            `a_url` = %s
                            $revision_where_clause;
                    ",
                $page_url
            )
        );

        return (int) $result;
    }

    /**
     * Get all product tracking data.
     *
     * @param int $product_id ID of the product (post).
     *
     * @return array
     */
    public static function get_all( $product_id ) {
        return [
            'viewedInCatalog'       => self::get_viewed_in_catalog_count( $product_id ),
            'productViewsCount'     => self::get_product_all_time_unique_views_count( $product_id ),
            'addedToCartCount'      => self::get_added_to_cart_count( $product_id ),
            'purchasedCount'        => self::get_purchased_count( $product_id ),
            'purchasedAllTimeCount' => self::get_all_time_purchased_count( $product_id ),
        ];
    }
}
