<?php
/**
 * A class for the WooCommerce orders data tracking.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\WooCommerce;

use First_Party_Analytics\First_Party_Analytics;
use First_Party_Analytics\Tracking_Cookies;
use First_Party_Analytics_Premium\Revision_Specific_Data;

/**
 * Class WC_Orders.
 */
final class WC_Order {

    /**
     * Order meta key that contains the ID of the user.
     */
    const UID_META_KEY = 'first-party-analytics-uid';

    /**
     * The time() of the last order customer-initiated status change (usually, a purchase (on-hold, processing)).
     */
    const LAST_USER_INITIATED_STATUS_CHANGE_TIME_META_KEY = 'first-party-analytics-order-last-user-initiated-status-change';

    /**
     * The list of pages that were visited before making a purchase.
     */
    const VISITED_PAGES_META_KEY = 'first-party-analytics-session-visited-pages';

    /**
     * Constructor.
     */
    public function __construct() {

        /**
         *  Map order on "completed" to the particular session.
         *
         * And so get the data like:
         *
         * - Conversion depth (of each page in the session that preceded the order).
         * - Conversion value (of each page in the session that preceded the order).
         */

        add_action( 'woocommerce_order_status_changed', [ $this, 'add_uid_to_order' ], 10, 4 );
        add_action( 'woocommerce_order_status_completed', [ $this, 'order_completed' ], 10, 2 );

        /**
         * Completed orders counter.
         */

        add_action( 'woocommerce_order_status_completed', [ $this, 'increment_completed_order_tracker' ] );
    }

    /**
     * Adds analytics visitor ID to the order.
     *
     * @param int       $order_id    Order ID.
     * @param string    $status_from Old status.
     * @param string    $status_to   New status.
     * @param \WC_Order $order       WC Order object.
     *
     * @noinspection PhpUnusedParameterInspection
     */
    public function add_uid_to_order( $order_id, $status_from, $status_to, $order ) {
        if ( ! is_admin() && ! $order->get_meta( self::UID_META_KEY ) ) { // When order's status is changed upon customer's actions.
            $order->update_meta_data( self::UID_META_KEY, Tracking_Cookies::get_cookie() );
            $order->update_meta_data( self::LAST_USER_INITIATED_STATUS_CHANGE_TIME_META_KEY, time() );

            $order->save();
        }
    }

    /**
     * When the order is completed.
     *
     * @param int       $order_id Order ID.
     * @param \WC_Order $order    WC Order object.
     *
     * @noinspection PhpUnusedParameterInspection
     */
    public function order_completed( $order_id, $order ) {
        $order_session_visited_pages = $order->get_meta( self::VISITED_PAGES_META_KEY );

        if ( $order_session_visited_pages ) {

            /**
             * Update average conversion depth per visited page (before the converted purchase).
             *
             * That is an array with 2 values, where first is the number of conversions for this page, and the second is the average.
             */

            $page_ids = [];

            $conversion_depth = 1;
            foreach ( array_reverse( array_column( $order_session_visited_pages, 'post_id' ) ) as $visited_page_before_order ) {
                if ( ! isset( $page_ids[ $visited_page_before_order ] ) ) {
                    $page_ids[ $visited_page_before_order ] = $conversion_depth;
                }
                $conversion_depth ++;
            }

            $page_ids = array_filter(
                $page_ids,
                function( $page ) {
                    return $page; // Page ID can be undefined (empty string).
                }
            );

            foreach ( $page_ids as $page_id => $conversion_depth ) {
                foreach (
                    Revision_Specific_Data::get_passed_and_revision_specific_key( Pages_Tracking_Data::CONVERSION_DEPTH_META_KEY )
                    as $meta_key
                ) {
                    $prev_data       = get_post_meta( $page_id, $meta_key, true );
                    $prev_val        = $prev_data ? (float) $prev_data[1] : 0;
                    $prev_orders_num = $prev_data ? (int) $prev_data[0] : 0;

                    $val_or_this_order = $conversion_depth;

                    update_post_meta(
                        $page_id,
                        $meta_key,
                        [
                            $prev_orders_num + 1,
                            round( ( $prev_val * $prev_orders_num + $val_or_this_order ) / ( $prev_orders_num + 1 ), 2 ),
                        ]
                    );
                }
            }

            /**
             * Update conversion value per visited page (before the converted purchase).
             */

            $order_currency = $order->get_currency();
            $order_total    = $order->get_total();

            foreach (
                Revision_Specific_Data::get_passed_and_revision_specific_key( Pages_Tracking_Data::CONVERSIONS_VALUE_META_KEY )
                as $meta_key
            ) {
                $meta_key = $meta_key . '-' . strtolower( $order_currency );

                foreach ( $page_ids as $page_id => $conversion_depth ) {
                    $prev_val = (int) get_post_meta( $page_id, $meta_key, true );

                    update_post_meta(
                        $page_id,
                        $meta_key,
                        $prev_val + $order_total
                    );
                }
            }
        }

        // Delete FPA metadata from the order, as it's not needed anymore.

        $order->delete_meta_data( self::UID_META_KEY );
        $order->delete_meta_data( self::LAST_USER_INITIATED_STATUS_CHANGE_TIME_META_KEY );
        $order->delete_meta_data( self::VISITED_PAGES_META_KEY );
        $order->save();
    }

    /**
     * Increment completed orders tracker count.
     */
    public function increment_completed_order_tracker() {
        $completed_orders_counts = (array) First_Party_Analytics::$options->get( 'completed-orders-count' );

        foreach ( Revision_Specific_Data::get_passed_and_revision_specific_key( '' ) as $k ) {
            $array_index = (int) $k;

            $previous_val = (int) ( isset( $completed_orders_counts[ $array_index ] ) ? $completed_orders_counts[ $array_index ] : 0 );

            $completed_orders_counts[ $array_index ] = $previous_val + 1;
        }

        First_Party_Analytics::$options->set( 'completed-orders-count', $completed_orders_counts );
    }
}
