<?php
/**
 * A class for WC taxonomies data retrieval.
 *
 * @package First-Party-Analytics
 */

namespace First_Party_Analytics\WooCommerce;

use First_Party_Analytics\Databases\Page_Visits;

/**
 * Class WC_Taxonomies_Data.
 */
final class WC_Taxonomies_Data {

    /**
     * Returns unique views per tags.
     *
     * @return array The list of tags, and views per them.
     */
    public static function get_unique_views_per_tags() {
        $products = Page_Visits::get_visits_for_all_pages( 'product' );

        $views = [];

        foreach ( $products as $product_id => $views_count ) {
            $product_tags = wc_get_product_terms( $product_id, 'product_tag', [ 'fields' => 'ids' ] );
            if ( $product_tags ) {
                foreach ( $product_tags as $product_tag ) {
                    $views[ $product_tag ] = $views_count + ( isset( $views[ $product_tag ] ) ? $views[ $product_tag ] : 0 );
                }
            }
        }

        return $views;
    }

    /**
     * Returns unique views per categories.
     *
     * @return array The list of categories, and views per them.
     */
    public static function get_unique_views_per_categories() {
        $products = Page_Visits::get_visits_for_all_pages( 'product' );

        $views = [];

        foreach ( $products as $product_id => $views_count ) {
            $product_cats = wc_get_product_terms( $product_id, 'product_cat', [ 'fields' => 'ids' ] );
            if ( $product_cats ) {
                foreach ( $product_cats as $product_cat ) {
                    $views[ $product_cat ] = $views_count + ( isset( $views[ $product_cat ] ) ? $views[ $product_cat ] : 0 );
                }
            }
        }

        return $views;
    }

    /**
     * Returns unique views per attribute terms.
     *
     * @param string $attribute_name Attribute to get the views for the terms of.
     *
     * @return array The list of attribute terms, and views per them.
     */
    public static function get_unique_views_per_attribute_terms( $attribute_name ) {
        $products = Page_Visits::get_visits_for_all_pages( 'product' );

        $views = [];

        foreach ( $products as $product_id => $views_count ) {
            $product_attribute_terms = wc_get_product_terms( $product_id, $attribute_name, [ 'fields' => 'ids' ] );
            if ( $product_attribute_terms ) {
                foreach ( $product_attribute_terms as $attribute_term ) {
                    $views[ $attribute_term ] = $views_count + ( isset( $views[ $attribute_term ] ) ? $views[ $attribute_term ] : 0 );
                }
            }
        }

        return $views;
    }

    /**
     * Returns unique views per attributes.
     *
     * @return array The list of attributes, and views per them.
     */
    public static function get_unique_views_per_attributes() {
        $products = Page_Visits::get_visits_for_all_pages( 'product' );

        $views = [];

        foreach ( $products as $product_id => $views_count ) {
            $product_attributes = get_post_meta( $product_id, '_product_attributes');
            if ( $product_attributes ) {
                foreach ( array_keys( $product_attributes[0] ) as $product_attribute ) {
                    if ( str_starts_with( $product_attribute, 'pa_' ) ) {
                        $views[ $product_attribute ] = $views_count + ( isset( $views[ $product_attribute ] ) ? $views[ $product_attribute ] : 0 );
                    }
                }
            }
        }

        return $views;
    }
}
